%* lsaf_copyprocessflow                                                                                  *;
%*                                                                                                       *;
%* Copies a process flow. The new process flow is created in the same context as the source process      *;
%* flow.                                                                                                 *;
%*                                                                                                       *;
%* Syntax:                                                                                               *;
%* %LSAF_COPYPROCESSFLOW(LSAF_PATH=lsaf-path, LSAF_PROCESSFLOW=lsaf-processflow, LSAF_NAME=lsaf-name,    *;
%*    <, LSAF_USESOURCEOWNER=lsaf-usesourceowner, LSAF_INCLUDEATTACHMENTS=lsaf-includeattachments,       *;
%*       LSAF_INCLUDEPROCESSDATA=lsaf-includeprocessdata,                                                *; 
%*       LSAF_ACTIVATENEWPROCESSFLOW=lsaf-activatenewprocessflow >);                                     *;
%*                                                                                                       *;
%* @macvar _lsafRC_  The return code.                                                                    *;
%* @macvar _lsafMsg_ The return message.                                                                 *;
%*                                                                                                       *;
%* @param lsaf_path - Required - The case-sensitive path of the context in which the source process flow *;
%*         is located.                                                                                   *;
%* @param lsaf_processflow - Required - The case-sensitive name of the source process flow. This can be  *;
%*         retrieved by calling the lsaf_getprocessflows macro.                                          *;
%* @param lsaf_name - Required - The name of the new process flow.                                       *;
%* @param lsaf_usesourceowner - Optional - Indicates whether the owner of the new process flow is the    *;
%*         same as the source process flow.                                                              *;
%*         Valid values: 0 (Default): No, 1: Yes.                                                        *;
%* @param lsaf_includeattachments - Optional - Indicates whether to copy the attachments of the source   *;
%*         to the new process flow.                                                                      *;
%*         Valid values: 0 (Default): No, 1: Yes.                                                        *;
%* @param lsaf_includeprocessdata - Optional - Indicates whether to copy the values of the process data  *;
%*         from the source to the new process flow.                                                      *;
%*         Valid values: 0 (Default): No, 1: Yes.                                                        *;
%* @param lsaf_activatenewprocessflow - Optional - Indicates whether to activate the new process flow.   *;
%*         Valid values: 0 (Default): No, 1: Yes.                                                        *;
%*                                                                                                       *;
%* @macrotype LSAF                                                                                       *;
%* @since 2.4                                                                                            *;
%* @exposure external                                                                                    *;
%*                                                                                                       *;
%* @history 2020-10-09  Initial coding                                                                   *;
%*                                                                                                       *;

%macro lsaf_copyprocessflow(
  lsaf_path=,
  lsaf_processflow=,
  lsaf_name=,
  lsaf_useSourceOwner=0,
  lsaf_includeAttachments=0,
  lsaf_includeProcessData=0,
  lsaf_activateNewProcessFlow=0
  ) /des='Copy the process flow.';

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafRC_
  ;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data _null_;
    attrib returnCode      length=8
           message         length=$200
           logMsgPrefix    length=$70
           logMsgPrimary   length=$70
           logMsg          length=$300
           processFlowId   length=$64
    ;

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/lsaf/macro/workflow/processflow/ProcessFlowService");
    declare javaobj results("com/sas/lsaf/macro/results/Result");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticStringMethod("copyProcessFlow", "%bquote(&lsaf_path)", "%bquote(&lsaf_processflow)", "%bquote(&lsaf_name)", 
           "%bquote(&lsaf_useSourceOwner)", "%bquote(&lsaf_includeAttachments)", "%bquote(&lsaf_includeProcessData)", 
           "%bquote(&lsaf_activateNewProcessFlow)", results, processFlowId);

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
    if (returnCode ne 0) then
    do;
      logMsgPrimary="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Cannot copy process flow: &lsaf_processflow..";
    end;
    else
    do;
      logMsgPrimary=logMsgPrefix;
      logMsg=" The process flow: &lsaf_processflow was copied as &lsaf_name";
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrimary " " logMsg;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    put logMsgPrefix " _lsafMsg_= " message;
    put logMsgPrefix " _lsafRC_= " returnCode;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);

  run;

%mend lsaf_copyprocessflow;
