%* lsaf_createfile                                                                                 *;
%*                                                                                                 *;
%* Uploads a file from the local machine to the repository.                                        *;
%*                                                                                                 *;
%* All parent folders that do not exist are created.                                               *;
%*                                                                                                 *;
%* NOTE: This macro is supported only from a PC SAS session. Using this macro within an active     *;
%* SAS Life Science Analytics Framework application session results in a failure.                  *;
%*                                                                                                 *;
%* Syntax:                                                                                         *;
%* %LSAF_CREATEFILE(LOCAL_PATH=local-path, LSAF_PATH=lsaf-path                                     *;
%*        <, LSAF_ENABLEVERSIONING=lsaf-enable-versioning, LSAF_VERSION=lsaf-version,              *; 
%*        LSAF_COMMENT=lsaf_comment>);                                                             *;
%*                                                                                                 *;
%* @macvar _lsafRC_  The return code.                                                              *;
%* @macvar _lsafMsg_ The return message.                                                           *;
%*                                                                                                 *;
%* @param local_path - Required - The absolute path and name of the file on the local computer.    *;
%* @param lsaf_path - Required - The path and name of the file to create. Files can only be        *;
%*            created in SAS Life Science Analytics Framework areas that support folder and file   *;
%*            creation.                                                                            *;
%* @param lsaf_enableversioning - Optional - Indicates whether the file to create is versioned.    *;
%*            Valid values:                                                                        *;
%*            - 0 (Default): Create a non-versioned file.                                          *; 
%*            - 1: Create a versioned file. The user must have the Enable Versioning privilege     *;
%*              in the context in which to create the file or this operation fails.                *;
%* @param lsaf_version - Conditional - The version number to assign to the file. The value must    *;
%*            be in the valid version label format, which includes a decimal point. This option    *;
%*            is ignored when creating an unversioned file.                                        *;
%* @param lsaf_comment - Conditional - The comment for the versioned file. This option is          *;
%*            ignored when creating an unversioned file.                                           *;
%*                                                                                                 *;
%* @macrotype LSAF                                                                                 *;
%* @since 1.1                                                                                      *;
%* @exposure external                                                                              *;
%*                                                                                                 *;
%* @history 2013-08-30  updates for standardization and coding conventions                         *;
%* @history 2016-08-29  rebrand as LSAF                                                            *;
%* @history 2018-12-10  documentation update for LSAF 5.x behavior                                 *;
%*                                                                                                 *;

%macro lsaf_createfile(
  local_path=,
  lsaf_path=,
  lsaf_enableversioning=,
  lsaf_version=,
  lsaf_comment=
  ) /des='Upload a file from the local computer as a new file into SAS Life Science Analytics Framework.';

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafRC_
  ;

  %let _lsafRC_=%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data _null_;
    attrib returnCode      length=8;
    attrib message         length=$200;
    attrib logMsgPrefix    length=$70;
    attrib logMsg          length=$300;
    attrib logMsgPrimary   length=$70;

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/lsaf/macro/content/repository/RepositoryService");
    declare javaobj results("com/sas/lsaf/macro/results/Result");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticVoidMethod("createFile", "%bquote(&local_path)", "%bquote(&lsaf_path)", "%bquote(&lsaf_enableversioning)", "%bquote(&lsaf_version)", "%bquote(&lsaf_comment)", results);

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
    if (returnCode eq 0) then
    do;
      logMsgPrimary=logMsgPrefix;
      if ("&lsaf_enableversioning" eq "1") then
      do;
        logMsg=" Versioned file created as version &lsaf_version at &lsaf_path";
      end;
      else
      do;
        logMsg=" Unversioned file was created at &lsaf_path";
      end;
    end;
    else
    do;
      logMsgPrimary="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" The file was not created: &lsaf_path" ;
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrimary " " logMsg;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    put logMsgPrefix " _lsafMsg_= " message ;
    put logMsgPrefix " _lsafRC_= " returnCode ;
    put logMsgPrefix " local_path= &local_path" ;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);

  run;

%mend lsaf_createfile ;
