%* lsaf_createintervalschedule                                                                       *;
%*                                                                                                   *;
%* Creates a schedule that runs at the specified interval. Sets the SAS macro variable               *;
%* _lsafIntervalScheduleId_ to the identifier for the created schedule. If the schedule cannot be    *;
%* created, _lsafIntervalScheduleId_ is set to blank.                                                *;
%*                                                                                                   *;
%* You can specify a data set that represents the job parameters values to overwrite when the        *;
%* schedule runs. If no job parameters data set is specified, the default values for all job         *;
%* parameters are used.                                                                              *;
%*                                                                                                   *;
%* The output data set from the macro lsaf_getjobparameters or lsaf_getworkspacejobparameters        *;
%* contains all of the data that is relevant to the parameters for a specific job. You can use this  *;
%* data set as a source for the input parameters data set for this macro. Modify the data set with   *;
%* changes before calling this macro.                                                                *;
%* - Rename the defaultValue variable to "value" or add a variable with the name "value" to          *;
%*   indicate the values to use for the scheduled submission.                                        *;
%* - Default job parameter values are used for missing parameters (deleted rows).                    *;
%* - Indicating an empty string for a parameter value results in using a blank as the parameter      *;
%*   value for the current submission.                                                               *;
%* - Added parameters are ignored.                                                                   *;
%* - The parameters in the input data set are matched with the job parameters by name and type.      *;
%*   If no match is found for a parameter, its default value is used.                                *;
%* - Data set rows with duplicate values for parameter name cause an execution error.                *;
%* - DATE job parameter values must be in SAS datetime20. format (such as 02NOV2013:09:33:22), or    *;
%*   in SAS E8601DZ25. format (such as 2013-11-02T09:33:22-04:00). For SAS datetime20. format, an    *;
%*   offset of zero hours from GMT is used. To specify an offset from GMT, use the E8601DZ25.        *;
%*   format. Date values in formats other than the two described above result in an execution error. *;
%*                                                                                                   *;
%* Syntax:                                                                                           *;
%* %LSAF_CREATEINTERVALSCHEDULE(LSAF_NAME=lsaf-name <, LSAF_DESCRIPTION=lsaf-description >,          *;
%*                         LSAF_LOCATION=lsaf-location, LSAF_PATH=lsaf-path                          *;
%*                         <, LSAF_VERSION=lsaf-version, SAS_DSNAME=SAS-data-set>,                   *;
%*                         LSAF_STARTDATE=lsaf-startdate,                                            *;
%*                         LSAF_ENDTYPE=lsaf-endtype, LSAF_ENDVALUE=lsaf-endvalue,                   *;
%*                         LSAF_INTERVALTYPE=lsaf-intervaltype, LSAF_INTERVAL=lsaf-interval          *;
%*                         );                                                                        *;
%*                                                                                                   *;
%* @macvar _lsafRC_  The return code.                                                                *;
%* @macvar _lsafMsg_ The return message.                                                             *;
%*                                                                                                   *;
%* @param lsaf_name - Required - The name of the schedule to create.                                 *;
%* @param lsaf_description - Optional - The description of the schedule to create.                   *;
%* @param  lsaf_location - Required - The case-insensitive location of the job to schedule.          *;
%*     Valid values: WORKSPACE and REPOSITORY.                                                       *;
%* @param lsaf_path - Required - The path of the job to schedule.                                    *;
%* @param lsaf_version - Optional - The version of the repository job to schedule. If                *;
%*     omitted for a repository job, the latest version is scheduled to run. If the lsaf_location    *;
%*     is WORKSPACE, the value is ignored.                                                           *;
%* @param sas_dsname - Optional - The name of the SAS data set that contains the job parameter       *;
%*            information, specified as libref.dataset. If not specified, the default job            *;
%*            parameters values are used. The data set must contain at least the character           *;
%*            variables that are listed below. Additional variables are ignored.                     *;
%*            - name: The name of the parameter.                                                     *;
%*            - type: The parameter type. Valid values: CHARACTER, CHARACTER_MASKED, NUMERIC,        *;
%*              DATE, FILE, and FOLDER.                                                              *;
%*            - value: The value of the parameter to use in the current job submission.              *;
%*            - fileVersion: The file version to use, if the parameter is of type FILE. For          *;
%*              versioned files, a blank value results in the latest file version used. For          *;
%*              unversioned files, the value is ignored.                                             *;
%*            - includeSubFolders: Indicates whether the subfolders of a container are included      *;
%*              during job execution.                                                                *;
%* @param lsaf_startdate - Required - The date and time on which the schedule runs. The value must   *;
%*     be in SAS datetime20. format (such as 02NOV2013:09:33:22), or in SAS E8601DZ25. format        *;
%*     (such as 2013-11-02T09:33:22-04:00). For SAS datetime20. format, an offset of zero hours      *;
%*     from GMT is used. To specify an offset from GMT, use the E8601DZ25. format. Date values in    *;
%*     formats other than the two described above result in an execution error.                      *;
%* @param lsaf_endtype - Required - The case-insensitive value that specifies the method to end      *;
%*     the schedule. Valid values:                                                                   *;
%*        NONE - The schedule runs indefinitely or until the schedule is deleted.                    *;
%*        DATE - The schedule runs until a specified date and time.                                  *;
%*        OCCURRENCES - The schedule runs a specified number of times.                               *;
%* @param lsaf_endvalue - Conditional - If lsaf_endtype is DATE, this value must be a valid date     *;
%*        for the last schedule run. See lsaf_startdate for valid date formats.                      *;
%*        If lsaf_endtype is OCCURRENCES, this value must be a valid integer value for the number    *;
%*        of times to run the schedule. If lsaf_endtype is NONE, this value is ignored.              *;
%* @param lsaf_intervaltype - Required - The case-insensitive type of interval.                      *;
%*     Valid values: MINUTELY, HOURLY, and DAILY.                                                    *;
%* @param lsaf_interval - Required - The integer value for the interval to pass before each          *;
%*     schedule run.                                                                                 *;
%*                                                                                                   *;
%* @macrotype LSAF                                                                                   *;
%* @since 2.3                                                                                        *;
%* @exposure external                                                                                *;
%*                                                                                                   *;
%*                                                                                                   *;
%* @history 2019-10-01  initial code                                                                 *;
%* @history 2020-06-30  update service location                                                      *;
%*                                                                                                   *;

%macro lsaf_createintervalschedule(
  lsaf_intervaltype=,
  lsaf_interval=,
  lsaf_name=,
  lsaf_description=,
  lsaf_location=,
  lsaf_path=,
  lsaf_version=,
  sas_dsname=,
  lsaf_startdate=,
  lsaf_endtype=,
  lsaf_endvalue=
  ) /des='Creates a schedule that runs based on the specified interval.';

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafIntervalScheduleId_
          _lsafRC_
  ;


   %local logMsgPrefix
          logMsgPrimary
          macroName
          notesOption
          parametersProvided
          _tempRUNJOB_tempDS_
   ;

   %let _lsafIntervalScheduleId_=;
   %let _lsafRC_ =%str(-999);
   %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

   %let macroName=%sysfunc(lowcase(&SYSMACRONAME));
   %let logMsgPrimary=%str(ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro: );
   %let logMsgPrefix=%str(NOTE: &_LSAF_PRODUCT_NAME_ Macro: );
   %let parametersProvided=0;

   %*************************************************************;
   %* Turn off the notes generated for all the pre-processing.  *;
   %* Save original option setting so it can be reset properly. *;
   %*************************************************************;
   %let notesOption=%sysfunc(getoption(notes));

   %if ( %CMPRES(&sas_dsname) NE %str() ) %then
   %do;

      %***************************************************************************;
      %* Validate the parameters data set if supplied.                           *;
      %* sets global macro variable _lsafRC_=0 if validation successful           *;
      %***************************************************************************;
      %lsaf_validate_parameters_dataset(callingMacro=&macroName,
                                      sas_dsn=&sas_dsname);
      %if (&_lsafRC_ ne 0) %then
      %do;
         %GOTO PRINT_DSERROR_LOG_MSGS;
      %end;

      %let parametersProvided=1;
   %end;

   options &notesOption;

   %**********************************************;
   %* Process the records                        *;
   %**********************************************;
   data _null_;
      attrib returnCode      length=8
             message         length=$200
             scheduleId      length=$50
             logMsgPrefix    length=$70
             logMsgPrimary   length=$70
             logMsg          length=$300
      ;

      %if (&parametersProvided) %then
      %do;
         set &sas_dsname end=eof;
         if (_n_ = 1) then
         do;
      %end;

      %***********************************;
      %* Declare the java objects, once  *;
      %***********************************;
      declare javaobj params("com/sas/lsaf/macro/execution/job/SasParameterValues");
      declare javaobj srv("com/sas/lsaf/macro/schedule/ScheduleService");
      declare javaobj results("com/sas/lsaf/macro/results/Result");

      %if (&parametersProvided) %then
      %do;
         end;
         params.callVoidMethod("addValue", kstrip(name), kstrip(value), kstrip(type), kstrip(fileVersion), includeSubFolders);
         if (eof);
      %end;

      %***********************************;
      %* Submit the job                  *;
      %***********************************;
      srv.callStaticStringMethod("createIntervalSchedule", "%bquote(&lsaf_name)", "%bquote(&lsaf_description)", 
                                    "%bquote(&lsaf_location)", "%bquote(&lsaf_path)", "%bquote(&lsaf_version)", params, 
                                    "%bquote(&lsaf_startdate)", "%bquote(&lsaf_endtype)", "%bquote(&lsaf_endvalue)",
                                    "%bquote(&lsaf_intervaltype)", "%bquote(&lsaf_interval)", results, scheduleId);

      %***********************************;
      %* Retrieve the results            *;
      %***********************************;
      results.callIntMethod("getReturnCode", returnCode);
      results.callStringMethod("getReturnMessage", message);

      %***********************************;
      %* Process the results             *;
      %***********************************;
      logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
      if (returnCode ne 0) then
      do;
         logMsgPrimary="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
         logMsg=" Cannot create schedule &lsaf_name.";
      end;
      else
      do;
         logMsgPrimary=logMsgPrefix;
         logMsg=" Created schedule &lsaf_name.";
      end;

      %***********************************;
      %* Print messages in log           *;
      %***********************************;
      put;
      put logMsgPrimary " " logMsg;
      put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
      put logMsgPrefix " _lsafMsg_= " message ;
      put logMsgPrefix " _lsafRC_= " returnCode ;
      if "&sas_dsname" = "" 
        then put logMsgPrefix "No job parameters were supplied.";
        else put logMsgPrefix "sas_dsname = &sas_dsname";
      put logMsgPrefix "_lsafIntervalScheduleId_= " scheduleId;
      put;

      %***********************************;
      %* Set the macro variables         *;
      %***********************************;
      call symputx("_lsafRC_", returnCode);
      call symputx("_lsafMsg_", message);
      call symputx("_lsafIntervalScheduleId_", strip(scheduleId));

   run;

   %return;

   %PRINT_DSERROR_LOG_MSGS:
      options &notesOption;
      %put &logMsgPrimary  &macroName ;
      %put &logMsgPrefix _lsafRC_= &_lsafRC_ ;
      %put &logMsgPrefix _lsafMsg_= &_lsafMsg_ ;
      %put &logMsgPrefix Parameter data set = &sas_dsname;
      %put;

%mend lsaf_createintervalschedule;
