%* lsaf_deleterbitemsbyquery                                                                     *;
%*                                                                                               *;
%* Permanently deletes items from the recycle bin that match the specified criteria. The paths   *;
%* of the items that were permanently deleted are returned in a SAS data set.                    *;
%*                                                                                               *;
%* To delete items put in the recycle bin by other users, you must have the Permanently Delete   *;
%* Items privilege.                                                                              *;
%*                                                                                               *;
%* Syntax:                                                                                       *;
%* %LSAF_DELETERBITEMSBYQUERY(LSAF_LOCATION=lsaf-location <, LSAF_NAME=lsaf-name,                *;
%*         LSAF_DELETEDBY=lsaf-deletedby, LSAF_FROM=lsaf-from-date, LSAF_TO=lsaf-to-date,        *;
%*         LSAF_INCLUDESUBFOLDERS=lsaf-includeSubfolders, SAS_RESULT_DSNAME=SAS-data-set);       *;
%*                                                                                               *;
%* @macvar _lsafRC_  The return code.                                                            *;
%* @macvar _lsafMsg_ The return message.                                                         *;
%*                                                                                               *;
%* @param lsaf_location - Optional - The location of the container in the repository that is the *;
%*            parent of the deleted items. Items from child paths are also returned.             *;
%* @param lsaf_name - Optional - The name of the item to return by the search. If set to         *;
%*            * (such as lsaf_name=*) or is omitted, items of any name are returned.             *;
%*            Wildcards can be used. For example, lsaf_name=*.sas returns all of the items with  *;
%*            a .sas extension.                                                                  *;
%* @param lsaf_deletedby - Optional - The identifier of the user who deleted the item.           *;
%*            If omitted, set to blank, or * (such as lsaf_deletedby=*), all items that the      *;
%*            current user has the privilege to retrieve, regardless of who deleted the items,   *;
%*            are returned. If the current user lacks the privileges, only current user items    *;
%*            are returned.                                                                      *;
%* @param lsaf_from - Optional - The beginning of the date range for items that were deleted     *;
%*            within a specific time period. Values must be one of the formats described below.  *;
%*            Date values in formats other than those described below result in an execution     *;
%*            failure.                                                                           *;
%*            - SAS datetime20. format (such as 02NOV2013:09:33:22) - an offset of zero hours    *;
%*              from GMT is used.                                                                *;
%*            - SAS E8601DZ25. format (such as 2013-11-02T09:33:22-04:00).                       *;
%* @param lsaf_to - Optional - The end of the date range for items that were deleted within a    *;
%*            specific time period. Values must be one of the formats described below. Date      *;
%*            values in formats other than those described below result in an execution          *;
%*            failure.                                                                           *;
%*            - SAS datetime20. format (such as 02NOV2013:09:33:22) - an offset of zero hours    *;
%*              from GMT is used.                                                                *;
%*            - SAS E8601DZ25. format (such as 2013-11-02T09:33:22-04:00).                       *;
%* @param lsaf_includeSubfolders - Optional - Indicates whether the search includes containers   *;
%*            within the specified location. Valid values:                                       *;
%*            - 0: Exclude subfolders.                                                           *;
%*            - 1 (Default): Include subfolders.                                                 *;
%* @param sas_result_dsname - Optional - The name of the output SAS data set to contain the      *;
%*            deleted paths, specified as libref.dataset. The default is                         *;
%*            WORK.LSAFDELETERBITEMSBYQUERY. The data set contains a row for each deleted path   *;
%*            and columns with the following names. The data set is sorted by path.              *;
%*            - name: The name of the item.                                                      *;
%*            - path: The path of the item.                                                      *;
%*                                                                                               *;
%* @macrotype LSAF                                                                               *;
%* @since 2.2                                                                                    *;
%* @exposure external                                                                            *;
%*                                                                                               *;
%* @history 2019-08-29  original coding.                                                         *;
%* @history 2019-11-01  update to handle a blank SAS output file.                                *;
%* @history 2020-07-13  update for refactor of java service hierarchy.                           *;
%*                                                                                               *;

%macro lsaf_deleterbitemsbyquery(
  lsaf_location=,
  lsaf_name=,
  lsaf_deletedby=,
  lsaf_from=,
  lsaf_to=,
  lsaf_includesubfolders=,
  sas_result_dsname=work.lsafdeleterbitemsbyquery
  ) /des='Deletes the items from the recycle bin that match the given criteria.';

  %if "&sas_result_dsname" EQ "" %then %let sas_result_dsname=work.lsafdeleterbitemsbyquery;

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafRC_
          _lsaf_PrintMessage_
  ;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %local _lsafRowCount_
         notesOption
  ;

  %let _lsafRowCount_=0;

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data &sas_result_dsname (label="Created by &SYSMACRONAME" keep=path name sortPath);
    attrib returnCode         length=8
           message            length=$200
           logMsgPrefix       length=$70
           logMsgPrimary      length=$70
           logMsg             length=$300
           rowCount           length=8
           path               length=$2048    label="Item Path"
           name               length=$256     label="Item Name"
           sortpath           length=$2048    label="NEEDED FOR SORTING - will be dropped"
    ;
    
    rowCount=0;

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/lsaf/macro/content/repository/recyclebin/RecycleBinService") ;
    declare javaobj results("com/sas/lsaf/macro/results/SasPathResult");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticVoidMethod("deleteRecycleBinItemsByQuery", "%bquote(&lsaf_location)", "%bquote(&lsaf_name)", 
                             "%bquote(&lsaf_deletedby)", "%bquote(&lsaf_from)", "%bquote(&lsaf_to)", 
                             "%bquote(&lsaf_includesubfolders)",  results);

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);
    logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
    if (returnCode ne 0) then
    do;
      logMsgPrimary="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Could not permanently delete items from the recycle bin." ;
    end;
    else
    do;
      logMsgPrimary=logMsgPrefix;
      logMsg=" Returned the list of paths deleted from the recycle bin.";
      results.callIntMethod("getSize", rowCount);
      do row=0 to rowCount - 1;
        results.callStringMethod("getPath", row, path);
        sortPath = kstrip( kupcase( ktranslate(path,'1','_') ) );
        results.callStringMethod("getName", row, name);
        output;
      end;
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrimary " " logMsg;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    if (rowCount=0) 
      then put logMsgPrefix " No items were deleted from the recycle bin.";
      else put logMsgPrefix " " rowCount " records written to data set &sas_result_dsname";
    put logMsgPrefix " _lsafMsg_= " message ;
    put logMsgPrefix " _lsafRC_= " returnCode ;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);
    call symputx("_lsafRowCount_", rowCount);
  run;

  %***********************************;
  %* Sort the data set               *;
  %***********************************;
  %let notesOption=%sysfunc(getoption(notes));
  options nonotes;

  %if (&_lsafRowCount_ > 0) %then
  %do;
     proc sort data=&sas_result_dsname
               out=&sas_result_dsname (drop=sortpath);
         by sortpath;
     run;
  %end;
  %else
  %do;
     proc sql;
       alter table &sas_result_dsname
          drop sortpath;
     quit;
  %end;
  options &notesOption;


%mend lsaf_deleterbitemsbyquery;
