%* lsaf_deleterecyclebinitems                                                                      *;
%*                                                                                                 *;
%* Permanently deletes items from the SAS Life Science Analytics Framework recycle bin using a     *;
%* data set as input. Returns a SAS data set that contains the paths for the items that were       *;
%* permanently deleted from the SAS Life Science Analytics Framework recycle bin.                  *;
%*                                                                                                 *;
%* The items listed in the input data set are permanently removed from the recycle bin,            *;
%* provided that the user has the Permanently Delete Items role privilege at the organization      *;
%* context.  No error message is provided for items that cannot be permanently deleted.            *;
%*                                                                                                 *;
%* A path that is created and deleted multiple times from the repository appear multiple times     *;
%* in the recycle bin. Therefore, the items must be referenced by their unique identifier. The     *;
%* identifier can be retrieved using the lsaf_getrecyclebinitems macro.                            *;
%*                                                                                                 *;
%* The output data set contains one row for each location that is permanently deleted from         *;
%* the recycle bin. If multiple items with the same path are deleted, the path is listed only      *;
%* once. The locations for each item that are deleted from a container (such as a folder)          *;
%* will be listed separately in the output data set.                                               *;
%*                                                                                                 *;
%* Syntax:                                                                                         *;
%* %LSAF_DELETERECYCLEBINITEMS(SAS_DSNAME=SAS-data-set, SAS_RESULT_DSNAME=SAS-data-set);           *;
%*                                                                                                 *;
%* Example Code:                                                                                   *;
%* Delete everything a specific user deleted.                                                      *;
%*                                                                                                 *;
%* * start code example;                                                                           *;
%* %lsaf_getrecyclebinitems(sas_dsname=lsafGetRBItems);                                            *;
%*                                                                                                 *;
%* data deleterbitems;                                                                             *;
%*   set lsafGetRBItems;                                                                           *;
%*   if (deletedBy="myuserid");                                                                    *;
%* run;                                                                                            *;
%*                                                                                                 *;
%* %lsaf_deleterecyclebinitems(sas_dsname=deleterbitems, sas_result_dsname=deletedpaths);          *;
%* * end code example;                                                                             *;
%*                                                                                                 *;
%* @macvar _lsafRC_  The return code.                                                              *;
%* @macvar _lsafMsg_ The return message.                                                           *;
%*                                                                                                 *;
%* @param sas_dsname - Required - The name of a SAS data set that contains the identifiers for the *;
%*            items to delete from the recycle bin. The data set must contain at least the         *;
%*            variables listed below. Additional variables are ignored.                            *;
%*            - identifier: The unique identifier for the item.                                    *;
%* @param sas_result_dsname - Optional - The name of the output SAS data set to contain the        *;
%*            deleted paths, specified as libref.dataset. The default is                           *;
%*            WORK.LSAFDELETERECYCLEBINITEMS. The data set contains a row for each deleted path    *;
%*            and columns with the following names. The data set is sorted by path.                *;
%*            - name: The name of the item.                                                        *;
%*            - path: The path of the item.                                                        *;
%*                                                                                                 *;
%* @macrotype LSAF                                                                                 *;
%* @since 1.4                                                                                      *;
%* @exposure external                                                                              *;
%*                                                                                                 *;
%* @history 2013-03-27  initial code                                                               *;
%* @history 2016-08-26  rebrand as LSAF                                                            *;
%* @history 2019-02-20  update to 2.1 and rename from lsaf_deleterbitems to                        *;
%*                      lsaf_deleterecyclebinitems                                                 *;
%* @history 2019-11-01  update to handle a blank SAS output file.                                  *;
%* @history 2020-07-13  update for refactor of java service hierarchy.                             *;
%*                                                                                                 *;

%macro lsaf_deleterecyclebinitems(
  sas_dsname=,
  sas_result_dsname=work.lsafdeleterecyclebinitems
  ) /des='Deletes the specified items from the SAS Life Science Analytics Framework recycle bin.';

  %if "&sas_result_dsname" EQ "" %then %let sas_result_dsname=work.lsafdeleterecyclebinitems;

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafRC_
  ;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %local _lsafRowCount_
         logMsgPrefix
         logMsgPrimary
         macroName
         notesOption
  ;

  %*************************************************************;
  %* Turn off the notes generated for all the pre-processing.  *;
  %* Save original option setting so it can be reset properly. *;
  %*************************************************************;
  %let notesOption=%sysfunc(getoption(notes));
  options nonotes;

  %let _lsafRowCount_=0;

  %*******************************************************************;
  %* Validate the input data set                                     *;
  %* sets global macro variable _lsafRC_=0 if validation successful  *;
  %*******************************************************************;
  %let macroName=%sysfunc(lowcase(&SYSMACRONAME));
  %lsaf_validate_sasdataset(callingMacro=&macroName, sas_dsn=&sas_dsname, required_variables=%str(identifier));
  
  %if (&_lsafRC_ ne 0) %then
  %do;
    options &notesOption;
    %let logMsgPrimary=%str(ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro: );
    %let logMsgPrefix=%str(NOTE: &_LSAF_PRODUCT_NAME_ Macro: );
    %put &logMsgPrimary _lsafMsg_= &_lsafMsg_ ;
    %put &logMsgPrefix &_LSAF_PRODUCT_NAME_ Macro &macroName ;
    %put &logMsgPrefix Input data set = &sas_dsname;
    %put &logMsgPrefix _lsafRC_= &_lsafRC_ ;
    %put;
    %return;
  %end;

  %***************************************************************************;
  %* If data set is valid, process it.                                       *;
  %***************************************************************************;
  %put NOTE: &macroName: Processing data set &sas_dsname;

  option notes;

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data &sas_result_dsname (label="Created by &SYSMACRONAME" keep=path name sortPath);
    attrib returnCode         length=8
           message            length=$200
           logMsgPrefix       length=$70
           logMsgPrimary      length=$70
           logMsg             length=$300
           rowCount           length=8
           path               length=$2048    label="Item Path"
           name               length=$256     label="Item Name"
           sortpath           length=$2048    label="NEEDED FOR SORTING - will be dropped"
    ;

    set &sas_dsname end=eof;

    %***********************************;
    %* Declare the java objects, once  *;
    %***********************************;
    if (_n_ = 1) then
    do;
      declare javaobj srv("com/sas/lsaf/macro/content/repository/recyclebin/RecycleBinService") ;
      declare javaobj rbItems("com/sas/lsaf/macro/content/repository/recyclebin/SasRecycleBinItems");
      declare javaobj results("com/sas/lsaf/macro/results/SasPathResult");
    end;

    %*******************************************;
    %* Update the result with the identifiers  *;
    %*******************************************;

    rbItems.callVoidMethod("addItem", kstrip(identifier), kstrip(path));

    if (eof);

    rowCount=0;

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticVoidMethod("deleteItems", rbItems, results);

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);
    logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
    if (returnCode ne 0) then
    do;
      logMsgPrimary="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Could not permanently delete items from the recycle bin." ;
    end;
    else
    do;
      logMsgPrimary=logMsgPrefix;
      logMsg=" Returned the list of paths deleted from the recycle bin.";
      results.callIntMethod("getSize", rowCount);
      do row=0 to rowCount - 1;
        results.callStringMethod("getPath", row, path);
        sortPath = kstrip( kupcase( ktranslate(path,'1','_') ) );
        results.callStringMethod("getName", row, name);
        output;
      end;
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrimary " " logMsg;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    if (rowCount=0) 
      then put logMsgPrefix " No items were deleted from the recycle bin.";
      else put logMsgPrefix " " rowCount " records written to data set &sas_result_dsname";
    put logMsgPrefix " _lsafMsg_= " message ;
    put logMsgPrefix " _lsafRC_= " returnCode ;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);
    call symputx("_lsafRowCount_", rowCount);
  run;

  %***********************************;
  %* Sort the data set               *;
  %***********************************;
  %let notesOption=%sysfunc(getoption(notes));
  options nonotes;

  %if (&_lsafRowCount_ > 0) %then
  %do;
     proc sort data=&sas_result_dsname
               out=&sas_result_dsname (drop=sortpath);
         by sortpath;
     run;
  %end;
  %else
  %do;
     proc sql;
       alter table &sas_result_dsname
          drop sortpath;
     quit;
  %end;
  options &notesOption;

%mend lsaf_deleterecyclebinitems;
