%* lsaf_deleteversion                                                                      *;
%*                                                                                         *;
%* Permanently deletes the specified version of a repository item.                        *;
%* The macro will delete only 1 version of the file at a time.                             *;
%*                                                                                         *;
%* Syntax:                                                                                 *;
%* %LSAF_DELETEVERSION(LSAF_PATH=lsaf-path, LSAF_VERSION=lsaf-version);                    *;
%*                                                                                         *;
%* @macvar _lsafRC_  The return code.                                                      *;
%* @macvar _lsafMsg_ The return message.                                                   *;
%*                                                                                         *;
%* @param lsaf_path - Required - The case-sensitive path of the item.                      *;
%* @param lsaf_version - Required - The version number of the item to delete.              *;
%*                                                                                         *;
%* @macrotype LSAF                                                                         *;
%* @since 2.5                                                                              *;
%* @exposure external                                                                      *;
%*                                                                                         *;
%* @history 2021-02-09  initial coding                                                     *;
%*                                                                                         *;

%macro lsaf_deleteversion(
  lsaf_path=,
  lsaf_version=
  ) /des='Delete a version of an item from the repository.';

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafRC_
  ;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data _null_;
    attrib returnCode      length=8;
    attrib message         length=$200;
    attrib logMsgPrefix    length=$70;
    attrib logMsg          length=$300;
    attrib logMsgPrimary   length=$70;

    %***********************************;
    %* Declare the java items        *;
    %***********************************;
    declare javaobj srv("com/sas/lsaf/macro/content/repository/RepositoryService");
    declare javaobj results("com/sas/lsaf/macro/results/Result");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticVoidMethod("deleteVersion", "%bquote(&lsaf_path)", "%bquote(&lsaf_version)", results);

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
    logMsgPrimary=logMsgPrefix;
    logMsg=" Version &lsaf_version. of item &lsaf_path. was deleted";
    if (returnCode ne 0) then
    do;
      logMsgPrimary="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Cannot delete version &lsaf_version. of item &lsaf_path.." ;
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrimary " " logMsg;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    put logMsgPrefix " _lsafMsg_= " message ;
    put logMsgPrefix " _lsafRC_= " returnCode ;
    put logMsgPrefix " lsaf_path = &lsaf_path" ;
    put logMsgPrefix " lsaf_version = &lsaf_version" ;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);

  run;

%mend lsaf_deleteversion;
