%* lsaf_deployprocessdefinition                                                                     *;
%*                                                                                                  *;
%* Deploys a process flow definition using a BPMN file from the repository. The macro variable      *;
%*  _lsafProcessDefinitionId_ contains the process definition identifier that is set as part of     *;
%*  the deployment. The user must have the global privilege                                         *;
%*  PRIVILEGE_MANAGE_PROCESS_FLOW_DEFINITIONS.                                                      *;
%*                                                                                                  *;
%* Syntax:                                                                                          *;
%* %LSAF_DEPLOYPROCESSDEFINITION(LSAF_PATH=lsaf-path, LSAF_VERSION=lsaf-version,                    *;
%*            SAS_DSNAME=SAS-data-set, LSAF_COMMENT=lsaf-comment, LSAF_ACTIVATE=lsaf-activate,      *;
%*            LSAF_OVERWRITE=lsaf-overwrite);                                                       *;
%*                                                                                                  *;
%* @macvar _lsafRC_  The return code.                                                               *;
%* @macvar _lsafMsg_ The return message.                                                            *;
%* @macvar _lsafProcessDefinitionId_ - The identifier of the deployed process definition.           *;
%*                                                                                                  *;
%* @param lsaf_path - Required - The case-sensitive path to the process BPMN file.                  *;
%* @param lsaf_version - Optional - The version of the BPMN file. If blank, the most recent version *;
%*            is used.                                                                              *;
%* @param sas_dsname - Required - The name of a SAS data set that contains the context type         *;
%*           identifiers where the process flow definition is to be deployed, specified as          *;
%*           libref.dataset. The data set must contain at least the variable listed below.          *;
%*           Additional variables are ignored.                                                      *;
%*            - id: The context type id. The identifiers can be retrieved by calling the            *;
%*              lsaf_getcontexttpes macro.                                                          *;
%* @param lsaf_comment - Optional - The audit trail comment for the deployment.                     *;
%* @param lsaf_activate - Optional - Indicates whether to activate the process flow definition.     *;
%*            Valid values: 0: Do not activate, 1: Activate (Default).                              *;
%* @param lsaf_overwrite - Optional - Indicates whether an existing process flow definition with    *;
%*             same identifier is overwritten.                                                      *;
%*             Valid values: 0: Do not overwrite (Default), 1: Overwrite.                           *;
%*                                                                                                  *;
%* @macrotype LSAF                                                                                  *;
%* @since 2.2                                                                                       *;
%* @exposure external                                                                               *;
%*                                                                                                  *;
%* @history 2019-09-12  initial coding                                                              *;
%*                                                                                                  *;

%macro lsaf_deployprocessdefinition(
  lsaf_path=,
  lsaf_version=,
  sas_dsname=,
  lsaf_comment=,
  lsaf_activate=1,
  lsaf_overwrite=0
  ) /des='Deploy the process flow definitions for a SAS Life Science Analytics Framework process.';

  %*************************************************************;
  %* Turn off the notes generated for all the pre-processing.  *;
  %* Save original option setting so it can be reset properly. *;
  %*************************************************************;
  %let notesOption=%sysfunc(getoption(notes));

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafRC_
          _lsafProcessDefinitionId_
  ;

  %local logMsgPrefix
         logMsgPrimary
         macroName
  ;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);
  %let macroName=%sysfunc(lowcase(&SYSMACRONAME));
  
  %let logMsgPrimary=%str(ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro: );
  %let logMsgPrefix=%str(NOTE: &_LSAF_PRODUCT_NAME_ Macro: );

  %***************************************************************************;
  %* Validate the input data set                                             *;
  %* sets global macro variable _lsafRC_=0 if validation successful          *;
  %***************************************************************************;
  %lsaf_validate_sasdataset(callingMacro=&macroName,
        sas_dsn=&sas_dsname,
        required_variables=%str(id)
        );
  %if (&_lsafRC_ ne 0) %then
  %do;
    options &notesOption;
    %put &logMsgPrimary _lsafMsg_= &_lsafMsg_ ;
    %put &logMsgPrefix &macroName ;
    %put &logMsgPrefix Input data set = &sas_dsname;
    %put &logMsgPrefix _lsafRC_= &_lsafRC_ ;
    %put;
    %return;
  %end;

  %***************************************************************************;
  %* If data set is valid, process it.                                       *;
  %***************************************************************************;
  %put NOTE: &macroName: Processing data set &sas_dsname;

  option notes;

  %***********************;
  %* Process the records *;
  %***********************;

  data _null_;
    attrib returnCode    length=8
           message       length=$200
           logMsgPrefix  length=$70
           logMsgPrimary length=$70
           logMsg        length=$300
           updateDef     length=$300
    ;
    set &sas_dsname end=eof;

    %**********************************;
    %* Declare the java object, once  *;
    %**********************************;
    if (_n_ = 1) then
    do;
      declare javaobj processDefinitionIds("com/sas/lsaf/macro/common/SasStrings");
    end;

    %******************************************;
    %* Populate the processDefinitionIds list *; 
    %******************************************;
    processDefinitionIds.callVoidMethod("addValue", kstrip(id));

    if eof then 
    do;
      %***********************************;
      %* Call the update                 *;
      %***********************************; 
      declare javaobj srv("com/sas/lsaf/macro/workflow/definition/ProcessDefinitionService");
      declare javaobj results("com/sas/lsaf/macro/results/Result");
      
      srv.callStaticStringMethod("deployProcessDefinitionFromRepository", "%bquote(&lsaf_path)", "%bquote(&lsaf_version)", processDefinitionIds,
                               "%bquote(&lsaf_comment)", "%bquote(&lsaf_activate)", "%bquote(&lsaf_overwrite)", results, updateDef);

      %***********************************;
      %* Retrieve the results            *;
      %***********************************;
      results.callIntMethod("getReturnCode", returnCode);
      results.callStringMethod("getReturnMessage", message);

      %***********************************;
      %* Process the results             *;
      %***********************************;
      logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
      if (returnCode ne 0) then
      do;
        logMsgPrimary="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
        logMsg=" Cannot deploy process flow definition.";
      end;
      else
      do;
        logMsgPrimary=logMsgPrefix;
        logMsg=" Deployment of process defintion was performed.";
      end;

      %***********************************;
      %* Print messages in log           *;
      %***********************************;
      put;
      put logMsgPrimary " " logMsg;
      put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
      put logMsgPrefix " _lsafMsg_= " message ;
      put logMsgPrefix " _lsafRC_= " returnCode ;
      put logMsgPrefix " Input data set = &sas_dsname";
      put logMsgPrefix " Deployed process definition ID = " updateDef;
      put;

      %***********************************;
      %* Set the macro variables         *;
      %***********************************;
      call symputx("_lsafRC_", returnCode);
      call symputx("_lsafMsg_", message);
      call symputx("_lsafProcessDefinitionId_", updateDef);
    end;
  run;

%mend lsaf_deployprocessdefinition;