%* lsaf_exportdictionary                                                                              *;
%*                                                                                                    *;
%* Exports the metadata for a controlled terminology dictionary. The metadata is stored in a SAS      *;
%* data set based on the specified location (WORKSPACE or REPOSITORY).                                *;
%*                                                                                                    *;
%* Syntax:                                                                                            *;
%* %LSAF_EXPORTDICTIONARY(LSAF_DICTIONARYID=lsaf-dictionaryid,                                        *;
%*     LSAF_STATUS=lsaf-status,                                                                       *;
%*     LSAF_DICTIONARYVERSION=lsaf-dictionaryversion,                                                 *;
%*     LSAF_LOCATION=lsaf-location,                                                                   *;
%*     LSAF_PATH=lsaf-path,                                                                           *;
%*     LSAF_OVERWRITE=lsaf-overwrite,                                                                 *;
%*     LSAF_ENABLEVERSIONING=lsaf-enableversioning,                                                   *;
%*     LSAF_VERSIONTYPE=lsaf-versiontype,                                                             *;
%*     LSAF_CUSTOMVERSION=lsaf-customversion,                                                         *;
%*     LSAF_COMMENT=lsaf-comment);                                                                    *;
%*                                                                                                    *;
%* @macvar _lsafRC_  The return code.                                                                 *;
%* @macvar _lsafMsg_ The return message.                                                              *;
%* @macvar _lsafDictionaryPath_ The full path of the dictionary data set, which includes the file     *;
%*     extension. If the dictionary data set is not created, _lsafDictionaryPath_ is blank.           *;
%*                                                                                                    *;
%* @param lsaf_dictionaryid - Required - The identifier of the dictionary. The identifier can be      *;
%*     retrieved by calling the lsaf_getdictionaryid macro.                                           *;
%* @param lsaf_status - Required - The status of the dictionary. Valid values: PUBLISHED,             *;
%*     INPROGRESS, and CHECKEDOUT. The default value is PUBLISHED.                                    *;
%* @param  lsaf_dictionaryversion - Optional - The version of a PUBLISHED dictionary to export. A     *;
%*     blank value returns the latest version. This option is ignored if the status is not PUBLISHED. *;
%* @param  lsaf_location - Required - The case-insensitive output location for the dictionary file.   *;
%*     Valid values: WORKPSACE and REPOSITORY.                                                        *;
%* @param  lsaf_path - Required - The output path for the dictionary file. If the file extension is   *;
%*     not specified, it is added during processing.                                                  *;
%* @param lsaf_overwrite - Optional -  Indicates whether an existing unversioned repository or        *;
%*     existing workspace exported file will be overwritten by the file with a same path.             *;
%*     When a versioned repository file already exists, this parameter is ignored and the             *;
%*     parameters versionType and customVersion, or their defaults if not specified, are              *;
%*     applied to the new version of the file.                                                        *;
%*     Valid values: 0 = No (Default), 1 = Yes.                                                       *;
%* @param lsaf_enableversioning - Optional - Indicates whether a new exported file is versioned.      *;
%*     Ignored if lsaf_location=WORKSPACE. Valid values: 0 (Default): No, 1: Yes                      *;
%* @param lsaf_versiontype - Optional - The case insensitive type for a versioned file                *;
%*     in the repository. This parameter is ignored when writing to the workspace and will            *;
%*     not apply if the file exists as a non-versioned repository file. If not provided, the          *;
%*     system defaults will be applied.                                                               *;
%*     Valid values: MAJOR, MINOR, or CUSTOM.                                                         *;
%* @param lsaf_customversion - Conditional - Sets the specified version value if lsaf_versiontype is  *;
%*     CUSTOM. This parameter is ignored when writing to the workspace and will not apply if the      *;
%*     file exists as a non-versioned repository file.                                                *;
%* @param  lsaf_comment - Optional - The check-in comment to associate with the action of adding the  *;
%*     data set to the repository. This option is ignored if lsaf_location=WORKSPACE.                 *;
%*                                                                                                    *;
%* @macrotype LSAF                                                                                    *;
%* @since 2.2                                                                                         *;
%* @exposure external                                                                                 *;
%*                                                                                                    *;
%* @history 2019-06-27  initial coding                                                                *;
%* @history 2020-07-14  update for refactor of java service hierarchy.                                *;
%* @history 2020-11-23  update for versioning strategy.                                               *;
%*                                                                                                    *;

%macro lsaf_exportdictionary(
  lsaf_dictionaryid=,
  lsaf_status=,
  lsaf_dictionaryversion=,
  lsaf_location=,
  lsaf_path=,
  lsaf_overwrite=,
  lsaf_enableversioning=,
  lsaf_versiontype=,
  lsaf_customversion=,
  lsaf_comment=
  ) /des='Exports a control terminology dictionary to a specified location in the SAS Life Science Analytics Framework.';

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafRC_
          _lsafDictionaryPath_
  ;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data _null_;
    attrib returnCode      length=8
           message         length=$200
           dictionaryPath  length=$2048
           logMsgPrefix    length=$70
           logMsgPrimary   length=$70
           logMsg          length=$300
    ;
    
    dictionaryPath="";
    
    %*****************************;
    %* Declare the java objects  *;
    %*****************************;
    declare javaobj srv("com/sas/lsaf/macro/clinical/dictionary/DictionaryService");
    declare javaobj exportData("com/sas/lsaf/macro/content/SasFileCreateParameters"); 
    declare javaobj results("com/sas/lsaf/macro/results/Result");
 
    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    exportData.callVoidMethod("updateData", "%bquote(&lsaf_location)", "%bquote(&lsaf_path)", "%bquote(&lsaf_overwrite)", 
                                "%bquote(&lsaf_enableversioning)", "%bquote(&lsaf_versiontype)", "%bquote(&lsaf_customversion)", 
                                "%bquote(&lsaf_comment)");
    srv.callStaticStringMethod("exportDictionaryToDataSet", "%bquote(&lsaf_dictionaryid)", "%bquote(&lsaf_status)", "%bquote(&lsaf_dictionaryversion)", 
       		exportData, results, dictionaryPath);

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);
    logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
    if (returnCode ne 0) then
    do;
      logMsgPrimary="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Could not export the dictionary." ;
    end;
    else
    do;
      logMsgPrimary=logMsgPrefix;
      logMsg=" Returned the path for the created dictionary file.";
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrimary " " logMsg;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    if (returnCode ne 0) 
      then put logMsgPrefix " The dictionary was not exported.";
      else put logMsgPrefix " The dictionary was exported to " dictionaryPath;
    put logMsgPrefix " _lsafMsg_= " message ;
    put logMsgPrefix " _lsafRC_= " returnCode ;
    put logMsgPrefix " _lsafDictionaryPath_=" dictionaryPath;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);
    call symputx("_lsafDictionaryPath_", dictionaryPath);
  run;

%mend lsaf_exportdictionary;
