%* lsaf_exportstudydocs                                                                               *;
%*                                                                                                    *;
%* Exports the metadata for the study supporting documents. The metadata is stored in a SAS data set  *;
%* in the workspace or repository.                                                                    *;
%*                                                                                                    *;
%* Syntax:                                                                                            *;
%* %LSAF_EXPORTSTUDYDOCS(LSAF_STUDYID=lsaf-studyid,                                                   *;
%*     LSAF_BASESTANDARDNAME=lsaf-basestandardname,                                                   *;
%*     LSAF_MODELID=lsaf-modelid,                                                                     *;
%*     LSAF_LOCATION=lsaf-location,                                                                   *;
%*     LSAF_PATH=lsaf-path,                                                                           *;
%*     LSAF_OVERWRITE=lsaf-overwrite,                                                                 *;
%*     LSAF_ENABLEVERSIONING=lsaf-enableversioning,                                                   *;
%*     LSAF_VERSIONTYPE=lsaf-versiontype,                                                             *;
%*     LSAF_CUSTOMVERSION=lsaf-customversion,                                                         *;
%*     LSAF_COMMENT=lsaf-comment);                                                                    *;
%*                                                                                                    *;
%* @macvar _lsafRC_  The return code.                                                                 *;
%* @macvar _lsafMsg_ The return message.                                                              *;
%* @macvar _lsafexportstudydocspath_ The full path of the documents data set, which includes the file *;
%*     extension. If the data set is not created, _lsafexportstudydocspath_ is blank.                 *;
%*                                                                                                    *;
%* @param lsaf_studyid - Required - The identifier of the study. The identifier can be                *;
%*     retrieved by calling the lsaf_getstudyid macro.                                                *;
%* @param lsaf_basestandardname - Required - The case sensitive name of the base standard, such as    *;
%*     SDTM and ADaM.                                                                                 *;
%* @param  lsaf_modelid - Required - The identifier of the model. This can be retrieved using the     *;
%*     lsaf_getstudystandards macro.                                                                  *;
%* @param  lsaf_location - Required - The case-insensitive output location for the exported file.     *;
%*     Valid values: WORKSPACE and REPOSITORY.                                                        *;
%* @param  lsaf_path - Required - The case-sensitive output path for the exported files.              *;
%* @param  lsaf_overwrite - Optional - Indicates whether existing nonversioned data sets are          *;
%*     overwritten. Valid values: 0 (Default): No, 1: Yes.                                            *;
%* @param  lsaf_enableversioning - Optional - Indicates whether versioning is enabled for new         *;
%*     exported files. Valid values: 0 (Default): No, 1: Yes.                                         *;
%* @param lsaf_versiontype - Optional - The case insensitive type for a versioned file                *;
%*     in the repository. This parameter is ignored when writing to the workspace and will            *;
%*     not apply if the file exists as a non-versioned repository file. If not provided, the          *;
%*     system defaults will be applied.                                                               *;
%*     Valid values: MAJOR, MINOR, or CUSTOM.                                                         *;
%* @param lsaf_customversion - Conditional - Sets the specified version value if lsaf_versiontype is  *;
%*     CUSTOM. This parameter is ignored when writing to the workspace and will not apply if the      *;
%*     file exists as a non-versioned repository file.                                                *;
%* @param  lsaf_comment - Optional - The check-in comment to associate with the action of adding the  *;
%*      data set to the repository. If lsaf_location=WORKSPACE, this option is ignored.               *;
%*                                                                                                    *;
%* @macrotype LSAF                                                                                    *;
%* @since 2.2                                                                                         *;
%* @exposure external                                                                                 *;
%*                                                                                                    *;
%* @history 2019-08-21  initial coding                                                                *;
%* @history 2020-11-23  update for versioning strategy.                                               *;
%*                                                                                                    *;


%macro lsaf_exportstudydocs(
  lsaf_studyid=,
  lsaf_basestandardname=,
  lsaf_modelid=,
  lsaf_location=,
  lsaf_path=,
  lsaf_overwrite=,
  lsaf_enableversioning=,
  lsaf_versiontype=,
  lsaf_customversion=,
  lsaf_comment=
  ) /des='Exports study supporting documents to a specified location in the SAS Life Science Analytics Framework.';

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafRC_
		  _lsafexportstudydocspath_
  ;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data _null_;
	attrib returnCode         length=8
           message            length=$200
           logMsgPrefix       length=$70
           logMsgPrimary      length=$70
           logMsg             length=$300
		   exportPath         length=$2048
    ;

    %*****************************;
    %* Declare the java objects  *;
    %*****************************;
    declare javaobj srv("com/sas/lsaf/macro/clinical/study/StudyService") ;
    declare javaobj exportData("com/sas/lsaf/macro/content/SasFileCreateParameters"); 
    declare javaobj results("com/sas/lsaf/macro/results/Result");
 
    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    exportData.callVoidMethod("updateData", "%bquote(&lsaf_location)", "%bquote(&lsaf_path)", "%bquote(&lsaf_overwrite)", 
                                "%bquote(&lsaf_enableversioning)", "%bquote(&lsaf_versiontype)", "%bquote(&lsaf_customversion)", 
                                "%bquote(&lsaf_comment)");
                                
    srv.callStaticStringMethod("exportStudySupportingDocuments", "%bquote(&lsaf_studyid)", "%bquote(&lsaf_basestandardname)", "%bquote(&lsaf_modelid)", 
         exportData, results, exportPath);

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);
    logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
    if (returnCode ne 0) then
    do;
      logMsgPrimary="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Could not export the study supporting documents." ;
    end;
    else
    do;
      logMsgPrimary=logMsgPrefix;
      logMsg=" Returned the path for the created study supporting documents file.";
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrimary " " logMsg;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    if (returnCode ne 0) 
      then put logMsgPrefix " The study supporting documents were not exported.";
      else put logMsgPrefix " The study supporting documents were exported to " exportPath;
    put logMsgPrefix " _lsafMsg_= " message ;
    put logMsgPrefix " _lsafRC_= " returnCode ;
    put logMsgPrefix " _lsafexportstudydocspath_=" exportPath;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);
    call symputx("_lsafexportstudydocspath_", exportPath);
  run;

%mend lsaf_exportstudydocs;
