%* lsaf_exportstudytablestows                                                                         *;
%*                                                                                                    *;
%* Exports the metadata for the study tables to a location in the workspace. The metadata are         *;
%* stored in SAS data sets. The paths of the exported files are stored in a SAS data set.             *;
%*                                                                                                    *;
%* Syntax:                                                                                            *;
%* %LSAF_EXPORTSTUDYTABLESTOWS(LSAF_STUDYID=lsaf-studyid,                                             *;
%*     LSAF_BASESTANDARDNAME=lsaf-basestandardname,                                                   *;
%*     LSAF_MODELID=lsaf-modelid,                                                                     *;
%*     LSAF_PATH=lsaf-path,                                                                           *;
%*     LSAF_PREFIX=lsaf-prefix,                                                                       *;
%*     LSAF_OVERWRITE=lsaf-overwrite,                                                                 *;
%*     SAS_RESULT_DSNAME=WORK.LSAFEXPORTSTUDYTABLESTOWS);                                             *;
%*                                                                                                    *;
%* @macvar _lsafRC_  The return code.                                                                 *;
%* @macvar _lsafMsg_ The return message.                                                              *;
%*                                                                                                    *;
%* @param lsaf_studyid - Required - The identifier of the study. The identifier can be                *;
%*     retrieved by calling the lsaf_getstudyid macro.                                                *;
%* @param lsaf_basestandardname - Required - The case sensitive name of the base standard, such as    *;
%*     SDTM and ADaM.                                                                                 *;
%* @param  lsaf_modelid - Required - The identifier of the model. This can be retrieved using the     *;
%*     lsaf_getstudystandards macro.                                                                  *;
%* @param  lsaf_path - Required - The case-sensitive output path for the exported files.              *;
%* @param  lsaf_prefix - Required - The lower case prefix of the SAS data sets that are created for   *;
%*     the exported tables.                                                                           *;
%* @param  lsaf_overwrite - Optional - Indicates whether existing data sets are overwritten.          *;
%*     Valid values: 0 (Default): No, 1: Yes.                                                         *;
%* @param sas_result_dsname - Optional - The name of the output SAS data set to contain the path,     *;
%*     name, and location of the exported table SAS data sets, specified as libref.dataset.           *;
%*     The default is WORK.LSAFEXPORTSTUDYTABLESTOWS. The data set contains a row for each exported   *;
%*     data set and columns with the following names. The data set is sorted by path.                 *;
%*     - location: The location of the exported file (WORKSPACE).                                     *;
%*     - name: The name of the exported file.                                                         *;
%*     - path: The path of the exported file.                                                         *;
%*                                                                                                    *;
%* @macrotype LSAF                                                                                    *;
%* @since 2.2                                                                                         *;
%* @exposure external                                                                                 *;
%*                                                                                                    *;
%* @history 2019-08-21  initial coding                                                                *;
%* @history 2019-11-01  update to handle a blank SAS output file.                                     *;
%*                                                                                                    *;

%macro lsaf_exportstudytablestows(
  lsaf_studyid=,
  lsaf_basestandardname=,
  lsaf_modelid=,
  lsaf_path=,
  lsaf_prefix=,
  lsaf_overwrite=,
  sas_result_dsname=LSAFEXPORTSTUDYTABLESTOWS
  ) /des='Exports study tables to data sets in the workspace.';
  
  %if "&sas_result_dsname" EQ "" %then %let sas_result_dsname=work.LSAFEXPORTSTUDYTABLESTOWS;

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafRC_
  ;

  %local _lsafRowCount_;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);
  %let _lsafRowCount_=0;

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data &sas_result_dsname (label="Created by &SYSMACRONAME" keep=location path name sortPath);
	attrib returnCode         length=8
           message            length=$200
           logMsgPrefix       length=$70
           logMsgPrimary      length=$70
           logMsg             length=$300
           rowCount           length=8
           location           length=$10      label="Location"
           path               length=$2048    label="Export file Path"
           name               length=$256     label="Export file Name"
           sortpath           length=$2048    label="NEEDED FOR SORTING - will be dropped"    ;

    %*****************************;
    %* Declare the java objects  *;
    %*****************************;
    declare javaobj srv("com/sas/lsaf/macro/clinical/study/StudyService") ;
    declare javaobj exportData("com/sas/lsaf/macro/content/SasBatchFileCreateParameters");
    declare javaobj results("com/sas/lsaf/macro/results/SasPathResult");
 
    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    exportData.callVoidMethod("updateBatchData", "%bquote(&lsaf_path)", "%bquote(&lsaf_overwrite)", "%bquote(&lsaf_prefix)");
    srv.callStaticVoidMethod("exportStudyTablesToWorkspace", "%bquote(&lsaf_studyid)", "%bquote(&lsaf_basestandardname)", "%bquote(&lsaf_modelid)", 
      exportData, results);

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);
    logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
    if (returnCode ne 0) then
    do;
      logMsgPrimary="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Could not export the study tables." ;
    end;
    else
    do;
      logMsgPrimary=logMsgPrefix;
      logMsg=" Returned the list of paths for the study tables.";
      results.callIntMethod("getSize", rowCount);
      results.callStringMethod("getLocation", location);
      do row=0 to rowCount - 1;
        results.callStringMethod("getPath", row, path);
        sortPath = kstrip( kupcase( ktranslate(path,'1','_') ) );
        results.callStringMethod("getName", row, name);
        output;
      end;
    end;
    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrimary " " logMsg;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
	if (rowCount<=0) 
      then put logMsgPrefix " The study tables were not exported.";
      else put logMsgPrefix " " rowCount " records written to data set &sas_result_dsname";
    put logMsgPrefix " _lsafMsg_= " message ;
    put logMsgPrefix " _lsafRC_= " returnCode ;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);
    call symputx("_lsafRowCount_", rowCount);
  run;

  %***********************************;
  %* Sort the data set               *;
  %***********************************;
  %let notesOption=%sysfunc(getoption(notes));
  options nonotes;

  %if (&_lsafRowCount_ > 0) %then
  %do;
     proc sort data=&sas_result_dsname
               out=&sas_result_dsname (drop=sortpath);
         by sortpath;
     run;
  %end;
  %else
  %do;
     proc sql;
       alter table &sas_result_dsname
          drop sortpath;
     quit;
  %end;
  
  options &notesOption;

%mend lsaf_exportstudytablestows;
