%* lsaf_getallschedules                                                                            *;
%*                                                                                                 *;
%* Gets the metadata for all of the schedules. The metadata is stored in a SAS data set.           *;
%* The global privilege PRIVILEGE_MANAGE_ALL_SCHEDULES is required to retrieve any schedules,      *;
%* including the schedules that are owned by the current user.                                     *;
%*                                                                                                 *;
%* Syntax:                                                                                         *;
%* %LSAF_GETALLSCHEDULES(<SAS_DSNAME=SAS-data-set>);                                               *;
%*                                                                                                 *;
%* @macvar _lsafRC_  The return code.                                                              *;
%* @macvar _lsafMsg_ The return message.                                                           *;
%*                                                                                                 *;
%* @param sas_dsname - Optional - The name of the output SAS data set to contain the metadata      *;
%*            for the schedules, specified as libref.dataset. The default value is                 *;
%*            WORK.LSAFGETALLSCHEDULES and is sorted by the schedule identifier.                   *;
%*            The data set contains a row for each schedule deployed and has columns with the      *;
%*            following names.                                                                     *;
%*            - scheduleID: The identifier of the schedule.                                        *;
%*            - name: The name of the schedule.                                                    *;
%*            - description: The description of the schedule.                                      *;
%*            - owner: The owner of the schedule.                                                  *;
%*            - status: The status of the schedule.                                                *;
%*            - lastModified: The date on which the schedule was last modified, represented as a   *;
%*              String.                                                                            *;
%*            - dateLastModified: The date on which the schedule was last modified, represented as *;
%*              a SAS Datetime format.                                                             *;
%*                                                                                                 *;
%*                                                                                                 *;
%* @macrotype LSAF                                                                                 *;
%* @since 2.3                                                                                      *;
%* @exposure external                                                                              *;
%*                                                                                                 *;
%* @history 2019-10-01  initial code                                                               *;
%* @history 2020-06-30  update service location                                                    *;
%*                                                                                                 *;

%macro lsaf_getallschedules(
  sas_dsname=work.lsafGetAllSchedules
  ) /des='Get a list of all schedules.';

  %if "&sas_dsname" EQ "" %then %let sas_dsname=work.lsafGetAllSchedules;

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafRC_
  ;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %local _lsafRowCount_;

  %let _lsafRowCount_=0;

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data &sas_dsname (label="Created by &SYSMACRONAME"
                    keep=scheduleID name description owner status 
                    lastModified dateLastModified)
  ;
    attrib returnCode          length=8
           message             length=$200
           logMsgPrefix        length=$70
           logMsgPrimary       length=$70
           logMsg              length=$300
           rowCount            length=8

           scheduleID          length=$256   label="Schedule ID"
           name                length=$256   label="Schedule Name"
           description         length=$1024  label="Schedule Description"
           owner               length=$32    label="Schedule Owner"
           status              length=$32    label="Schedule Status"
           lastModified        length=$64    label="Last Schedule Modification Date"
           dateLastModified                  label="Date the schedule was last modified (SAS DateTime Format)"
           
           lastModifiedTempSAS  length=$64   label="TEMPORARY - WILL BE DROPPED"
    ;

    format dateLastModified datetime.
    ;

    rowCount=0;

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/lsaf/macro/schedule/ScheduleService") ;
    declare javaobj results("com/sas/lsaf/macro/schedule/SasSchedulesResult");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticVoidMethod("getAllSchedules", results) ;

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
    if (returnCode ne 0) then
    do;
      logMsgPrimary="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Cannot retrieve schedules." ;
    end;
    else
    do;
      logMsgPrimary=logMsgPrefix;
      logMsg=" Schedules were returned." ;
      results.callIntMethod("getSize", rowCount);
      do row=0 to rowCount - 1;
        results.callStringMethod("getId", row, scheduleID);
        results.callStringMethod("getName", row, name);
        results.callStringMethod("getDescription", row, description);
        results.callStringMethod("getOwner", row, owner);
        results.callStringMethod("getStatus", row, status);
        results.callStringMethod("getLastModified", row, lastModified);
        results.callStringMethod("getLastModifiedSasFormat", row, lastModifiedTempSas);
        dateLastModified=input(lastModifiedTempSas, datetime.);
        output;
      end;
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrimary " " logMsg;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    if (rowCount=0) 
      then put logMsgPrefix " No schedules were found.";
      else put logMsgPrefix " " rowCount " records written to data set &sas_dsname";
    put logMsgPrefix " _lsafMsg_= " message ;
    put logMsgPrefix " _lsafRC_= " returnCode ;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);
    call symputx("_lsafRowCount_", rowCount);

  run;

  %***********************************;
  %* Sort by name                    *;
  %***********************************;
  %if (&_lsafRowCount_ > 0) %then
  %do;
    %let notesOption=%sysfunc(getoption(notes));
    options nonotes;
    proc sort data=&sas_dsname;
      by scheduleID;
    run;
    options &notesOption;
  %end;

%mend lsaf_getallschedules;
