%* lsaf_getcontextallowablechildren                                                              *;
%*                                                                                               *;
%* Gets the metadata for the item types that can be created as immediate children of the         *;
%* specified context type in the repository. The metadata is stored in a SAS data set.           *;
%*                                                                                               *;
%* To get a list of valid context types, run the macro %lsaf_getcontexttypes().                  *; 
%*                                                                                               *;
%* Syntax:                                                                                       *;
%* %LSAF_GETCONTEXTALLOWABLECHILDREN(LSAF_TYPEID=lsaf-typeid, SAS_DSNAME=SAS-data-set>);         *;
%*                                                                                               *;
%* @macvar _lsafRC_  The return code.                                                            *;
%* @macvar _lsafMsg_ The return message.                                                         *;
%*                                                                                               *;
%* @param lsaf_typeid - Required - The type identifier of the context item in the repository.    *;
%*                                                                                               *;
%* @param sas_dsname - Optional - The name of the output SAS data set to contain the metadata    *;
%*            for the items, specified as libref.dataset. The default value is                   *;
%*            WORK.LSAFGETCONTEXTALLOWABLECHILDREN. The data set contains a row for each item    *;
%*            in the list and columns with the following names.  The data set is sorted by the   *;
%*            id.                                                                                *;
%*            - ListType: The list type, in this case Context allowable children.                *;
%*            - typeId: The context type ID.                                                     *;
%*            - id: The child ID of the item.                                                    *;
%*            - name: The name of the item.                                                      *;
%*            - description: The description of the item.                                        *;
%*            - lsafType: The type of the item.                                                  *;
%*            - allowableChildTypes: The allowable child types for this item.                    *;
%*            - capabilities: The capabilities of this child.                                    *;
%*                                                                                               *;
%* @macrotype LSAF                                                                               *;
%* @since 2.1                                                                                    *;
%* @exposure external                                                                            *;
%*                                                                                               *;
%* @history 2019-01-15  initial coding                                                           *;
%* @history 2019-11-01  update to handle a blank SAS output file.                                *;
%* @history 2020-07-13  update for refactor of java service hierarchy.                           *;
%*                                                                                               *;

%macro lsaf_getcontextallowablechildren(
  lsaf_typeid=,
  sas_dsname=work.lsafGetContextAllowableChildren
  ) /des='Get a list of the items in a context in SAS Life Science Analytics Framework.';
  
  %if "&sas_dsname" EQ "" %then %let sas_dsname=work.lsafGetContextAllowableChildren;

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafRC_
  ;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %local _lsafRowCount_;

  %let _lsafRowCount_=0;

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data &sas_dsname (label="Created by &SYSMACRONAME"
                    keep=typeId id name description lsafType
                    allowableChildTypes capabilities listType
                    );

    attrib returnCode                length=8
           message                   length=$200
           logMsgPrefix              length=$70
           logMsg                    length=$300
           rowCount                  length=8
           logMsgPrimary             length=$70

           listType                  length=$32      label="LSAF List Type"           
           typeId                    length=$2048    label="Context Type ID"
           id                        length=$2048    label="Child Type ID"
           name                      length=$256     label="Type Name"
           lsafType                  length=$32      label="LSAF Type"
           description               length=$512     label="Description"
           allowableChildTypes       length=$2048    label="Allowable Child Types"
           capabilities              length=$2048    label="Capabilities"
    ;

    rowCount=0;

    %***********************************;
    %* Declare the java items        *;
    %***********************************;
    declare javaobj srv("com/sas/lsaf/macro/core/type/TypeService");                                                                                  
    declare javaobj results("com/sas/lsaf/macro/core/type/SasTypesResult");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticVoidMethod("getContextAllowableChildren", "%bquote(&lsaf_typeid)", results);

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
    if (returnCode ne 0) then
    do;
      logMsgPrimary="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Could not retrieve list of context items from %bquote(&lsaf_typeid)" ;
    end;
    else
    do;
      logMsgPrimary=logMsgPrefix;
      logMsg=" Returned a list of context items in %bquote(&lsaf_typeid)";
      results.callIntMethod("getSize", rowCount);
      results.callStringMethod("getListType", listType);                                                             
      typeId="&lsaf_typeid";
      do row=0 to rowCount - 1;
        results.callStringMethod("getId", row, id);
        results.callStringMethod("getName", row, name);
        results.callStringMethod("getDescription", row, description);
        results.callStringMethod("getLsafType", row, lsafType);
        results.callStringMethod("getAllowableChildTypes", row, allowableChildTypes);
        results.callStringMethod("getCapabilities", row, capabilities);
        output;
      end;
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrimary " " logMsg;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    if (rowCount=0) 
      then put logMsgPrefix " No allowable children were found.";
      else put logMsgPrefix " " rowCount " records written to data set &sas_dsname";
    put logMsgPrefix " _lsafMsg_= " message ;
    put logMsgPrefix " _lsafRC_= " returnCode ;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);
    call symputx("_lsafRowCount_", rowCount);

  run;

  %let notesOption=%sysfunc(getoption(notes));
  options nonotes;

  %***********************************;
  %* Sort the data set               *;
  %***********************************;
  %if (&_lsafRowCount_ > 0) %then
  %do;
    proc sort data=&sas_dsname;
      by id;
    run;
  %end;

  options &notesOption;

%mend lsaf_getcontextallowablechildren ;
