%* lsaf_getcontextstate                                                                           *;
%*                                                                                                *;
%* Gets the state value for the specified context in the repository. The SAS macro variable       *;
%* _lsafContextState_ is set to the state value. Values returned:                                 *;
%* - ACTIVE: The context is in active state.                                                      *;
%* - CLOSED: The context is in closed state.                                                      *;
%*                                                                                                *;
%* Syntax:                                                                                        *;
%* %LSAF_GETCONTEXTSTATE(LSAF_PATH=lsaf-path);                                                    *;
%*                                                                                                *;
%* @macvar _lsafRC_  The return code.                                                             *;
%* @macvar _lsafMsg_ The return message.                                                          *;
%* @macvar _lsafContextState_ The state of the context.                                           *;
%*                                                                                                *;
%* @param lsaf_path - Required - The path to the context.                                         *;
%*                                                                                                *;
%* @macrotype LSAF                                                                                *;
%* @since 2.1                                                                                     *;
%* @exposure external                                                                             *;
%*                                                                                                *;
%* @history 2019-01-30  initial code                                                              *;
%*                                                                                                *;

%macro lsaf_getcontextstate(
  lsaf_path=
  ) /des='Returns the state of a SAS Life Science Analytics Framework context.';

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafContextState_
          _lsafRC_
  ;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data _null_;
    attrib returnCode      length=8;
    attrib message         length=$200;
    attrib logMsgPrefix    length=$70;
    attrib logMsgPrimary   length=$70;
    attrib logMsg          length=$300;
    attrib state           length=$6;

    state="";

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/lsaf/macro/content/repository/RepositoryService");
    declare javaobj results("com/sas/lsaf/macro/results/Result");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticStringMethod("getContextState", "%bquote(&lsaf_path)", results, state);

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
    if (returnCode ne 0) then
    do;
      logMsgPrimary="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Cannot determine context state: &lsaf_path." ;
    end;
    else
    do;
      logMsgPrimary=logMsgPrefix;
      logMsg=" Returned context state: &lsaf_path." ;
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrimary " " logMsg;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    put logMsgPrefix " _lsafMsg_= " message;
    put logMsgPrefix " _lsafRC_= " returnCode;
    put logMsgPrefix " _lsafContextState_= " state;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);
    call symputx("_lsafContextState_", strip(state) ) ;

  run;

%mend lsaf_getcontextstate;
