%* lsaf_getcopytoworkspacestatus                                                                  *;
%*                                                                                                *;
%* Gets the value for the Copy to Workspace status of a file or folder in the repository. The     *;
%* SAS macro variable _lsafCopyToWSStatus_ is set to the status value. Values returned:           *;
%* - ALLOW: Allow all.                                                                            *;
%* - WARN: Warn (excluding published jobs).                                                       *;
%* - DENY: Deny (excluding published jobs).                                                       *;
%* - DENY_ALL: Deny all.                                                                          *;
%*                                                                                                *;
%* Syntax:                                                                                        *;
%* %LSAF_GETCOPYTOWORKSPACESTATUS(LSAF_PATH=lsaf-path);                                           *;
%*                                                                                                *;
%* @macvar _lsafRC_  The return code.                                                             *;
%* @macvar _lsafMsg_ The return message.                                                          *;
%* @macvar _lsafCopyToWSStatus_ The Copy to Workspace status.                                     *;
%*                                                                                                *;
%* @param lsaf_path - Required - The path of the file.                                            *;
%*                                                                                                *;
%* @macrotype LSAF                                                                                *;
%* @since 1.5                                                                                     *;
%* @exposure external                                                                             *;
%*                                                                                                *;
%* @history 2016-11-22  initial code                                                              *;
%*                                                                                                *;

%macro lsaf_getcopytoworkspacestatus(
  lsaf_path=
  ) /des='Returns the copy to workspace status of a SAS Life Science Analytics Framework file or folder.';

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafCopyToWSStatus_
          _lsafRC_
  ;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data _null_;
    attrib returnCode      length=8;
    attrib message         length=$200;
    attrib logMsgPrefix    length=$70;
    attrib logMsgPrimary   length=$70;
    attrib logMsg          length=$300;
    attrib status          length=$25;

    status="";

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/lsaf/macro/content/repository/RepositoryService");
    declare javaobj results("com/sas/lsaf/macro/results/Result");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticStringMethod("getCopyToWorkspaceStatus", "%bquote(&lsaf_path)", results, status);

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
    if (returnCode ne 0) then
    do;
      logMsgPrimary="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Cannot determine copy to workspace status: &lsaf_path." ;
    end;
    else
    do;
      logMsgPrimary=logMsgPrefix;
      logMsg=" Returned copy to workspace status: &lsaf_path." ;
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrimary " " logMsg;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    put logMsgPrefix " _lsafMsg_= " message;
    put logMsgPrefix " _lsafRC_= " returnCode;
    put logMsgPrefix " _lsafCopyToWSStatus_= " status;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);
    call symputx("_lsafCopyToWSStatus_", strip(status) ) ;

  run;

%mend lsaf_getcopytoworkspacestatus;
