%* lsaf_getprocessflows                                                                            *;
%*                                                                                                 *;
%* Gets the metadata for all of the process flows defined at, and optionally the children of, the  *;
%* specified context. The metadata is stored in a SAS data set.                                    *;
%*                                                                                                 *;
%* Syntax:                                                                                         *;
%* %LSAF_GETPROCESSFLOWS(LSAF_PATH=lsaf-path <,SAS_DSNAME=SAS-data-set,                            *;
%* LSAF_INCLUDECHILDREN=includeChildren-flag>);                                                    *;
%*                                                                                                 *;
%* @macvar _lsafRC_  The return code.                                                              *;
%* @macvar _lsafMsg_ The return message.                                                           *;
%*                                                                                                 *;
%* @param lsaf_path - Optional - The path of the context in which the process flows are located.   *;
%*            If not secified, the macro returns all of the process flows for all contexts.        *;
%* @param sas_dsname - Optional - The name of the output SAS data set to contain the metadata      *;
%*            for the process flows, specified as libref.dataset. The default value is             *;
%*            WORK.LSAFGETPROCESSFLOWS. The data set contains a row for each process flow in       *;
%*            the context and columns with the following names. The data set is sorted by the      *;
%*            process flow name.                                                                   *;
%*            - path: The path of the context in which the process flow is located.                *;
%*            - name: The name of the process flow.                                                *;
%*            - description: The process flow description.                                         *;
%*            - createdBy: The user ID that created the process flow.                              *;
%*            - created: The full date and time on which the process flow was created. This value  *;
%*              is represented as a String.                                                        *;
%*            - dateCreated: The date on which the process flow was created, represented as a SAS  *;
%*              Datetime format.                                                                   *;
%*            - lastModifiedBy: The user ID that last modified the process flow.                   *;
%*            - lastModified: The date on which the process flow was last modified, represented    *;
%*              as a String.                                                                       *;
%*            - dateLastModified: The date on which the process flow was last modified,            *;
%*              represented as a SAS Datetime format.                                              *;
%*            - owner: The user ID of the owner of the process flow.                               *;
%*            - status: The current state of the process flow. Valid values: NOT_STARTED, ACTIVE,  *;
%*              SUSPENDED, or COMPLETED.                                                           *;
%*            - processDefinitionId: The ID of the process flow definition that was used to        *;
%*              create the process flow.                                                           *;
%*            - processDefinitionVersion: The version of the process flow definition that was      *;
%*              used to create the process flow.                                                   *;
%* @param lsaf_includeChildren - Optional - Indicates whether to include process flows from        *;
%*            child contexts. This parameter is ignored, if no path is provided. Valid values:     *;
%*            - 0: Exclude.                                                                        *;
%*            - 1 (Default): Include.                                                              *;
%*                                                                                                 *;
%* @macrotype LSAF                                                                                 *;
%* @since 1.5                                                                                      *;
%* @exposure external                                                                              *;
%*                                                                                                 *;
%* @history 2016-08-17  original creation.                                                         *;
%* @history 2019-02-12  update to 2.1                                                              *;
%* @history 2019-11-01  update to handle a blank SAS output file.                                  *;
%* @history 2021-01-22  add processFlowId column to sas_dsname.                                    *;
%*                                                                                                 *;

%macro lsaf_getprocessflows(
  lsaf_path=,
  sas_dsname=work.lsafGetProcessFlows,
  lsaf_includeChildren=1
  ) /des='Get a list of process flows in a SAS Life Science Analytics Framework context.';
  
  %if "&sas_dsname" EQ "" %then %let sas_dsname=work.lsafGetProcessFlows;

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafRC_
  ;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %local _lsafRowCount_;

  %let _lsafRowCount_=0;

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data &sas_dsname (label="Created by &SYSMACRONAME"
                    keep=contextPath name description createdBy created dateCreated lastModifiedBy lastModified dateLastModified
                         owner status processDefinitionId processDefinitionVersion processFlowId);

    attrib returnCode                length=8
           message                   length=$200
           logMsgPrefix              length=$70
           logMsgPrimary             length=$70
           logMsg                    length=$300
           rowCount                  length=8

           contextPath               length=$2048  label="Context Path"
           name                      length=$256   label="Process Flow Name"
           description               length=$1024  label="Process Flow Description"
           createdBy                 length=$64    label="Userid that created the process flow"
           created                   length=$64    label="Process Flow Creation Date"
           dateCreated                             label="Date the Process Flow was created (SAS DateTime Format)"
           lastModifiedBy            length=$64    label="Userid that last modified the process flow"
           lastModified              length=$64    label="Last Modification Date"
           dateLastModified                        label="Date the process flow was last modified (SAS DateTime Format)"
           owner                     length=$64    label="Userid of process flow owner"
           status                    length=$64    label="Status of the Process Flow"
           processDefinitionId       length=$256   label="Name of the Process Flow Definition used to create the Process Flow"
           processDefinitionVersion  length=$32    label="Version Number of the Process Flow Definition used to create the Process Flow"
           processFlowId             length=$32    label="Process Flow ID"

           createdTempSAS            length=$64    label="TEMPORARY - WILL BE DROPPED"
           lastModifiedTempSAS       length=$64    label="TEMPORARY - WILL BE DROPPED"
    ;

    format dateCreated
           dateLastModified datetime.
    ;

    rowCount=0;

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/lsaf/macro/workflow/processflow/ProcessFlowService") ;
    declare javaobj results("com/sas/lsaf/macro/workflow/processflow/SasProcessFlowsResult");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticVoidMethod("getProcessFlows", "%bquote(&lsaf_path)", "%bquote(&lsaf_includeChildren)", results) ;

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
    if (returnCode ne 0) then
    do;
      logMsgPrimary="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Cannot retrieve process flows for &lsaf_path" ;
    end;
    else
    do;
      logMsgPrimary=logMsgPrefix;
      logMsg=" Process flows were returned for &lsaf_path" ;
      results.callIntMethod("getSize", rowCount);
      do row=0 to rowCount - 1;
        results.callStringMethod("getContextPath", row, contextPath);
        results.callStringMethod("getName", row, name);
        results.callStringMethod("getDescription", row, description);
        results.callStringMethod("getCreated", row, created);
        results.callStringMethod("getCreatedSasFormat", row, createdTempSas);
        results.callStringMethod("getCreatedBy", row, createdBy);
        results.callStringMethod("getLastModified", row, lastModified);
        results.callStringMethod("getLastModifiedSasFormat", row, lastModifiedTempSas);
        results.callStringMethod("getLastModifiedBy", row, lastModifiedBy);
        results.callStringMethod("getStatus", row, status);
        results.callStringMethod("getOwner", row, owner);
        results.callStringMethod("getProcessDefinitionId", row, processDefinitionId);
        results.callStringMethod("getProcessDefinitionVersion", row, processDefinitionVersion);
        results.callStringMethod("getId", row, processFlowId);
        dateCreated=input(createdTempSAS, datetime.);
        dateLastModified=input(lastModifiedTempSas, datetime.);
        output;
      end;
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrimary " " logMsg;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    if (rowCount=0) 
      then put logMsgPrefix " No process flows were found.";
      else put logMsgPrefix " " rowCount " records written to data set &sas_dsname";
    put logMsgPrefix " _lsafMsg_= " message ;
    put logMsgPrefix " _lsafRC_= " returnCode ;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);
    call symputx("_lsafRowCount_", rowCount);

  run;

  %***********************************;
  %* Sort by name                    *;
  %***********************************;
  %if (&_lsafRowCount_ > 0) %then
  %do;
    %let notesOption=%sysfunc(getoption(notes));
    options nonotes;
    proc sort data=&sas_dsname;
      by name;
    run;
    options &notesOption;
  %end;

%mend lsaf_getprocessflows;
