%* lsaf_getrecyclebinitems                                                                       *;
%*                                                                                               *;
%* Gets the metadata for the items in the SAS Life Science Analytics Framework recycle bin that  *;
%* match the given search criteria. The metadata is stored in a SAS data set.                    *;
%*                                                                                               *;
%* The number of items that can be returned from a query is limited. The macro                   *;
%* variable _lsafRBSearchLimitReached_ returns a 0 if this maximum limit is not reached and a 1  *;
%* if the limit is reached. If the limit is reached, there might be items in the recycle bin     *;
%* that meet the given criteria. A more refined search is needed to retrieve those records.      *;
%*                                                                                               *;
%* Users must have the Permanently Delete Items or Restore Items Deleted by Others role          *;
%* privilege at the specified location to retrieve items put in the recycle bin by other users.  *;
%*                                                                                               *;
%* Syntax:                                                                                       *;
%* %LSAF_GETRECYCLEBINITEMS(LSAF_LOCATION=lsaf-location <, LSAF_NAME=lsaf-name,                  *;
%*         LSAF_DELETEDBY=lsaf-deletedby, LSAF_FROM=lsaf-from-date, LSAF_TO=lsaf-to-date,        *;
%*         LSAF_INCLUDESUBFOLDERS=lsaf-includeSubfolders, SAS_DSNAME=SAS-data-set,               *;
%*         LSAF_PRINTLIMITREACHEDWARNING = lsaf-print-limit-reached-warning>);                   *;
%*                                                                                               *;
%* @macvar _lsafRC_  The return code.                                                            *;
%* @macvar _lsafMsg_ The return message.                                                         *;
%* @macvar _lsafRBSearchLimitReached_ The search limit reached flag.                             *;
%*                                                                                               *;
%* @param lsaf_location - Optional - The location of the container in the repository that is the *;
%*            parent of the deleted items. Items from child paths are also returned.             *;
%* @param lsaf_name - Optional - The name of the item to return by the search. If set to         *;
%*            * (such as lsaf_name=*) or is omitted, items of any name are returned.             *;
%*            Wildcard notation can be used. For example, lsaf_name=*.sas returns all of the     *;
%*            items with a .sas extension.                                                       *;
%* @param lsaf_deletedby - Optional - The ID of the user who deleted the item.                   *;
%*            If omitted, set to blank, or * (such as lsaf_deletedby=*), all items that the      *;
%*            current user has the privilege to retrieve, regardless of who deleted the items,   *;
%*            are returned. If the current user lacks the privileges, only current user items    *;
%*            are returned.                                                                      *;
%* @param lsaf_from - Optional - The beginning of the date range for items deleted within a      *;
%*            specific time period. Values must be one of the formats described below. Date      *;
%*            values in formats other than the two described below result in an execution        *;
%*            failure.                                                                           *;
%*            - SAS datetime20. format (such as 02NOV2013:09:33:22) - an offset of zero hours    *;
%*              from GMT is used.                                                                *;
%*            - SAS E8601DZ25. format (such as 2013-11-02T09:33:22-04:00).                       *;
%* @param lsaf_to - Optional - The end of the date range for items deleted within a              *;
%*            specific time period. Values must be one of the formats described below. Date      *;
%*            values in formats other than the two described below result in an execution        *;
%*            failure.                                                                           *;
%*            - SAS datetime20. format (such as 02NOV2013:09:33:22) - an offset of zero hours    *;
%*              from GMT is used.                                                                *;
%*            - SAS E8601DZ25. format (such  2013-11-02T09:33:22-04:00).                         *;
%* @param lsaf_includeSubfolders - Optional -  Indicates whether recycle bin searches containers *;
%*            beyond the specified location.  Valid values:                                      *;
%*            - 0: Exclude subfolders.                                                           *;
%*            - 1 (Default): Include subfolders.                                                 *;
%* @param sas_dsname - Optional - The name of the output SAS data set to contain the metadata    *;
%*            for the items, specified as libref.dataset. The default value is                   *;
%*            WORK.LSAFGETRECYCLEBINITEMS. The data set contains a row for each item in the      *;
%*            list and columns with the following names. The data set is be sorted by path.      *;
%*            - name: The name of the item.                                                      *;
%*            - path: The path of the item.                                                      *;
%*            - type: Type of the item (such as folder or data set)                              *;
%*            - identifier: The unique identifier of the item.                                   *;
%*            - size: The size (in bytes) of the item. The size is 0 for a container.            *;
%*            - deletedBy: The user ID that deleted the item.                                    *;
%*            - deletedOn: The full date and time on which the item was deleted, represented as  *;
%*              a String.                                                                        *;
%*            - dateDeleted: The date on which the item was deleted, specified as a SAS          *;
%*              Datetime format.                                                                 *;
%*            - lastModifiedBy: The user ID that last modified the item.                         *;
%*            - lastModifiedOn: The date on which the item was last modified, represented as a   *;
%*              String.                                                                          *;
%*            - dateLastModified: The date on which the item was last modified, represented as   *;
%*              a SAS Datetime format.                                                           *;
%* @param lsaf_printLimitReachedWarning - Optional - Indicates whether to print the message      *;
%*            about the server query limit being reached as a WARNING or a NOTE. Valid values:   *;
%*            - 0: NOTE.                                                                         *;
%*            - 1 (Default): WARNING.                                                            *;
%*            This option applies only when the _lsafRBSearchLimitReached_ macro is 1.           *;
%*                                                                                               *;
%* @macrotype LSAF                                                                               *;
%* @since 1.5                                                                                    *;
%* @exposure external                                                                            *;
%*                                                                                               *;
%* @history 2017-01-24  original coding.                                                         *;
%* @history 2019-02-15  update to 2.1                                                            *;
%* @history 2019-02-20  renamed from lsaf_getrbitemsbyquery to lsaf_getrecyclebinitems           *;
%* @history 2019-09-10  added includesubfolders parameter                                        *;
%* @history 2019-11-01  update to handle a blank SAS output file.                                *;
%* @history 2020-07-13  update for refactor of java service hierarchy.                           *;
%*                                                                                               *;

%macro lsaf_getrecyclebinitems(
  lsaf_location=,
  lsaf_name=,
  lsaf_deletedby=,
  lsaf_from=,
  lsaf_to=,
  lsaf_includesubfolders=,
  sas_dsname=work.lsafgetrecyclebinitems,
  lsaf_printLimitReachedWarning=1
  ) /des='Get a list of the items in the recycle bin based on given criteria.';
  
  %if "&sas_dsname" EQ "" %then %let sas_dsname=work.lsafgetrecyclebinitems;

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafRC_
          _lsaf_PrintMessage_
          _lsafRBSearchLimitReached_
  ;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %local _lsafRowCount_
         notesOption
  ;

  %let _lsafRowCount_=0;

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data &sas_dsname (label="Created by &SYSMACRONAME"
                    keep=path name type identifier size deleted deletedBy dateDeleted lastModified lastModifiedBy dateLastModified sortPath);
    attrib returnCode         length=8
           message            length=$200
           logMsgPrefix       length=$70
           logMsgPrimary      length=$200
           logMsg             length=$300
           rowCount           length=8
           limitReached       length=8

           identifier         length=$512                     label="Unique Identifier"
           path               length=$2048                    label="Item Path"
           name               length=$256                     label="Item Name"
           type               length=$32                      label="Item Type"
           size               length=8                        label="Size In Bytes"
           deletedBy          length=$64                      label="Userid that deleted the item"
           deleted            length=$64                      label="Date the item was deleted"
           dateDeleted        length=8      format=datetime.  label="Date the item was deleted (SAS DateTime Format)"
           lastModifiedBy     length=$64                      label="Userid that last modified the item content"
           lastModified       length=$64                      label="Date the item's content was last modified"
           dateLastModified   length=8      format=datetime.  label="Date the item's content was last modified (SAS DateTime Format)"
           tempDateDeleted    length=$64                      label="USED FOR FORMATTING - will be dropped"
           tempDateModified   length=$64                      label="USED FOR FORMATTING - will be dropped"
           sortpath           length=$2048                    label="NEEDED FOR SORTING - will be dropped"
    ;
    
    rowCount=0;

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/lsaf/macro/content/repository/recyclebin/RecycleBinService") ;
    declare javaobj results("com/sas/lsaf/macro/content/repository/recyclebin/SasRecycleBinResult");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticVoidMethod("getRecycleBinItemsByQuery", "%bquote(&lsaf_location)", "%bquote(&lsaf_name)", 
                             "%bquote(&lsaf_deletedby)", "%bquote(&lsaf_from)", "%bquote(&lsaf_to)", 
                             "%bquote(&lsaf_includesubfolders)",  results);

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);
    results.callBooleanMethod("isLimitReached", limitReached);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
    if (returnCode ne 0) then
    do;
      logMsgPrimary="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Could not retrieve list of items in the recycle bin.";
    end;
    else
    do;
      results.callIntMethod("getSize", rowCount);
      if limitReached then
      do;
        %lsaf_printwarningmessage(lsaf_printmessage=&lsaf_printlimitreachedwarning);
        logMsgPrimary="&_lsaf_PrintMessage_";
        logMsg=" Recycle bin items were found matching the given criteria.";
      end;
      else 
      do;
        logMsgPrimary=logMsgPrefix;
        if (rowCount=0) 
          then logMsg = " No recycle bin items were found matching the given criteria.";
          else logMsg = " Recycle bin items were found matching the given criteria.";
      end;

      do row=0 to rowCount - 1;
        results.callStringMethod("getPath", row, path);
        sortPath = kstrip( kupcase( ktranslate(path,'1','_') ) );
        results.callStringMethod("getName", row, name);
        results.callStringMethod("getType", row, type);
        results.callStringMethod("getId", row, identifier);
        results.callLongMethod("getItemSize", row, size);
        results.callStringMethod("getDeletedBy", row, deletedBy);
        results.callStringMethod("getDeleted", row, deleted);
        results.callStringMethod("getDeletedSasFormat", row, tempDateDeleted);
        dateDeleted=input(tempDateDeleted, datetime.);
        results.callStringMethod("getLastModifiedBy", row, lastModifiedBy);
        results.callStringMethod("getLastModified", row, lastModified);
        results.callStringMethod("getLastModifiedSasFormat", row, tempDateModified);
        dateLastModified=input(tempDateModified, datetime.);
        output;
      end;
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrimary " " logMsg;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    put logMsgPrefix " _lsafMsg_= " message ;
    put logMsgPrefix " _lsafRC_= " returnCode ;
    put logMsgPrefix " _lsafRBSearchLimitReached_= " limitReached;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);
    call symputx("_lsafRowCount_", rowCount);
    call symputx("_lsafRBSearchLimitReached_", limitReached);

  run;

  %***********************************;
  %* Sort the data set               *;
  %***********************************;
  %let notesOption=%sysfunc(getoption(notes));
  options nonotes;

  %if (&_lsafRowCount_ > 0) %then
  %do;
     proc sort data=&sas_dsname out=&sas_dsname (drop=sortpath);
       by sortpath;
     run;
  %end;
  %else
  %do;
     proc sql;
       alter table &sas_dsname
       drop sortpath;
     quit;
  %end;
  options &notesOption;

%mend lsaf_getrecyclebinitems;
