%* lsaf_getsessionlistsize                                                                          *;
%*                                                                                                  *;
%* Returns the list size of a session. The macro variable _lsafSessionListSize_ contains the        *;
%* value.                                                                                           *;
%*                                                                                                  *;
%* Syntax:                                                                                          *;
%* %LSAF_GETSESSIONLISTSIZE(LSAF_SESSIONID=lsaf-sessionid, LSAF_FORMATVALUE=lsaf-formatvalue);      *;
%*                                                                                                  *;
%* @macvar _lsafRC_  The return code.                                                               *;
%* @macvar _lsafMsg_ The return message.                                                            *;
%* @macvar _lsafSessionListSize_ The size of the session list.                                      *;
%*                                                                                                  *;
%* @param lsaf_sessionid - Required - The identifier of the session in the repository. This         *;
%*            can be retrieved by calling the lsaf_getallsessions, lsaf_getmysessions, or           *;
%*            lsaf_getusersessions macros.                                                          *;
%* @param lsaf_formatvalue - Optional - Indicates whether to format the value. Non-formatted        *;
%*            returns the raw numeric value in bytes. Formatted output sets the value to the        *;
%*            character representation with the appropriate label.                                  *;
%*            For example, the non-formatted value 1024 would be 1 KB, if formatted.                *;
%*            Valid values:                                                                         *; 
%*            - 0 (Default): Non-formatted value.                                                   *;
%*            - 1: Formatted value.                                                                 *;                                                            *;
%*                                                                                                  *;
%* @macrotype LSAF                                                                                  *;
%* @since 2.3                                                                                       *;
%* @exposure external                                                                               *;
%*                                                                                                  *;
%* @history 2020-01-21  initial coding                                                              *;
%*                                                                                                  *;

%macro lsaf_getsessionlistsize(
  lsaf_sessionid=,
  lsaf_formatvalue=0
  ) /des='Get the session list file size.';

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafOwner_
          _lsafRC_
          _lsafSessionListSize_
  ;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data _null_;
    attrib returnCode      length=8;
    attrib message         length=$200;
    attrib logMsgPrefix    length=$70;
    attrib logMsgPrimary   length=$70;
    attrib logMsg          length=$300;
    attrib listsize        length=$64;

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/lsaf/macro/execution/session/SessionService") ;
    declare javaobj results("com/sas/lsaf/macro/results/Result");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticStringMethod("getSessionListSize", "%bquote(&lsaf_sessionid)",  "%bquote(&lsaf_formatvalue)", results, listsize);

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
    if (returnCode ne 0) then
    do;
      logMsgPrimary="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Cannot retrieve the session list file size for session &lsaf_sessionid.." ;
    end;
    else
    do;
      logMsgPrimary=logMsgPrefix;
      logMsg=" The session list file size for session &lsaf_sessionid is " || listsize ;
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrimary " " logMsg;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    put logMsgPrefix " _lsafMsg_= " message ;
    put logMsgPrefix " _lsafRC_= " returnCode ;
    put logMsgPrefix " _lsafSessionListSize_= " listsize;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);
    call symputx("_lsafSessionListSize_", listsize) ;

  run;

%mend lsaf_getsessionlistsize;
