%* lsaf_getsessionsbyquery                                                                         *;
%*                                                                                                 *;
%* Gets the metadata for all of the sessions that are based on a user-specified query. The         *;
%* metadata is stored in a SAS data set. The privilege PRIVILEGE_MANAGE_ALL_SESSIONS is required   *;
%* to retrieve sessions that are not owned by the user who runs the macro.                         *;
%*                                                                                                 *;
%* Syntax:                                                                                         *;
%* %LSAF_GETSESSIONSBYQUERY(<LSAF_USERID=lsaf-userid, LSAF_ENGINE=lsaf-engine, LSAF_TYPE=lsaf-type,*;
%*       LSAF_STATE=lsaf-state, LSAF_SUBMISSIONLABEL=lsaf-submissionlabel,                         *;
%*       LSAF_STARTEDFROMDATE=lsaf-startedfromdate, LSAF_STARTEDTODATE=lsaf-startedtodate,         *;
%*       LSAF_ENDEDFROMDATE=lsaf-endedfromdate, LSAF_ENDEDTODATE=lsaf-endedtodate,                 *;
%*       LSAF_SESSIONLIMIT=lsaf-sessionlimit, SAS_DSNAME=SAS-data-set>);                           *;
%*                                                                                                 *;
%* @macvar _lsafRC_  The return code.                                                              *;
%* @macvar _lsafMsg_ The return message.                                                           *;
%*                                                                                                 *;
%* @param lsaf_userid - Optional - The user identifier to return from the search. If a value is    *;
%*            not specified, all users are returned.                                               *;
%* @param lsaf_engine - Optional - The session engine to return from the search. If a value is     *;
%*            not specified, all engines are returned. Valid values:                               *;
%*            - SA: The session uses a SAS engine.                                                 *;
%*            - R: The session uses an R engine.                                                   *;
%*            - COMPOSITE: The session of a job that uses both a SAS and an R engine.              *; 
%* @param lsaf_type - Optional - The session type to return from the search. If a value is not     *;
%*            specified, all types are returned. Valid values:                                     *;
%*            - INTERACTIVE: Initiated by launching the session editor in the application.         *;
%*            - TEST_JOB: Initiated by testing a job in the workspace.                             *;
%*            - WORKSPACE_JOB: Initiated by running a job in the workspace.                        *;
%*            - PUBLISHED_JOB: Initiated by running a job in the repository.                       *;
%*            - SYSTEM: Initiated by the system internally to perform a function.                  *;
%*            - UNKNOWN: Unknown.                                                                  *;
%* @param lsaf_state - Optional - The session state to return from the search. If a value is not   *;
%*            specified, all session states are returned. Valid values:                            *;
%*            - IDLE: The session is available for code execution.                                 *;
%*            - ACTIVE: The code is currently executing in the session.                            *;
%*            - ENDED: Ended and not available for code execution. Session artifacts exist.        *;
%*            - ENDING: End session has been called and is in the process of ending.               *;
%*            - BROKEN: Invalid state. Not available for code execution. Session artifacts might   *;
%*              still exist.                                                                       *;
%* @param lsaf_submissionlabel - Optional - The name that is associated with a submission. If a    *;
%*            value is not provided, all are returned.                                             *;
%* @param lsaf_startedfromdate - Optional - The started from date range for sessions within a      *;
%*            specific time period. Values must be one of the formats below. Date values in        *;
%*            formats other than the two described below result in an execution failure.           *;
%*            - SAS datetime20. format (such as 02NOV2013:09:33:22): An offset of zero hours       *;
%*              from GMT is used.                                                                  *;
%*            - SAS E8601DZ25. format (such as 2013-11-02T09:33:22-04:00).                         *;
%*            If a value is not specified, all dates are returned.                                 *;
%* @param lsaf_startedtodate - Optional - The started to date range for sessions within a specific *;
%*            time period. Values must be one of the formats below. Date values in formats         *;
%*            other than the two described below result in an execution failure.                   *;
%*            - SAS datetime20. format (such as 02NOV2013:09:33:22): An offset of zero hours       *;
%*              from GMT is used.                                                                  *;
%*            - SAS E8601DZ25. format (such as 2013-11-02T09:33:22-04:00).                         *;
%*            If a value is not specified, all dates are returned.                                 *;
%* @param lsaf_endedfromdate - Optional - The ended from date range for sessions within a specific *;
%*            time period. Values must be one of the formats below. Date values in formats other   *;
%*            than the two described below result in an execution failure.                         *;
%*            - SAS datetime20. format (such as 02NOV2013:09:33:22): An offset of zero hours       *;
%*              from GMT is used.                                                                  *;
%*            - SAS E8601DZ25. format (such as 2013-11-02T09:33:22-04:00).                         *;
%*            If a value is not specified, all dates are returned.                                 *;
%* @param lsaf_endedtodate - Optional - The ended to date range for sessions within a specific     *;
%*            time period. Values must be one of the formats below. Date values in formats other   *;
%*            than the two described below result in an execution failure.                         *;
%*            - SAS datetime20. format (such as 02NOV2013:09:33:22): An offset of zero hours       *;
%*              from GMT is used.                                                                  *;
%*            - SAS E8601DZ25. format (such as 2013-11-02T09:33:22-04:00).                         *;
%*            If a value is not specified, all dates are returned.                                 *;
%* @param lsaf_sessionlimit - Optional - The maximum number of sessions to return at one time.     *;
%*            If a value is not specified, all sessions are returned.                              *;
%* @param sas_dsname - Optional - The name of the output SAS data set to contain the metadata      *;
%*            for all the sessions, specified as libref.dataset. The default value is              *;
%*            WORK.LSAFGETSESSIONSBYQUERY. The data set contains a row for each session and        *;
%*            columns with the following names:                                                    *;
%*            - sessionId: The unique identifier for the session.                                  *;
%*            - dateStarted: The date on which the session started.                                *;
%*            - dateStartedSASFormat: The date on which the session started, in SAS DATETIME       *;
%*              format.                                                                            *;
%*            - dateEnded: The date on which the session ended.                                    *;
%*            - dateEndededSASFormat: The date on which the session started, in SAS DATETIME       *;
%*              format.                                                                            *;
%*            - sessionType: The session type.                                                     *;
%*            - sessionOwner: The session owner.                                                   *;
%*            - sessionStatus: The session status.                                                 *;
%*            - submissionLabel: The label of the session, if specified.                           *;
%*            - clientId: The incremental number assigned to session.                              *;
%*                                                                                                 *;
%* @macrotype LSAF                                                                                 *;
%* @since 2.4                                                                                      *;
%* @exposure external                                                                              *;
%*                                                                                                 *;
%* @history 2020-11-17  initial coding                                                             *;
%*                                                                                                 *;

%macro lsaf_getsessionsbyquery(
  lsaf_userid=, 
  lsaf_engine=, 
  lsaf_type=, 
  lsaf_state=, 
  lsaf_submissionlabel=, 
  lsaf_startedfromdate=, 
  lsaf_startedtodate=, 
  lsaf_endedfromdate=, 
  lsaf_endedtodate=, 
  lsaf_sessionlimit=,
  sas_dsname=work.lsafgetsessionsbyquery
  ) /des='Get a list of all the sessions in the system as defined by a user query.';
  
  %if "&sas_dsname" EQ "" %then %let sas_dsname=work.lsafgetsessionsbyquery;

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafRC_
  ;

  %local _lsafRowCount_;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);
  %let _lsafRowCount_=0;

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data &sas_dsname (label="Created by &SYSMACRONAME"
                    keep= sessionId dateStarted dateStartedSASFormat dateEnded dateEndedSASFormat sessionType sessionState 
                          sessionOwner submissionLabel clientId
    );

    attrib returnCode              length=8;
    attrib message                 length=$200;
    attrib logMsgPrefix            length=$70;
    attrib logMsgPrimary           length=$70;
    attrib logMsg                  length=$300;
    attrib rowCount                length=8;

    attrib sessionOwner            length=$32    label="Session Owner";
    attrib dateStartedSASFormat                  label="Date Session Started (SAS DateTime Format)";
    attrib dateStarted             length=$64    label="Date Session Started";
    attrib dateEndedSASFormat                    label="Date Session Ended (SAS DateTime Format)";
    attrib dateEnded               length=$64    label="Date Session Ended";
    attrib sessionId               length=$64    label="Ssession ID";
    attrib sessionType             length=$32    label="Session Type";
    attrib sessionState            length=$32    label="Session State";
    attrib submissionLabel         length=$1024  label="Submission Label";
    attrib clientId                length=8      label="Incremental Number Assigned to Session";
    attrib limitReached            length=3;
    ;

    attrib dateStartedSasFormatTmp length=$64    label="SAS Format: USED FOR FORMATTING - will be dropped";
    attrib dateEndedSasFormatTmp   length=$64    label="SAS Format: USED FOR FORMATTING - will be dropped";

    format dateStartedSASFormat dateEndedSASFormat datetime.;

    rowCount=0;
    limitReached=0;

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/lsaf/macro/execution/session/SessionService") ;
    declare javaobj results("com/sas/lsaf/macro/execution/session/SasSessionsResult");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;   
    srv.callStaticBooleanMethod("search", "%bquote(&lsaf_userid)", "%bquote(&lsaf_engine)", "%bquote(&lsaf_type)", "%bquote(&lsaf_state)", 
      "%bquote(&lsaf_submissionlabel)", "%bquote(&lsaf_startedfromdate)", "%bquote(&lsaf_startedtodate)", "%bquote(&lsaf_endedfromdate)", 
      "%bquote(&lsaf_endedtodate)", "%bquote(&lsaf_sessionlimit)", results, limitReached);

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
    if (returnCode ne 0) then
    do;
      logMsgPrimary="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Cannot retrieve list of queried sessions.";
    end;
    else
    do;
      logMsgPrimary=logMsgPrefix;
      logMsg=" List of all the queried sessions was returned.";
      results.callIntMethod("getSize", rowCount);
      do row=0 to rowCount - 1;
        results.callStringMethod("getId", row, sessionId);
        results.callStringMethod("getDateStarted", row, dateStarted);
        results.callStringMethod("getDateStartedSasFormat", row, dateStartedSasFormatTmp);
        results.callStringMethod("getDateEnded", row, dateEnded);
        results.callStringMethod("getDateEndedSasFormat", row, dateEndedSasFormatTmp);
        results.callStringMethod("getType", row, sessionType);
        results.callStringMethod("getState", row, sessionState);
        results.callStringMethod("getOwner", row, sessionOwner);
        results.callStringMethod("getSubmissionLabel", row, submissionLabel);
        results.callIntMethod("getClientId", row, clientId);
        
        dateStartedSasFormat = input(dateStartedSasFormatTmp, datetime.);
        dateEndedSasFormat = input(dateEndedSasFormatTmp, datetime.);

        output;
      end;
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrimary " " logMsg;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    if (rowCount=0) 
      then put logMsgPrefix " No queried sessions were found.";
      else put logMsgPrefix " " rowCount " records written to data set &sas_dsname";
    put logMsgPrefix " _lsafMsg_= " message ;
    put logMsgPrefix " _lsafRC_= " returnCode ;
    put logMsgPrefix " Session Limit Reached= " limitReached;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);
    call symputx("_lsafRowCount_", rowCount);

  run;

  %let notesOption=%sysfunc(getoption(notes));
  options nonotes;

  %***********************************;
  %* Sort the data set               *;
  %***********************************;
  %if (&_lsafRowCount_ > 0) %then
  %do;
    proc sort data=&sas_dsname;
      by sessionOwner dateStartedSasFormat dateEndedSasFormat;
    run;
  %end;

  options &notesOption;

%mend lsaf_getsessionsbyquery;
