%* lsaf_getstandardsbystatus                                                                        *;
%*                                                                                                  *;
%* Gets the metadata for all of the data standards by the specified status. The metadata is stored  *;
%* in a SAS data set.                                                                               *;
%*                                                                                                  *;
%* Syntax:                                                                                          *;
%* %LSAF_GETSTANDARDSBYSTATUS(LSAF_STATUS=lsaf-status, <SAS_DSNAME=WORK.LSAFGETSTANDARDSBYSTATUS>); *;
%*                                                                                                  *;
%* @macvar _lsafRC_  The return code.                                                               *;
%* @macvar _lsafMsg_ The return message.                                                            *;
%*                                                                                                  *;
%* @param lsaf_status - Required - The status of the data standards to return.                      *;
%*            Valid values: PUBLISHED, INPROGRESS, and CHECKEDOUT. The default is PUBLISHED.        *;
%*                                                                                                  *;
%* @param sas_dsname - Optional - The name of the output SAS data set to contain the metadata       *;
%*            for the data standards, specified as libref.dataset. The default value is             *;
%*            WORK.LSAFGETSTANDARDSBYSTATUS. The data set contains a row for each data standard,    *;
%*            sorted by the standard ID, and columns with the following names:                      *;
%*            - id: The unique standard identifier.                                                 *;
%*            - name: The data standard name.                                                       *;
%*            - baseStandardType: The base data standard type.                                      *;
%*            - baseStandardName: The base data standard name.                                      *;
%*            - baseStandardVersion: The base data standard version.                                *;
%*            - modelId: The model identifier.                                                      *;
%*            - description: The data standard description.                                         *;
%*            - isCheckedOut: Indicates whether the data standard is checked out. Valid values:     *;
%*              0: no, 1: yes.                                                                      *;
%*            - checkedOutBy: The identifier of the user who has the data standard checked out, if  *;
%*              it is checked out. This should always be the identifier of the current user.        *;
%*            - checkedOutDate: The date on which the data standard was checked out, if it is       *;
%*              checked out.                                                                        *;
%*            - checkedOutDateSASFormat: The date on which the data standard was checked out        *;
%*              (in SAS format), if it is checked out.                                              *;
%*            - createdDate: The date on which the data standard was created.                       *;
%*            - createdDateSASFormat: The date on which the data standard was created (in SAS       *;
%*              format).                                                                            *;
%*            - createdBy: The identifier of the user who created the data standard.                *;
%*            - state: The state of the data standard.                                              *;
%*            - publishedState: The published state of the data standard.                           *;
%*            - currentVersion: The current published version of the data standard.                 *;
%*            - associatedStudiesCount: The number of studies that are associated with the data     *;
%*              standard.                                                                           *;
%*            - lastModifiedDate: The date on which the data standard was last modified.            *;
%*            - lastModifiedDateSASFormat: The date on which the data standard was last modified    *;
%*              (in SAS format).                                                                    *;
%*            - lastModifiedBy: The identifier of the user who last modified the data standard.     *;
%*            - standardStatus: The status of the data standard.                                    *;
%*                                                                                                  *;
%* @macrotype LSAF                                                                                  *;
%* @since 2.2                                                                                       *;
%* @exposure external                                                                               *;
%*                                                                                                  *;
%* @history 2019-06-07  initial coding                                                              *;
%* @history 2019-11-01  update to handle a blank SAS output file.                                   *;
%*                                                                                                  *;

%macro lsaf_getstandardsbystatus(
  lsaf_status=,
  sas_dsname=work.lsafgetstandardsbystatus
  ) /des='Get a list of all standards in the system by status.';
  
  %if "&sas_dsname" EQ "" %then %let sas_dsname=work.lsafgetstandardsbystatus;

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafRC_
  ;

  %local _lsafRowCount_;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);
  %let _lsafRowCount_=0;

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data &sas_dsname (label="Created by &SYSMACRONAME"
                    keep= id name baseStandardType baseStandardName baseStandardVersion modelId description
                          isCheckedOut checkedOutBy checkedOutDate checkedOutDateSASFormat createdDate createdDateSASFormat
                          createdBy state publishedState currentVersion associatedStudiesCount lastModifiedDate 
                          lastModifiedDateSASFormat lastModifiedBy standardStatus
  );

    attrib returnCode                length=8;
    attrib message                   length=$200;
    attrib logMsgPrefix              length=$70;
    attrib logMsgPrimary             length=$70;
    attrib logMsg                    length=$300;
    attrib rowCount                  length=8;

    attrib id                        length=$64    label="ID";
    attrib name                      length=$128   label="Name";
    attrib baseStandardType          length=$64    label="Base Standard Type";
    attrib baseStandardName          length=$64    label="Base Standard Name";   
    attrib baseStandardVersion       length=$128   label="Base Standard Version";
    attrib modelId                   length=$128   label="Model ID";
    attrib description               length=$64    label="Description";
    attrib isCheckedOut              length=8      label="Is Checked Out";
    attrib checkedOutBy              length=$64    label="Checked Out By";    
    attrib checkedOutDate            length=$64    label="Checked Out Date";
    attrib checkedOutDateSASFormat   length=$64    label="Checked Out Date (SAS Format)";
    attrib createdDate               length=$64    label="Date Created";
    attrib createdDateSASFormat      length=$64    label="Date Created (SAS Format)";
    attrib createdBy                 Length=$64    Label="Created by";
    attrib state                     length=$64    label="State";
    attrib publishedState            length=$64    label="Published State";
    attrib currentVersion            length=$32    label="Current Version";
    attrib associatedStudiesCount    length=8      label="Associated Studies Count";
    attrib lastModifiedDate          length=$64    label="Date Last Modified";
    attrib lastModifiedDateSASFormat length=$64    label="Date Last Modified (SAS Format)";
    attrib lastModifiedBy            length=$64    label="Last Modified By";    
    attrib standardStatus            length=$64    label="Standard Status";

    rowCount=0;

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/lsaf/macro/clinical/standards/StandardsService");
    declare javaobj results("com/sas/lsaf/macro/clinical/standards/SasStandardsResult");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticVoidMethod("getStandardsByStatus", "%bquote(&lsaf_status)", results);

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
    if (returnCode ne 0) then
    do;
      logMsgPrimary="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Cannot retrieve list of standards.";
    end;
    else
    do;
      logMsgPrimary=logMsgPrefix;
      logMsg=" List of standards was returned.";
      results.callIntMethod("getSize", rowCount);
      do row=0 to rowCount - 1;
        results.callStringMethod("getId", row, id);
        results.callStringMethod("getName", row, name);
        results.callStringMethod("getBaseStandardType", row, BaseStandardType);
        results.callStringMethod("getBaseStandardName", row, BaseStandardName);
        results.callStringMethod("getBaseStandardVersion", row, BaseStandardVersion);
        results.callStringMethod("getModelId", row, modelId);
        results.callStringMethod("getDescription", row, description);
        results.callIntMethod("isCheckedOut", row, isCheckedOut);
        results.callStringMethod("getCheckedOutBy", row, checkedOutBy);
        results.callStringMethod("getCheckedOutDate", row, checkedOutDate);
        results.callStringMethod("getCheckedOutDataSasFormat", row, checkedOutDateSASFormat);
        results.callStringMethod("getCreated", row, createdDate);
        results.callStringMethod("getCreatedSasFormat", row, createdDateSasFormat);
        results.callStringMethod("getCreatedBy", row, createdBy);
        results.callStringMethod("getState", row, state);
        results.callStringMethod("getPublishedState", row, publishedState);
        results.callStringMethod("getCurrentVersion", row, currentVersion);
        results.callLongMethod("getAssociatedStudiesCount", row, associatedStudiesCount);
        results.callStringMethod("getLastModified", row, lastModifiedDate);
        results.callStringMethod("getLastModifiedSasFormat", row, lastModifiedDateSASFormat);
        results.callStringMethod("getLastModifiedBy", row, lastModifiedBy);
        results.callStringMethod("getStandardStatus", row, standardStatus);
        output;
      end;
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrimary " " logMsg;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    if (rowCount<=0) 
      then put logMsgPrefix " No standards were found for status &lsaf_status..";
      else put logMsgPrefix " " rowCount " records written to data set &sas_dsname";
    put logMsgPrefix " _lsafMsg_= " message ;
    put logMsgPrefix " _lsafRC_= " returnCode ;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);
    call symputx("_lsafRowCount_", rowCount);

  run;

  %let notesOption=%sysfunc(getoption(notes));
  options nonotes;
  %***********************************;
  %* Sort the data set               *;
  %***********************************;
  %if (&_lsafRowCount_ > 0) %then
  %do;
    proc sort data=&sas_dsname;
      by name;
    run;
  %end;

  options &notesOption;

%mend lsaf_getstandardsbystatus;