%* lsaf_getuserprivileges                                                                        *;
%*                                                                                               *;
%* Gets the metadata for all of the global privileges that are assigned to a user account in     *;
%* the system. The metadata is stored in a SAS data set.                                         *;
%*                                                                                               *;
%* Syntax:                                                                                       *;
%* %LSAF_GETUSERPRIVILEGES(LSAF_USERID=lsaf-userid <, SAS_DSNAME=SAS-data-set>);                 *;
%*                                                                                               *;
%* @macvar _lsafRC_  The return code.                                                            *;
%* @macvar _lsafMsg_ The return message.                                                         *;
%*                                                                                               *;
%* @param lsaf_userid - Required - The user account.                                             *;
%* @param sas_dsname - Optional - The name of the output SAS data set to contain the metadata    *;
%*            for the global privileges, specified as libref.dataset. The default value is       *;
%*            WORK.LSAFGETUSERPRIVILEGES. The data set contains a row for each global privilege  *;
%*            that is assigned to the user account and the columns with the following names.     *;
%*            The data set is sorted by privilegeId.                                             *;
%*            - userId: The user ID of the user.                                                 *;
%*            - privilegeId: The identifier of the privilege.                                    *;
%*            - privilegeName: The name of the privilege.                                        *;
%*                                                                                               *;
%* @macrotype LSAF                                                                               *;
%* @since 1.3                                                                                    *;
%* @exposure external                                                                            *;
%*                                                                                               *;
%* @history 2013-11-01 initial coding                                                            *;
%* @history 2016-08-26  rebrand as LSAF                                                          *;
%* @history 2019-11-01  update to handle a blank SAS output file.                                *;
%* @history 2020-07-13  update for refactor of java service hierarchy.                           *;
%*                                                                                               *;

%macro lsaf_getuserprivileges(
  lsaf_userid=,
  sas_dsname=work.lsafGetUserPrivileges
  ) /des='Get a list of the global privileges assigned to a user.';
  
  %if "&sas_dsname" EQ "" %then %let sas_dsname=work.lsafGetUserPrivileges;

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafRC_
  ;

  %local _lsafRowCount_;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);
  %let _lsafRowCount_=0;

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data &sas_dsname (label="Created by &SYSMACRONAME"
                    keep=userId privilegeId privilegeName);
    attrib returnCode      length=8;
    attrib message         length=$200;
    attrib logMsgPrefix    length=$70;
    attrib logMsgPrimary   length=$70;
    attrib logMsg          length=$300;
    attrib rowCount        length=8;

    attrib userId          length=$64      label="User ID";
    attrib privilegeId     length=$64      label="Privilege Identifier";
    attrib privilegeName   length=$128     label="Privilege Name";

    rowCount=0;

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/lsaf/macro/security/privilege/global/GlobalPrivilegeService");
    declare javaobj results("com/sas/lsaf/macro/security/privilege/SasPrivilegesResult");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticVoidMethod("getGrantedPrivileges", "%bquote(&lsaf_userid)", results);

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
    if (returnCode ne 0) then
    do;
      logMsgPrimary="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Cannot retrieve assigned global privileges for user &lsaf_userid.." ;
    end;
    else
    do;
      logMsgPrimary=logMsgPrefix;
      logMsg=" Global privileges were retrieved for user &lsaf_userid" ;
      results.callIntMethod("getSize", rowCount);
      do row=0 to rowCount - 1;
        userId="&lsaf_userid";
        results.callStringMethod("getId", row, privilegeId);
        results.callStringMethod("getName", row, privilegeName);
        output;
      end;
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrimary " " logMsg;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    if (rowCount=0) 
      then put logMsgPrefix " No assigned global privileges were found.";
      else put logMsgPrefix " " rowCount " records written to data set &sas_dsname";
    put logMsgPrefix " _lsafMsg_= " message ;
    put logMsgPrefix " _lsafRC_= " returnCode ;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);
    call symputx("_lsafRowCount_", rowCount);

  run;

  %***********************************;
  %* Sort by privilege id            *;
  %***********************************;
  %if (&_lsafRowCount_ > 0) %then
  %do;
    %let notesOption=%sysfunc(getoption(notes));
    options nonotes;
    proc sort data=&sas_dsname;
      by privilegeId;
    run;
    options &notesOption;
  %end;

%mend lsaf_getuserprivileges;
