%* lsaf_getusersessions                                                                            *;
%*                                                                                                 *;
%* Gets the metadata for all of the user sessions. The metadata is stored in a SAS data set.       *;
%*                                                                                                 *;
%* Syntax:                                                                                         *;
%* %LSAF_GETUSERSESSIONS(LSAF_USERID=lsaf-userid <,SAS_DSNAME=SAS-data-set>);                      *;
%*                                                                                                 *;
%* NOTE: This macro will be deprecated in a future release and is replaced by                      *;
%*       lsaf_getsessionsbyquery.                                                                  *;
%*                                                                                                 *;
%* @deprecated Replaced by lsaf_getsessionsbyquery in this release, and will be deprecated in a    *; 
%*       future release.                                                                           *;
%*                                                                                                 *;
%* @macvar _lsafRC_  The return code.                                                              *;
%* @macvar _lsafMsg_ The return message.                                                           *;
%*                                                                                                 *;
%* @param lsaf_userid - Required - The user identifier of the sessions to retrieve.                *;
%*                                                                                                 *;
%* @param sas_dsname - Optional - The name of the output SAS data set to contain the metadata      *;
%*            for the user sessions, specified as libref.dataset.  The default value is            *;
%*            WORK.LSAFGETUSERSESSIONS.  The data set contains a row for each user session         *;
%*            and columns with the following names:                                                *;
%*            - sessionId: The unique identifier for the session.                                  *;
%*            - dateStarted: The date on which the session started.                                *;
%*            - dateStartedSASFormat: The date on which the session started, in SAS DATETIME       *;
%*              format.                                                                            *;
%*            - dateEnded: The date on which the session ended.                                    *;
%*            - dateEndededSASFormat: The date on which the session started, in SAS DATETIME       *;
%*              format.                                                                            *;
%*            - sessionType: The session type.                                                     *;
%*            - sessionOwner: The session owner.                                                   *;
%*            - sessionStatus: The session status.                                                 *;
%*            - submissionLabel: The label of the session, if present.                             *;
%*            - clientId: The client identifier in which the session resides.                      *;
%*                                                                                                 *;
%* @macrotype LSAF                                                                                 *;
%* @since 2.3                                                                                      *;
%* @exposure external                                                                              *;
%*                                                                                                 *;
%* @history 2019-11-25  initial coding                                                             *;
%* @history 2020-11-23  deprecation message added in header                                        *;
%*                                                                                                 *;

%macro lsaf_getusersessions(
  lsaf_userid=,
  sas_dsname=work.lsafgetusersessions
  ) /des='Get a list of all user sessions defined in the system.';
  
  %if "&sas_dsname" EQ "" %then %let sas_dsname=work.lsafgetusersessions;

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafRC_
  ;

  %local _lsafRowCount_;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);
  %let _lsafRowCount_=0;

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data &sas_dsname (label="Created by &SYSMACRONAME"
                    keep= sessionId dateStarted dateStartedSASFormat dateEnded dateEndedSASFormat sessionType sessionState 
                          sessionOwner submissionLabel clientId
    );

    attrib returnCode              length=8;
    attrib message                 length=$200;
    attrib logMsgPrefix            length=$70;
    attrib logMsgPrimary           length=$70;
    attrib logMsg                  length=$300;
    attrib rowCount                length=8;

    attrib sessionOwner            length=$32    label="Session Owner";
    attrib dateStartedSASFormat                  label="Date Session Started (SAS DateTime Format)";
    attrib dateStarted             length=$64    label="Date Session Started";
    attrib dateEndedSASFormat                    label="Date Session Ended (SAS DateTime Format)";
    attrib dateEnded               length=$64    label="Date Session Ended";
    attrib sessionId               length=$64    label="SAS Session ID";
    attrib sessionType             length=$32    label="Session Type";
    attrib sessionState            length=$32    label="Session State";
    attrib submissionLabel         length=$1024  label="Submission Label";
    attrib clientId                length=8      label="Client ID";
    ;

    attrib dateStartedSasFormatTmp length=$64    label="SAS Format: USED FOR FORMATTING - will be dropped";
    attrib dateEndedSasFormatTmp   length=$64    label="SAS Format: USED FOR FORMATTING - will be dropped";
 

    format dateStartedSASFormat dateEndedSASFormat datetime.;

    rowCount=0;

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/lsaf/macro/execution/session/SessionService") ;
    declare javaobj results("com/sas/lsaf/macro/execution/session/SasSessionsResult");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticVoidMethod("getSessionsByUser", "%bquote(&lsaf_userid)", results);

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
    if (returnCode ne 0) then
    do;
      logMsgPrimary="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Cannot retrieve list of sessions for user &lsaf_userid..";
    end;
    else
    do;
      logMsgPrimary=logMsgPrefix;
      logMsg=" List of all user sessions was returned.";
      results.callIntMethod("getSize", rowCount);
      do row=0 to rowCount - 1;
        results.callStringMethod("getId", row, sessionId);
        results.callStringMethod("getDateStarted", row, dateStarted);
        results.callStringMethod("getDateStartedSasFormat", row, dateStartedSasFormatTmp);
        results.callStringMethod("getDateEnded", row, dateEnded);
        results.callStringMethod("getDateEndedSasFormat", row, dateEndedSasFormatTmp);
        results.callStringMethod("getType", row, sessionType);
        results.callStringMethod("getState", row, sessionState);
        results.callStringMethod("getOwner", row, sessionOwner);
        results.callStringMethod("getSubmissionLabel", row, submissionLabel);
        results.callIntMethod("getClientId", row, clientId);
        
        dateStartedSasFormat = input(dateStartedSasFormatTmp, datetime.);
        dateEndedSasFormat = input(dateEndedSasFormatTmp, datetime.);

        output;
      end;
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put "NOTE: ***************************************************************************************";
    put "NOTE: The &SYSMACRONAME. macro will be deprecated in a future release of the SAS Life        ";
    put "NOTE: Science Analytics Framework and replaced with the lsaf_getsessionsbyquery macro.       ";
    put "NOTE: ***************************************************************************************";
    put;
    put logMsgPrimary " " logMsg;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    if (rowCount=0) 
      then put logMsgPrefix " No user sessions were found.";
      else put logMsgPrefix " " rowCount " records written to data set &sas_dsname";
    put logMsgPrefix " _lsafMsg_= " message ;
    put logMsgPrefix " _lsafRC_= " returnCode ;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);
    call symputx("_lsafRowCount_", rowCount);
  run;

  %let notesOption=%sysfunc(getoption(notes));
  options nonotes;

  %***********************************;
  %* Sort the data set               *;
  %***********************************;
  %if (&_lsafRowCount_ > 0) %then
  %do;
    proc sort data=&sas_dsname;
      by dateStartedSasFormat dateEndedSasFormat;
    run;
  %end;

  options &notesOption;

%mend lsaf_getusersessions;