%* lsaf_getuserswithglobalprivilege                                                                 *;
%*                                                                                                  *;
%* Gets all of the user accounts that have the specified privilege identifier. The properties are   *;
%* stored in the SAS data set LSAFGETUSERSWITHGLOBALPRIVILEGE.                                      *;
%*                                                                                                  *;
%* Syntax:                                                                                          *;
%* %LSAF_GETUSERSWITHGLOBALPRIVILEGE(LSAF_PRIVILEGEID=lsaf-privilegeid <,SAS_DSNAME=SAS-data-set>); *;
%*                                                                                                  *;
%* @macvar _lsafRC_  The return code.                                                               *;
%* @macvar _lsafMsg_ The return message.                                                            *;
%*                                                                                                  *;
%* @param lsaf_privilegeid - Required - The identifier of the global privilege. A list of the       *;
%*            privileges can be retrieved by calling the lsaf_getglobalprivileges macro.            *;
%* @param sas_dsname - Optional - The name of the output SAS data set to contain the user accounts  *;
%*            with the corresponding privilege identifier, specified as libref.dataset.  The        *;
%*            default value is WORK.LSAFGETUSERSWITHGLOBALPRIVILEGE. The data set contains a row    *;
%*            for each user account and columns with the following names. The data set is sorted    *;
%*            by user acccount name.                                                                *;
%*            - userName: The user account name.                                                    *;
%*            - userDescription: The user account description.                                      *;
%*            - privilegeType: The privilege type. Valid value: GLOBAL.                             *;
%*            - privilegeId: The identifier of the privilege.                                       *;
%*            - privilegeName: The name of the privilege.                                           *;
%*                                                                                                  *;
%* @macrotype LSAF                                                                                  *;
%* @since 2.3                                                                                       *;
%* @exposure external                                                                               *;
%*                                                                                                  *;
%* @history 2020-03-18  initial coding                                                              *;
%* @history 2020-07-13  update for refactor of java service hierarchy.                              *;
%*                                                                                                  *;

%macro lsaf_getuserswithglobalprivilege(
  lsaf_privilegeid=,
  sas_dsname=work.lsafgetuserswithglobalprivilege
  ) /des='Get a list of users with global privilege.';
  
  %if "&sas_dsname" EQ "" %then %let sas_dsname=work.lsafgetuserswithglobalprivilege;

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafRC_
  ;

  %local _lsafRowCount_;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);
  %let _lsafRowCount_=0;

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data &sas_dsname (label="Created by &SYSMACRONAME"
                    keep=  name description privilegeType privilegeId privilegeName
  );

    attrib returnCode      length=8;
    attrib message         length=$200;
    attrib logMsgPrefix    length=$70;
    attrib logMsgPrimary   length=$70;
    attrib logMsg          length=$300;
    attrib rowCount        length=8;

    attrib name            length=$256   label="User Name";
    attrib description     length=$256   label="User Description";
    attrib privilegeType   length=$20    label="Privilege Type" ;
    attrib privilegeId     length=$64    label="Privilege Identifier";
    attrib privilegeName   length=$128   label="Privilege Name";

    rowCount=0;

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/lsaf/macro/security/privilege/global/GlobalPrivilegeService");
    declare javaobj results("com/sas/lsaf/macro/security/privilege/global/SasPrivilegedUsersResult");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticVoidMethod("getUsersWithGlobalPrivilege", "%bquote(&lsaf_privilegeid)", results);

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
    if (returnCode ne 0) then
    do;
      logMsgPrimary="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Cannot retrieve list of users with global privileges.";
    end;
    else
    do;
      logMsgPrimary=logMsgPrefix;
      logMsg=" List of users with global privileges was returned.";
      results.callIntMethod("getSize", rowCount);
      results.callStringMethod("getPrivilegeTypeId", privilegeType);
      results.callStringMethod("getPrivilegeId", privilegeID);
      results.callStringMethod("getPrivilegeName", privilegeName);
      do row=0 to rowCount - 1;
        results.callStringMethod("getName", row, name);
        results.callStringMethod("getDescription", row, description);
        output;
      end;
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrimary " " logMsg;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    if (rowCount=0) 
      then put logMsgPrefix " No users with global privileges were found.";
      else put logMsgPrefix " " rowCount " records written to data set &sas_dsname";
    put logMsgPrefix " _lsafMsg_= " message ;
    put logMsgPrefix " _lsafRC_= " returnCode ;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);
    call symputx("_lsafRowCount_", rowCount);

  run;

  %***********************************;
  %* Sort the data set               *;
  %***********************************;
  %if (&_lsafRowCount_ > 0) %then
  %do;
    %let notesOption=%sysfunc(getoption(notes));
    options nonotes;
    proc sort data=&sas_dsname;
      by name;
    run;
    options &notesOption;
  %end;

%mend lsaf_getuserswithglobalprivilege;