%* lsaf_groupexists                                                                              *;
%*                                                                                               *;
%* Indicates whether a group is defined for the specified context.                               *;
%*                                                                                               *;
%* The macro variable _lsafGroupExists_ is set to 1, if the group exists, 0 if it does not.      *;
%* In the event of an incorrect call to the macro that results in an error, the return value of  *;
%* _lsafGroupExists_ is set to -1.                                                               *;
%*                                                                                               *;
%* Syntax:                                                                                       *;
%* %LSAF_GROUPEXISTS (LSAF_PATH=lsaf-path, LSAF_GROUP=lsaf- name);                               *;
%*                                                                                               *;
%* @macvar _lsafRC_  The return code.                                                            *;
%* @macvar _lsafMsg_ The return message.                                                         *;
%* @macvar _lsafGroupExists_ Indicates whether the group exists in the context.                  *;
%*                                                                                               *;
%* @param lsaf_path - Required -  The path of the context object.                                *;
%* @param lsaf_group - Required - The name of the group.                                         *;
%*                                                                                               *;
%* @macrotype LSAF                                                                               *;
%* @since 1.2                                                                                    *;
%* @exposure external                                                                            *;
%*                                                                                               *;
%* @history 2013-08-30  updates for standardization and coding conventions                       *;
%* @history 2016-08-26  rebrand as LSAF                                                          *;
%* @history 2019-03-06  update to 2.1                                                            *;
%* @history 2020-12-04  updates for service refactor                                             *;
%*                                                                                               *;

%macro lsaf_groupexists(
  lsaf_path=,
  lsaf_group=
  ) /des='Is the group defined in the context.';

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafGroupExists_
          _lsafMsg_
          _lsafRC_
  ;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data _null_;
    attrib returnCode      length=8;
    attrib message         length=$200;
    attrib logMsgPrefix    length=$70;
    attrib logMsgPrimary   length=$70;
    attrib logMsg          length=$300;
    attrib exists          length=3;

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/lsaf/macro/security/group/GroupService");
    declare javaobj results("com/sas/lsaf/macro/results/Result");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticBooleanMethod("exists", "%bquote(&lsaf_path)", "%bquote(&lsaf_group)", results, exists);

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
    if (returnCode ne 0) then
    do;
      logMsgPrimary="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Cannot determine the existence of group &lsaf_group in &lsaf_path.." ;
      exists=-1;
    end;
    else
    do;
      logMsgPrimary=logMsgPrefix;
      if (exists) 
        then logMsg=" The group &lsaf_group was found in &lsaf_path.";
        else logMsg=" The group &lsaf_group was not found in &lsaf_path.";
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrimary " " logMsg;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    put logMsgPrefix " _lsafMsg_= " message;
    put logMsgPrefix " _lsafRC_= " returnCode;
    put logMsgPrefix " _lsafGroupExists_= " exists;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);
    call symputx("_lsafGroupExists_", exists) ;

  run;

%mend lsaf_groupexists;
