%* lsaf_hasglobalprivilege                                                                    *;                                                         
%*                                                                                            *;                                                   
%* Determines whether a global privilege is assigned to a user account. A list of privilege   *;
%* IDs can be found by running the %lsaf_getglobalprivileges and checking the PrivilegeID     *;
%* column.                                                                                    *;                                                   
%*                                                                                            *;                                                   
%* The SAS macro variable _lsafHasGlobalPrivilege_ is set to a value of 1 when the global     *;                                                  
%* privilege is assigned, 0 if it is not. In the event of an incorrect call to the macro      *;
%* that results in an error, the return value of _lsafHasGlobalPrivilege_ is set to -1.       *;
%*                                                                                            *;                                                   
%* Syntax:                                                                                    *;                                                   
%* %LSAF_HASGLOBALPRIVILEGE(LSAF_USERID=lsaf-userid, LSAF_PRIVILEGE=lsaf-privilege-id);       *;                                                     
%*                                                                                            *;                                                   
%* @macvar _lsafRC_  The return code.                                                         *;                                                  
%* @macvar _lsafMsg_ The return message.                                                      *;                                                  
%* @macvar _lsafHasGlobalPrivilege_ Indicates whether the privilege is assigned to the user   *;
%*                 account.                                                                   *;                                                  
%*                                                                                            *;                                                   
%* @param lsaf_userid - Required - The user ID.                                               *;                                                  
%* @param lsaf_privilege - Required - The identifier of the global privilege.                 *;                                                  
%*                                                                                            *;                                                   
%* @macrotype LSAF                                                                            *;                                                  
%* @since 1.3                                                                                 *;                                                   
%* @exposure external                                                                         *;                                                   
%*                                                                                            *;                                                   
%* @history 2013-11-01 initial coding                                                         *;                                                   
%* @history 2016-08-26 rebrand as LSAF                                                        *;
%* @history 2019-01-18 renamed from lsaf_isuserprivilege to lsaf_hasglobalprivilege           *;
%* @history 2020-07-13  update for refactor of java service hierarchy.                        *; 
%*                                                                                            *;
                                                                                                                         
%macro lsaf_hasglobalprivilege(                                                                                                                          
  lsaf_userid=,                                                                                                                                       
  lsaf_privilege=                                                                                                                                     
  ) /des='Is the global privilege assigned to the specified user?';                                                                                   
                                                                                                                                                      
  %****************************************;                                                                                                          
  %* Initialize macro variables           *;                                                                                                          
  %****************************************;                                                                                                          
  %lsaf_getProductName();

  %global _lsafHasGlobalPrivilege_                                                                                                                       
          _lsafMsg_                                                                                                                                   
          _lsafRC_                                                                                                                                    
  ;                                                                                                                                                   
                                                                                                                                                      
  %let _lsafRC_ =%str(-999);                                                                                                                          
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);                                                                 
                                                                                                                                                      
  %**********************************************;                                                                                                    
  %* Start the datastep to call the java code   *;                                                                                                    
  %**********************************************;                                                                                                    
  data _null_;                                                                                                                                        
    attrib returnCode       length=8;                                                                                                                  
    attrib message          length=$200;                                                                                                               
    attrib logMsgPrefix     length=$70;                                                                                                                
    attrib logMsgPrimary    length=$70;                                                                                                                
    attrib logMsg           length=$300;                                                                                                               
    attrib hasPrivilege     length=3;                                                                                                                  
                                                                                                                                                      
    %***********************************;                                                                                                             
    %* Declare the java objects        *;                                                                                                             
    %***********************************;                                                                                                             
    declare javaobj srv("com/sas/lsaf/macro/security/privilege/global/GlobalPrivilegeService");                                                                              
    declare javaobj results("com/sas/lsaf/macro/results/Result");                                                                                      
                                                                                                                                                      
    %***********************************;                                                                                                             
    %* Attempt the operation           *;                                                                                                             
    %***********************************;                                                                                                             
    srv.callStaticBooleanMethod("hasPrivilege", "%bquote(&lsaf_userid)", "%bquote(&lsaf_privilege)", results, hasPrivilege);                          
                                                                                                                                                      
    %***********************************;                                                                                                             
    %* Retrieve the results            *;                                                                                                             
    %***********************************;                                                                                                             
    results.callIntMethod("getReturnCode", returnCode);                                                                                               
    results.callStringMethod("getReturnMessage", message);                                                                                            
                                                                                                                                                      
    %***********************************;                                                                                                             
    %* Process the results             *;                                                                                                             
    %***********************************;                                                                                                             
    logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";                                                                                            
    if (returnCode ne 0) then                                                                                                                         
    do;                                                                                                                                               
      logMsgPrimary="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";                                                                                        
      logMsg=" Cannot determine if &lsaf_privilege is assigned to the user &lsaf_userid..";                                                           
      hasPrivilege=-1;                                                                                                                                
    end;                                                                                                                                              
    else                                                                                                                                            
    do;                                                                                                                                               
      logMsgPrimary=logMsgPrefix;                                                                                            
      if (hasPrivilege) 
        then logMsg=" &lsaf_privilege is assigned to the user &lsaf_userid.." ;                                                                            
        else logMsg=" &lsaf_privilege is not assigned to the user &lsaf_userid.." ;                                                                        
    end;                                                                                                                                              
                                                                                                                                                      
    %***********************************;                                                                                                             
    %* Print messages in log           *;                                                                                                             
    %***********************************;                                                                                                             
    put;                                                                                                                                              
    put logMsgPrimary " " logMsg;                                                                                                                      
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));                                                                                          
    put logMsgPrefix " _lsafMsg_= " message;                                                                                                          
    put logMsgPrefix " _lsafRC_= " returnCode;                                                                                                        
    put logMsgPrefix " _lsafHasGlobalPrivilege_= " hasPrivilege;                                                                                         
    put;                                                                                                                                              
                                                                                                                                                      
    %***********************************;                                                                                                             
    %* Set the macro variables         *;                                                                                                             
    %***********************************;                                                                                                             
    call symputx("_lsafRC_", returnCode);                                                                                                             
    call symputx("_lsafMsg_", message);                                                                                                               
    call symputx("_lsafHasGlobalPrivilege_", hasPrivilege);                                                                                              
                                                                                                                                                      
  run;                                                                                                                                                
                                                                                                                                                      
%mend lsaf_hasglobalprivilege;                                                                                                                          
