%* lsaf_importstudytlfs                                                                          *;
%*                                                                                               *;
%* Imports study tables, figures, and listings from the specified data sets and optionally       *;
%* creates a validation report as a .csv file in the specified location.                         *;
%*                                                                                               *;
%* Failure and Message Handling                                                                  *;
%*                                                                                               *;
%* The macro performs a series of preliminary validation procedures to whether the specified     *;
%* parameters are valid and required parameters are specified. If none of these procedures       *;
%* fail, the import is attempted. The success or failure, and any issues that are encountered    *;
%* during the import are reported as follows.                                                    *;
%*                                                                                               *;
%* Return Code /  Description                                                                    *;
%*     0: The import was successful. Import details are reported in the sas_result_dsname data   *;
%*        set and in the validation report, if specified.                                        *;
%*    -1: Preliminary validation procedures failed. The sas_result_dsname data set is empty.     *;
%*        No validation report is created.                                                       *;
%*  -300: The import was successful but issues were noted. The notes are included in the         *;
%*        sas_result_dsname data set and the validation report, if specified.                    *;
%*        Additionally, each note is printed to the SAS log file and is designated as a NOTE.    *;
%*  -301: The import was successful but warnings were reported. The warnings and notes are       *;
%*        included in the sas_result_dsname data set and the validation report, if specified.    *;
%*        Additionally, each warning or note is printed to the SAS log file and is designated    *;
%*        as a WARNING or NOTE, respectively.                                                    *;
%*  -302: The import failed due to input data errors. The errors, warnings, and notes are        *;
%*        included in the sas_result_dsname data set and the validation report, if specified.    *;
%*        Additionally, each error, warning, or note is printed to the SAS log file and is       *;
%*        designated as an ERROR, WARNING, or NOTE, respectively.                                *;
%*                                                                                               *;
%* Syntax:                                                                                       *;
%* %LSAF_IMPORTSTUDYTLFS(                                                                        *;
%*       LSAF_STUDYID=lsaf-studyid,                                                              *;
%*       LSAF_BASESTANDARDNAME=lsaf-basestandardname,                                            *;
%*       LSAF_MODELID=lsaf-modelid,                                                              *;
%*       LSAF_IMPORTACTION=lsaf-importaction,                                                    *;
%*       LSAF_LOCATION=lsaf-location,                                                            *;
%*       LSAF_SECTIONSPATH=lsaf-sectionspath,                                                    *;
%*       <LSAF_SECTIONSVERSION=lsaf-sectionsversion,>                                            *;
%*       LSAF_TLFSPATH=lsaf-tlfspath,                                                            *;
%*       <LSAF_TLFSVERSION=lsaf-tlfsversion,>                                                    *;
%*       LSAF_RESULTSPATH=lsaf-resultspath,                                                      *;
%*       <LSAF_RESULTSVERSION=lsaf-resultsversion,>                                              *;
%*       LSAF_DATASETSPATH=lsaf-datasetspath,                                                    *;
%*       <LSAF_DATASETSVERSION=lsaf-datasetsversion,                                             *;
%*       LSAF_REPORTLOCATION=lsaf-reportlocation,                                                *;
%*       LSAF_REPORTPATH=lsaf-reportpath,                                                        *;
%*       LSAF_REPORTOVERWRITE=lsaf-reportoverwrite,                                              *;
%*       LSAF_REPORTENABLEVERSIONING=lsaf-reportenableversioning,                                *; 
%*       LSAF_REPORTVERSIONTYPE=lsaf-reportversiontype,                                          *;
%*       LSAF_REPORTCUSTOMVERSION=lsaf-reportcustomversion,                                      *;
%*       LSAF_REPORTCOMMENT=lsaf-reportcomment,                                                  *;
%*       SAS_RESULT_DSNAME=sas-dsname>                                                           *;
%*       );                                                                                      *;
%*                                                                                               *;
%* @macvar _lsafRC_  The return code.                                                            *;
%* @macvar _lsafMsg_ The return message.                                                         *;
%*                                                                                               *;
%* @param lsaf_studyid - Required - The study identifier. This can be retrieved by calling the   *;
%*        lsaf_getstudyid macro.                                                                 *;
%* @param lsaf_basestandardname - Required - The case-sensitive name of the base standard, such  *;
%*        as SDTM and ADaM.                                                                      *;
%* @param lsaf_modelid - Required -The identifier of the model. This can be retrieved by         *;
%*        calling the lsaf_getstudystandards macro.                                              *;
%* @param lsaf_importaction - Required - The case-insensitive value to indicate the action to    *;
%*        take by the import. Valid values: REPLACE or ADD_NEW.                                  *;
%* @param lsaf_location - Required - The location of the import source file.                     *;
%*        Valid values: REPOSITORY or WORKSPACE.                                                 *;
%* @param lsaf_sectionspath - Required - The case-sensitive path of the data set to import the   *;
%*        TLF sections.                                                                          *;
%* @param lsaf_sectionsversion - Optional - The version of the data set to import the TLF        *;
%*        sections. If not specified, the most recent version is used.                           *;
%* @param lsaf_tlfspath - Required - The case-sensitive path of the data set to import the TLF   *;
%*        TLFs.                                                                                  *;
%* @param lsaf_tlfsversion - Optional - The version of the data set to import the TLF TLFs. If   *;
%*        not specified, the most recent version is used.                                        *;
%* @param lsaf_resultspath - Required - The case-sensitive path of the data set to import the    *;
%*        TLF results.                                                                           *;
%* @param lsaf_resultsversion - Optional - The version of the data set to import the TLF results.*;
%*        If not specified, the most recent version is used.                                     *;
%* @param lsaf_datasetspath - Required - The case-sensitive path of the data set to import the   *;
%*        TLF data sets.                                                                         *;
%* @param lsaf_datasetsversion - Optional - The version of the data set to import the TLF data   *;
%*        sets. If not specified, the most recent version is used.                               *;
%* @param lsaf_reportlocation - Optional - The case-insensitive location in which to create      *;
%*        the report. If the report location and report path are not spcified, the import runs   *;
%*        but no validation report is created.                                                   *;
%*        Valid values: REPOSITORY or WORKSPACE.                                                 *;
%* @param lsaf_reportpath - Optional - The case-sensitive path of the report, which includes     *;
%*        the report file name. If the report location and report path are not specified, the    *;
%*        import runs but no validation report is created.                                       *;
%* @param lsaf_reportoverwrite - Optional - Indicates whether an existing unversioned repository *;
%*        or existing workspace report file is overwritten by the new report with the same path. *;
%*        When a versioned repository file exists, this parameter is ignored and the parameters  *;
%*        versionType and customVersion, or their defaults, if not specified, are applied to     *;
%*        the new version of the file.                                                           *;
%*        Valid values: 0 (Default): No, 1: Yes.                                                 *;
%* @param lsaf_reportenableversioning - Optional - Indicates whether to enable versioning for a  *;
%*        new file in the repository. The parameter is ignored when writing to the workspace     *;
%*        and does not apply when the repository file exists. This parameter does not enable or  *;
%*        disable versioning of an existing repository file.                                     *;
%*        Valid values: 0 (Default): No, 1: Yes.                                                 *;
%* @param lsaf_reportversiontype - Optional - The case-insensitive type for a versioned report   *;
%*        in the repository. This parameter is ignored when writing to the workspace and does    *;
%*        not apply when the file exists as a non-versioned repository file.                     *;
%*        Valid values: MAJOR, MINOR, or CUSTOM.                                                 *;
%* @param lsaf_reportcustomversion - Optional - The version value when                           *;
%*        lsaf_reportversiontype is CUSTOM. This parameter is ignored when writing to the        *;
%*        workspace and does not apply when the file exists as a non-versioned repository file.  *;
%* @param lsaf_reportcomment - Optional - The comment to associate with the new repository       *;
%*        file.                                                                                  *;
%* @param sas_result_dsname - Optional - The name of the output SAS data set to contain the      *;
%*        completion status information about the import, specified as libref.dataset. The       *;
%*        default value is WORK.LSAFIMPORTSTUDYTLFS. The data set contains information           *;
%*        pertaining to the import operation using the following columns.                        *;
%*        - studyId: The study identifier.                                                       *;
%*        - name: The name of the property.                                                      *;
%*        - value: The value of the property.                                                    *;
%*                                                                                               *;
%* @macrotype LSAF                                                                               *;
%* @since 2.4                                                                                    *;
%* @exposure external                                                                            *;
%*                                                                                               *;
%* @history 2020-09-22  initial coding                                                           *;
%*                                                                                               *;

%macro lsaf_importstudytlfs(
  lsaf_studyId=, 
  lsaf_basestandardname=,
  lsaf_modelId=,
  lsaf_importAction=, 
  lsaf_location=, 
  lsaf_sectionsPath=,
  lsaf_sectionsVersion=,
  lsaf_tlfsPath=,
  lsaf_tlfsVersion=,
  lsaf_resultsPath=,
  lsaf_resultsVersion=,
  lsaf_datasetsPath=,
  lsaf_datasetsVersion=,
  lsaf_reportLocation=,
  lsaf_reportPath=,
  lsaf_reportOverwrite=,
  lsaf_reportEnableVersioning=, 
  lsaf_reportVersionType=,
  lsaf_reportCustomVersion=,
  lsaf_reportComment=,
  sas_result_dsname=work.lsafimportstudytlfs
  ) /des='Imports study tables, figures, and listings to data sets in the repository.';
  
  %if "&sas_result_dsname" EQ "" %then %let sas_result_dsname=work.lsafimportstudytlfs;

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafRC_
  ;

  %local _lsafRowCount_;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);
  %let _lsafRowCount_=0;

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data &sas_result_dsname (label="Created by &SYSMACRONAME" keep=studyId name value);
    attrib returnCode           length=8
           message              length=$200
           logMsgPrefix         length=$70
           logMsgPrimary        length=$70
           logMsg               length=$300
           rowCount             length=8
           studyId              length=$32      label="Study Id"
           name                 length=$64      label="Property Name"
           value                length=$256     label="Property Value"
           completionStatus     length=$32
           completionStatusMsg  length=$256
           sasLogFlag           length=$200
           sasLogMsg            length=$200 
    ;

    %*****************************;
    %* Declare the java objects  *;
    %*****************************;
    declare javaobj srv("com/sas/lsaf/macro/clinical/study/StudyService") ;
    declare javaobj importParameters("com/sas/lsaf/macro/clinical/SasClinicalUpdateTlfsParameters");
    declare javaobj exportData("com/sas/lsaf/macro/content/SasFileCreateParameters"); 
    declare javaobj results("com/sas/lsaf/macro/clinical/study/SasStudyImportValidationResult");
 
    %***********************************;
    %* Attempt the operation           *;
    %***********************************;

    importParameters.callVoidMethod("updateImportData",
		"%bquote(&lsaf_location)", "%bquote(&lsaf_importaction)", 
		"%bquote(&lsaf_sectionsPath)", "%bquote(&lsaf_sectionsVersion)", "%bquote(&lsaf_tlfsPath)", "%bquote(&lsaf_tlfsVersion)", 
		"%bquote(&lsaf_resultsPath)", "%bquote(&lsaf_resultsVersion)", "%bquote(&lsaf_datasetsPath)", "%bquote(&lsaf_datasetsVersion)"); 
		
    exportData.callVoidMethod("updateData", "%bquote(&lsaf_reportLocation)", "%bquote(&lsaf_reportPath)", "%bquote(&lsaf_reportOverwrite)", 
                "%bquote(&lsaf_reportEnableVersioning)", "%bquote(&lsaf_reportVersionType)", "%bquote(&lsaf_reportCustomVersion)", "%bquote(&lsaf_reportComment)");

    srv.callStaticVoidMethod("importStudyTlfs", "%bquote(&lsaf_studyId)", "%bquote(&lsaf_baseStandardName)", "%bquote(&lsaf_modelId)", importParameters, exportData, results); * <---------------- CHANGED THIS (AGAIN).;

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);
    results.callStringMethod("getCompletionStatus", completionStatus);
    results.callStringMethod("getCompletionStatusMessage", completionStatusMsg);
    logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
    if (returnCode ne 0) then
    do;
      if (returnCode in (-400:-300)) then 
      do;
        logMsgPrimary= strip(completionStatus) || ": &_LSAF_PRODUCT_NAME_ Macro:";
        logMsg=" " || strip(completionStatusMsg) ;
      end;
      else 
      do;
        logMsgPrimary="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
        logMsg=" Could not import the study tables, listings, and figures." ;
      end;
    end;
    else
    do;
      logMsgPrimary=logMsgPrefix;
      logMsg=" Returned import completion information for the study tables, listings, and figures.";
    end;
    studyId="&lsaf_studyId";
    results.callIntMethod("getSize", rowCount);
    do row=0 to rowCount - 1;
      results.callStringMethod("getPropertyName", row, name);
      results.callStringMethod("getPropertyValue", row, value);
      output;
    end;
    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrimary " " logMsg;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    if (rowCount<=0) 
      then put logMsgPrefix " The study tables, listings, and figures were not imported.";
      else put logMsgPrefix " " rowCount " records written to data set &sas_result_dsname";
    put logMsgPrefix " _lsafMsg_= " message ;
    put logMsgPrefix " _lsafRC_= " returnCode ;
    put;

    results.callIntMethod("getValidationMessagesSize", rowCount);
    if (rowCount>0) then 
    do;
      macroNameStamp=strip(lowcase("&SYSMACRONAME")) || ": ";
      do row=0 to rowCount - 1;
        results.callStringMethod("getMessageSeverity", row, sasLogFlag);
        results.callStringMethod("getMessageDetailedProblem", row, sasLogMsg);
        sasLogFlag = strip(sasLogFlag) || ": ";
        sasLogMsg  = strip(sasLogMsg);
        put sasLogFlag macroNameStamp sasLogMsg;
      end;
      put "-" ;
    end;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);
    call symputx("_lsafRowCount_", rowCount);
  run;

  %***********************************;
  %* DO NOT Sort the data set        *;
  %***********************************;

%mend lsaf_importstudytlfs;
