%* lsaf_ismember                                                                                   *;
%*                                                                                                 *;
%* Indicates whether a user or group is a member in the specified context.                         *;
%*                                                                                                 *;
%* The SAS macro variable _lsafIsMember_ is set to a value of 1 when the specified user or group   *;
%* is a member of the context, 0 if it is not. In the event of an incorrect call to the macro      *;
%* that results in an error, the return value of _lsafIsMember_ is set to -1. Groups               *;
%* defined at the context level are not considered as members of that context.                     *;
%*                                                                                                 *;
%* Syntax:                                                                                         *;
%* %LSAF_ISMEMBER(LSAF_PATH=lsaf-path, LSAF_MEMBER=lsaf-member-identifier <, LSAF_TYPE=lsaf-type,  *;
%* LSAF_GROUP_CONTEXT=lsaf-group-context>, LSAF_INCLUDEIMPLICIT=implicit-flag>);                   *;
%*                                                                                                 *;
%* @macvar _lsafRC_  The return code.                                                              *;
%* @macvar _lsafMsg_ The return message.                                                           *;
%* @macvar _lsafIsMember_ Indicates whether the user or group is a member of the specified context.*;
%*                                                                                                 *;
%* @param lsaf_path - Required -  The absolute path of the context.                                *;
%* @param lsaf_member - Required - The user ID or group name of the member.                        *;
%* @param lsaf_type - Conditional - The type of member. Valid values (case-insensitive):           *;
%*            - USER (Default)                                                                     *;
%*            - GROUP                                                                              *;
%* @param lsaf_group_context - Conditional - The context in which the member group is defined,     *;
%*            specified as a path (such as /SAS). The parameter is required, if LSAF_TYPE=GROUP.   *;
%*            Otherwise, it is ignored.                                                            *;
%* @param lsaf_includeImplicit - Optional - Indicates whether to report that the user is an        *;
%*            implicit member of the context through group membership. The default value is 0,     *;
%*            which reports whether the user is an explicit member of the context by user ID only. *;
%*            Valid values:                                                                        *;
%*            - 0 (Default)                                                                        *;
%*            - 1                                                                                  *;
%*                                                                                                 *;
%* @macrotype LSAF                                                                                 *;
%* @since 1.2                                                                                      *;
%* @exposure external                                                                              *;
%*                                                                                                 *;
%* @history 2013-08-30  updates for standardization and coding conventions                         *;
%* @history 2016-08-26  rebrand as LSAF                                                            *;
%* @history 2019-02-11  update to 2.1                                                              *;
%* @history 2020-12-04  updates for service refactor                                               *;
%*                                                                                                 *;

%macro lsaf_ismember(
  lsaf_path=,
  lsaf_member=,
  lsaf_type=USER,
  lsaf_group_context=,
  lsaf_includeImplicit=0
  ) /des='Is the user or group a member in the specified context?';

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafIsMember_
          _lsafMsg_
          _lsafRC_
  ;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %**********************************************;
  %*  Start the datastep to call the java code  *;
  %**********************************************;
  data _null_;
    attrib returnCode      length=8;
    attrib message         length=$200;
    attrib logMsgPrefix    length=$70;
    attrib logMsgPrimary   length=$70;
    attrib logMsg          length=$300;
    attrib isMember        length=3;

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/lsaf/macro/security/membership/MembershipService");
    declare javaobj results("com/sas/lsaf/macro/results/Result");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticBooleanMethod("isMember", "%bquote(&lsaf_path)", "%bquote(&lsaf_type)", 
               "%bquote(&lsaf_member)", "%bquote(&lsaf_group_context)", "%bquote(&lsaf_includeImplicit)", 
               results, isMember);

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
    if (returnCode ne 0) then
    do;
      logMsgPrimary="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      if upcase("&lsaf_type")="GROUP"
        then logMsg=" Cannot determine if &lsaf_type &lsaf_group_context &lsaf_member is a member of &lsaf_path.";
        else logMsg=" Cannot determine if &lsaf_type &lsaf_member is a member of &lsaf_path.";
      isMember=-1;
    end;
    else
    do;
      logMsgPrimary=logMsgPrefix;
      if (isMember) 
        then if upcase("&lsaf_type")="GROUP"
          then logMsg=" The &lsaf_type &lsaf_group_context &lsaf_member is a member of &lsaf_path.";
          else logMsg=" The &lsaf_type &lsaf_member is a member of &lsaf_path.";
        else if upcase("&lsaf_type")="GROUP"
          then logMsg=" The &lsaf_type &lsaf_group_context &lsaf_member is not a member of &lsaf_path.";
          else logMsg=" The &lsaf_type &lsaf_member is not a member of &lsaf_path.";
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrimary " " logMsg;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    put logMsgPrefix " _lsafMsg_= " message;
    put logMsgPrefix " _lsafRC_= " returnCode;
    put logMsgPrefix " option lsaf_includeImplicit = &lsaf_includeImplicit";
    put logMsgPrefix " _lsafIsMember_= " isMember;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);
    call symputx("_lsafIsMember_", isMember);

  run;

%mend lsaf_ismember;
