%* lsaf_isowner                                                                                  *;
%*                                                                                               *;
%* Indicates whether the specified user is the owner of an item in the SAS Life Science          *;
%* Analytics Framework repository.                                                               *;
%*                                                                                               *;
%* The SAS macro variable _lsafIsOwner_ is set to a value of 1, if the specified user value is   *;
%* the valid owner of the item, 0 if it is not. In the event of an incorrect call to the macro   *;
%* that results in an error, the return value of _lsafIsOwner_ is set to -1.                     *;
%*                                                                                               *;
%* Syntax:                                                                                       *;
%* %LSAF_ISOWNER(LSAF_PATH=lsaf-path, LSAF_ACLTYPE=lsaf-acl-type, LSAF_USERID=lsaf-userid);      *;
%*                                                                                               *;
%* @macvar _lsafRC_  The return code.                                                            *;
%* @macvar _lsafMsg_ The return message.                                                         *;
%* @macvar _lsafIsOwner_ Indicates whether the user is the owner of the item.                    *;
%*                                                                                               *;
%* @param lsaf_path - Required - The path of the item in the SAS Life Science Analytics          *;
%*            Framework repository.                                                              *;
%* @param lsaf_aclType - Conditional - The type of permissions. Valid values (case-insensitive): *;
%*            - DEFAULT                                                                          *;
%*            - CURRENT                                                                          *;
%*            Files have only current permissions. Therefore, a value is not required when       *;
%*            lsaf_path is a file. However, if a value other than CURRENT or blank is specified  *;
%*            for a file, the operation fails.                                                   *;
%*            For container items, the value must be specified.                                  *;
%* @param lsaf_userid - Required - The user ID to check. To check for <creator>, specify it as   *;
%*            the user ID, including the brackets.                                               *;
%*                                                                                               *;
%* @macrotype LSAF                                                                               *;
%* @since 2.5                                                                                    *;
%* @exposure external                                                                            *;
%*                                                                                               *;
%* @history 2019-02-05  initial coding                                                           *;
%* @history 2020-07-13  update for refactor of java service hierarchy.                           *;
%*                                                                                               *;

%macro lsaf_isowner(
  lsaf_path=,
  lsaf_acltype=,
  lsaf_userid=
  ) /des='Is the specified user the owner of the item?';

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafIsOwner_
          _lsafMsg_
          _lsafRC_
  ;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data _null_;
    attrib returnCode          length=8;
    attrib message             length=$200;
    attrib logMsgPrefix        length=$70;
    attrib logMsg              length=$300;
    attrib isOwner             length=3;
    attrib logMsgPrimary       length=$70;

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/lsaf/macro/security/acl/AclService");
    declare javaobj results("com/sas/lsaf/macro/results/Result");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticBooleanMethod("isOwner", "%bquote(&lsaf_path)","%bquote(&lsaf_acltype)","%bquote(&lsaf_userid)", results, isOwner);

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
    if (returnCode ne 0) then
    do;
      logMsgPrimary="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Cannot determine if the user &lsaf_userid is the &lsaf_acltype owner of item: &lsaf_path." ;
      isOwner=-1;
    end;
    else
    do;
      logMsgPrimary=logMsgPrefix;
      if (isOwner) 
        then logMsg=" The user &lsaf_userid is the &lsaf_acltype owner of item: &lsaf_path.";
        else logMsg=" The user &lsaf_userid is not the &lsaf_acltype owner of item: &lsaf_path.";
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrimary " " logMsg;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    put logMsgPrefix " _lsafMsg_= " message;
    put logMsgPrefix " _lsafRC_= " returnCode;
    put logMsgPrefix " _lsafIsOwner_= " isOwner;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);
    call symputx("_lsafIsOwner_", isOwner);

  run;

%mend lsaf_isowner;
