%* lsaf_isstudyextdictionary                                                                           *;
%*                                                                                                     *;
%* Indicates whether an external dictionary is associated with a study.                                *;
%*                                                                                                     *;
%* The SAS macro variable _lsafIsStudyExtDictionary_ is set to a value of 1, if the specified          *;
%* dictionary is associated with the specified study, 0 if it is not. In the event of an incorrect     *;
%* call to the macro that results in an error, the return value of _lsafIsStudyExtDictionary_ is set   *;
%* to -1.                                                                                              *;
%*                                                                                                     *;
%* Syntax:                                                                                             *;
%* %LSAF_ISSTUDYEXTDICTIONARY(LSAF_STUDYID=lsaf-studyid, LSAF_EXTDICTIONARYID=lsaf-extdictionaryid);   *;
%*                                                                                                     *;
%* @macvar _lsafRC_  The return code.                                                                  *;
%* @macvar _lsafMsg_ The return message.                                                               *;
%* @macvar _lsafIsStudyExtDictionary_ Indicates whether the external dictionary is associated with the *;
%*           study.                                                                                    *;
%*                                                                                                     *;
%* @param lsaf_studyid - Required - The study item identifier. The identifier can be retrieved by      *;
%*           calling the lsaf_getstudyid macro.                                                        *;
%* @param lsaf_extdictionaryid - Required - The external dictionary identifier. The identifier can be  *;
%*           retrieved by calling the lsaf_getextdictionaryid macro.                                   *;
%*                                                                                                     *;
%* @macrotype LSAF                                                                                     *;
%* @since 2.2                                                                                          *;
%* @exposure external                                                                                  *;
%*                                                                                                     *;
%* @history 2019-08-26  initial coding                                                                 *;
%*                                                                                                     *;

%macro lsaf_isstudyextdictionary(
  lsaf_studyid=,
  lsaf_extdictionaryid=
  ) /des='Is the external dictionary associated with a SAS Life Science Analytics Framework study?';

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafIsStudyExtDictionary_
          _lsafMsg_
          _lsafRC_
  ;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data _null_;
    attrib returnCode           length=8;
    attrib message              length=$200;
    attrib logMsgPrefix         length=$70;
    attrib logMsgPrimary        length=$70;
    attrib logMsg               length=$300;
    attrib isStudyExtDictionary length=3;

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/lsaf/macro/clinical/study/StudyService");
    declare javaobj results("com/sas/lsaf/macro/results/Result");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticBooleanMethod("isStudyExternalDictionary", "%bquote(&lsaf_studyid)", "%bquote(&lsaf_extdictionaryid)", results, isStudyExtDictionary);

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
    if (returnCode ne 0) then
    do;
      logMsgPrimary="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Cannot determine if the external dictionary is associated to the study." ;
      isStudyExtDictionary=-1;
    end;
    else
    do;
      logMsgPrimary=logMsgPrefix;
      if (isStudyExtDictionary) then
      do;
        logMsg=" The external dictionary is associated with the study.";
      end;
      else
      do;
        logMsg=" The external dictionary is NOT associated with the study.";
      end;
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrimary " " logMsg;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    put logMsgPrefix " _lsafMsg_= " message;
    put logMsgPrefix " _lsafRC_= " returnCode;
    put logMsgPrefix " _lsafIsStudyExtDictionary_= " isStudyExtDictionary;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);
    call symputx("_lsafIsStudyExtDictionary_", isStudyExtDictionary);

  run;

%mend lsaf_isstudyextdictionary;
