%* lsaf_isstudystandard                                                                                *;
%*                                                                                                     *;
%* Indicates whether a standard is associated with a study.                                            *;
%*                                                                                                     *;
%* The SAS macro variable _lsafIsStudyStandard_ is set to a value of 1, if the specified standard is   *;
%* associated with the specified study, 0 if it is not. In the event of an incorrect call to the       *;
%* macro that results in an error, the return value of _lsafIsStudyStandard_ is set to -1.             *;
%*                                                                                                     *;
%* Syntax:                                                                                             *;
%* %LSAF_ISSTUDYSTANDARD(LSAF_STUDYID=lsaf-studyid, LSAF_STANDARDID=lsaf-standardid);                  *;
%*                                                                                                     *;
%* @macvar _lsafRC_  The return code.                                                                  *;
%* @macvar _lsafMsg_ The return message.                                                               *;
%* @macvar _lsafIsStudyStandard_ Indicates whether the standard is associated with the study.          *;
%*                                                                                                     *;
%* @param lsaf_studyid - Required - The study item identifier. The identifier can be retrieved by      *;
%*           calling the lsaf_getstudyid macro.                                                        *;
%* @param lsaf_standardid - Required - The standard identifier. The identifier can be retrieved by     *;
%*           calling the lsaf_getstandardid macro.                                                     *;
%*                                                                                                     *;
%* @macrotype LSAF                                                                                     *;
%* @since 2.2                                                                                          *;
%* @exposure external                                                                                  *;
%*                                                                                                     *;
%* @history 2019-08-26  initial coding                                                                 *;
%*                                                                                                     *;

%macro lsaf_isstudystandard(
  lsaf_studyid=,
  lsaf_standardid=
  ) /des='Is the standard associated with a SAS Life Science Analytics Framework study?';

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafIsStudyStandard_
          _lsafMsg_
          _lsafRC_
  ;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data _null_;
    attrib returnCode        length=8;
    attrib message           length=$200;
    attrib logMsgPrefix      length=$70;
    attrib logMsgPrimary     length=$70;
    attrib logMsg            length=$300;
    attrib isStudyStandard   length=3;

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/lsaf/macro/clinical/study/StudyService");
    declare javaobj results("com/sas/lsaf/macro/results/Result");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticBooleanMethod("isStudyStandard", "%bquote(&lsaf_studyid)", "%bquote(&lsaf_standardid)", results, isStudyStandard);

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
    if (returnCode ne 0) then
    do;
      logMsgPrimary="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Cannot determine if the standard is associated to the study." ;
      isStudyStandard=-1;
    end;
    else
    do;
      logMsgPrimary=logMsgPrefix;
      if (isStudyStandard) then
      do;
        logMsg=" The standard is associated with the study.";
      end;
      else
      do;
        logMsg=" The standard is NOT associated with the study.";
      end;
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrimary " " logMsg;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    put logMsgPrefix " _lsafMsg_= " message;
    put logMsgPrefix " _lsafRC_= " returnCode;
    put logMsgPrefix " _lsafIsStudyStandard_= " isStudyStandard;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);
    call symputx("_lsafIsStudyStandard_", isStudyStandard);

  run;

%mend lsaf_isstudystandard;
