%* lsaf_isversionsigned                                                                                *;
%*                                                                                                     *;
%* Indicates whether the specified version of a file in the repository is signed.                      *;
%*                                                                                                     *;
%* The SAS macro variable _lsafIsVersionSigned_ is set to a value of 1, if the specified version of a  *;
%* file is signed, 0 if it is not. In the event of an incorrect call to the macro that results in an   *;
%* error, the return value of _lsafIsVersionSigned_ is set to -1.                                      *;
%*                                                                                                     *;
%* Syntax:                                                                                             *;
%* %LSAF_ISVERSIONSIGNED(LSAF_PATH=lsaf-path, LSAF_VERSION=lsaf-version);                              *;
%*                                                                                                     *;
%* @macvar _lsafRC_  The return code.                                                                  *;
%* @macvar _lsafMsg_ The return message.                                                               *;
%* @macvar _lsafIsVersionSigned_ Indicates whether the version of the file is signed.                  *;
%*                                                                                                     *;
%* @param lsaf_path - Required - The full path, which includes the name of the file.                   *;
%* @param lsaf_version - Required - The version of the file.                                           *;
%*                                                                                                     *;
%* @macrotype LSAF                                                                                     *;
%* @since 2.1                                                                                          *;
%* @exposure external                                                                                  *;
%*                                                                                                     *;
%* @history 2019-02-04  initial coding                                                                 *;
%* @history 2020-07-13  update for refactor of java service hierarchy.                                 *;
%*                                                                                                     *;

%macro lsaf_isversionsigned(
  lsaf_path=,
  lsaf_version=
  ) /des='Is the version of the file signed?';

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafIsVersionSigned_
          _lsafMsg_
          _lsafRC_
  ;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data _null_;
    attrib returnCode          length=8;
    attrib message             length=$200;
    attrib logMsgPrefix        length=$70;
    attrib logMsg              length=$300;
    attrib isVersionSigned     length=3;
    attrib logMsgPrimary       length=$70;

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/lsaf/macro/content/repository/signature/SignatureService") ;
    declare javaobj results("com/sas/lsaf/macro/results/Result");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticBooleanMethod("isVersionSigned", "%bquote(&lsaf_path)", "%bquote(&lsaf_version)", results, isVersionSigned);

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
    if (returnCode ne 0) then
    do;
      logMsgPrimary="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Cannot determine if the file is signed for version: &lsaf_version." ;
      isVersionSigned=-1;
    end;
    else
    do;
      logMsgPrimary=logMsgPrefix;
      if (isVersionSigned) 
        then logMsg=" The file is signed for version: &lsaf_version.";
        else logMsg=" The file is not signed for version: &lsaf_version.";
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrimary " " logMsg;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    put logMsgPrefix " _lsafMsg_= " message;
    put logMsgPrefix " _lsafRC_= " returnCode;
    put logMsgPrefix " _lsafIsVersionSigned_= " isVersionSigned;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);
    call symputx("_lsafIsVersionSigned_", isVersionSigned);

  run;

%mend lsaf_isversionsigned;
