%* lsaf_permanentlydelete                                                                  *;
%*                                                                                         *;
%* Permanently deletes an item. Deleted items are not moved to the recycle bin and cannot  *; 
%* be restored. To move an item to the recycle bin, use the macro %lsaf_delete().          *; 
%*                                                                                         *;
%* NOTE: Context items cannot be permanently deleted. Attempting to delete a context       *;
%* results in a failure.                                                                   *;
%*                                                                                         *;
%* Syntax:                                                                                 *;
%* %LSAF_PERMANENTLYDELETE(LSAF_PATH=lsaf-path);                                           *;
%*                                                                                         *;
%* @macvar _lsafRC_  The return code.                                                      *;
%* @macvar _lsafMsg_ The return message.                                                   *;
%*                                                                                         *;
%* @param lsaf_path - Required - The path of the item.                                     *;
%*                                                                                         *;
%* @macrotype LSAF                                                                         *;
%* @since 2.1                                                                              *;
%* @exposure external                                                                      *;
%*                                                                                         *;
%* @history 2019-03-04  initial coding                                                     *;
%*                                                                                         *;

%macro lsaf_permanentlydelete(
  lsaf_path=
  ) /des='Permanently delete an item from the repository.';

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafRC_
  ;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data _null_;
    attrib returnCode      length=8;
    attrib message         length=$200;
    attrib logMsgPrefix    length=$70;
    attrib logMsg          length=$300;
    attrib logMsgPrimary   length=$70;

    %***********************************;
    %* Declare the java items        *;
    %***********************************;
    declare javaobj srv("com/sas/lsaf/macro/content/repository/RepositoryService");
    declare javaobj results("com/sas/lsaf/macro/results/Result");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticVoidMethod("permanentlyDelete", "%bquote(&lsaf_path)", results);

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
    logMsgPrimary=logMsgPrefix;
    logMsg=" The item was permanently deleted: &lsaf_path.." ;
    if (returnCode ne 0) then
    do;
      logMsgPrimary="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Cannot permanently delete item: &lsaf_path.." ;
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrimary " " logMsg;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    put logMsgPrefix " _lsafMsg_= " message ;
    put logMsgPrefix " _lsafRC_= " returnCode ;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);

  run;

%mend lsaf_permanentlydelete;
