%* lsaf_querycontextmembership                                                                              *;
%*                                                                                                          *;
%* Extracts the assigned membership of a context. You can also include the defined groups. The entries      *;
%* to extract are specified in a query that is contained in a SAS data set. The extracted data is stored    *;
%* in a comma-separated values file that is located in your workspace or in the repository.                 *;  
%*                                                                                                          *;
%* Information that is needed to specify the required columnClass and columnName data, as well as their     *;
%* data types, can be retrieved by calling lsaf_getquerycolumns() with the type ContextMembership.          *;
%*                                                                                                          *;
%* You must be an administrative mode user to execute the query.                                            *;
%*                                                                                                          *;
%* For a detailed explanation of the macro parameters, see the SAS Life Science Analytics Framework         *;
%* Macros API Users Guide for examples and the module QueryService header text.                             *;
%*                                                                                                          *;
%* Syntax:                                                                                                  *;
%* %LSAF_QUERYCONTEXTMEMBERSHIP(LSAF_PATH=lsaf-path,                                                        *;
%*     LSAF_SCOPE=lsaf-scope,                                                                               *;
%*     LSAF_QUERYDATASET=lsaf-querydataset,                                                                 *; 
%*     LSAF_VALIDATEDDATASET=lsaf-validateddataset,                                                         *;
%*     LSAF_EXPORTLOCATION=lsaf-exportlocation,                                                             *;
%*     LSAF_EXPORTPATH=lsaf-exportpath,                                                                     *;
%*     LSAF_OVERWRITE=lsaf-overwrite,                                                                       *; 
%*     LSAF_ENABLEVERSIONING=lsaf-enableversioning,                                                         *; 
%*     LSAF_VERSIONTYPE=lsaf-versiontype,                                                                   *; 
%*     LSAF_CUSTOMVERSION=lsaf-customversion,                                                               *; 
%*     LSAF_COMMENT=lsaf-comment);                                                                          *;
%*                                                                                                          *;
%* @macvar _lsafRC_  The return code.                                                                       *;
%* @macvar _lsafMsg_ The return message.                                                                    *;
%* @macvar _lsafContxtMembershpExpPath_ The full path to the query results CSV file, which includes         *;
%*            the file extension. If the query is not executed or if the macro processing fails,            *;
%*            _lsafContxtMembershpExpPath_ is blank.                                                        *;
%*                                                                                                          *;
%* @param lsaf_path - Required - The context path.                                                          *;
%* @param lsaf_scope - Optional - The membership scope. Valid values: ASSIGNED, DEFINED, and ALL (Default). *;
%* @param lsaf_querydataset - Optional - The name of the input SAS data set that contains the query         *;
%*            metadata, specified as libref.dataset. If not provided, the query returns the membership      *;
%*            information for the context.                                                                  *;
%* @param lsaf_validateddataset - Required - The name of the output SAS data set to contain the             *;
%*            results of the validation of the records from the input data set lsaf_querydataset,           *;
%*            specified as libref.dataset. The default data set name is WORK.LSAFQUERYCONTEXTMEMBERSHIP.    *;
%* @param lsaf_exportlocation - Required - The case-insensitive output location for the exported file.      *;
%*            Valid values: REPOSITORY and WORKSPACE.                                                       *;
%* @param lsaf_exportpath - Required - The case-sensitive output path for the exported CSV file.            *;
%* @param lsaf_overwrite - Optional - Indicates whether an existing nonversioned file is overwritten.       *;
%*            Valid values: 0 (Default, No) and 1 (Yes).                                                    *;
%* @param lsaf_enableversioning - Optional - Indicates whether versioning is enabled for a new file         *;
%*            in the repository. When lsaf_exportlocation=WORKSPACE, this value is ignored.                 *;
%*            Valid values: 0 (Default, No) and 1 (Yes).                                                    *; 
%* @param lsaf_versiontype - Optional - The case insensitive version type to apply to a versioned file.     *;
%*            This value is ignored when lsaf_exportlocation is WORKSPACE. If not provided for versioned    *;
%*            files, the system default will be applied. Valid values: MAJOR, MINOR, and CUSTOM.            *;
%* @param lsaf_customversion - Conditional - The version number to use to create a customized versioned     *;
%*            file in the repository. When lsaf_exportlocation is WORKSPACE or lsaf_versiontype is not      *;
%*            CUSTOM, this value is ignored.                                                                *;
%* @param lsaf_comment - Optional - The check-in comment to associate with the action of adding a file      *;
%*            or file version to the repository. When lsaf_exportlocation=WORKSPACE, this value is ignored. *;
%*                                                                                                          *;
%* @macrotype LSAF                                                                                          *;
%* @since 2.4                                                                                               *;
%* @exposure external                                                                                       *;
%*                                                                                                          *;
%* @history 2020-06-05  initial coding                                                                      *;
%* @history 2020-11-30  updated for version handling.                                                       *;
%* @history 2021-03-25  allow for missing query dataset.                                                    *;
%*                                                                                                          *;

%macro lsaf_querycontextmembership(
  lsaf_path=,
  lsaf_scope=ALL,
  lsaf_querydataset=,
  lsaf_validateddataset=work.lsafquerycontextmembership,
  lsaf_exportlocation=,
  lsaf_exportpath=,
  lsaf_overwrite=0,
  lsaf_enableversioning=0,
  lsaf_versiontype=MAJOR,
  lsaf_customversion=,
  lsaf_comment=
  ) /des='Performs a query and returns an output file to a specified location in the SAS Life Science Analytics Framework.';

  %if "&lsaf_validateddataset" EQ "" %then %let lsaf_validateddataset=work.lsafquerycontextmembership;
  %let macName = %sysfunc(lowcase(&SYSMACRONAME));
  
  %lsaf_query(lsaf_callingMacro=&macName,
              lsaf_path=&lsaf_path,
              lsaf_scope=&lsaf_scope,
              lsaf_queryMethod=%str(executeContextMembershipQuery),
              lsaf_returnPathVar=%str(_lsafContxtMembershpExpPath_),
              lsaf_querydataset=&lsaf_querydataset,
              lsaf_validateddataset=&lsaf_validateddataset,
              lsaf_exportlocation=&lsaf_exportlocation,
              lsaf_exportpath=&lsaf_exportpath,
              lsaf_overwrite=&lsaf_overwrite,
              lsaf_enableversioning=&lsaf_enableversioning,
              lsaf_versiontype=&lsaf_versiontype,
              lsaf_customversion=&lsaf_customversion,
              lsaf_comment=&lsaf_comment,
              lsaf_allowmissingqueryds=1

  );

%mend lsaf_querycontextmembership;
