%* lsaf_rename                                                                                   *;
%*                                                                                               *;
%* Renames an item in the repository. The new name must include the file extension, otherwise    *;
%* the renaming occurs without the file having any extension.                                    *;
%*                                                                                               *;
%* An item with the new name must not exist in the current location.                             *;
%*                                                                                               *;
%* Syntax:                                                                                       *;
%* %LSAF_RENAME(LSAF_SOURCE=lsaf-source-path, LSAF_NEWNAME=lsaf-new-name);                       *;
%*                                                                                               *;
%* @macvar _lsafRC_  The return code.                                                            *;
%* @macvar _lsafMsg_ The return message.                                                         *;
%*                                                                                               *;
%* @param lsaf_source  - Required - The path, which includes the name.                           *;
%* @param lsaf_newname - Required - The new name.                                                *;
%*                                                                                               *;
%* Sample calls                                                                                  *;
%*                                                                                               *;
%*    %lsaf_rename(lsaf_source=/SAS/File/saspgm.sas,lsaf_newname=saspgm2.sas)                    *;
%*             Renames the file saspgm.sas in /SAS/File to saspgm2.sas                           *;
%*                                                                                               *;
%*    %lsaf_rename(lsaf_source=/SAS/File/Progect1,lsaf_newname=Project2)                         *;
%*             Renames the container Project1 in /SAS/File to Project2                           *;
%*                                                                                               *;
%* @macrotype LSAF                                                                               *;
%* @since 1.4                                                                                    *;
%* @exposure external                                                                            *;
%*                                                                                               *;
%* @history 2014-03-03  initial coding                                                           *;
%* @history 2016-08-26  rebrand as LSAF                                                          *;
%* @history 2018-12-10  documentation update for LSAF 5.x behavior                               *;
%* @history 2019-01-08  rename lsaf_renameobject to lsaf_rename                                  *;
%* @history 2021-07-08  changed call to allow for return of new path                             *;
%*                                                                                               *;


%macro lsaf_rename(
  lsaf_source=,
  lsaf_newname=
  ) /des='Rename a file or folder.';

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafRC_
  ;

  %let _lsafRC_=%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %**********************************************;
  %*  Start the datastep to call the java code  *;
  %**********************************************;
  data _null_;
    attrib returnCode      length=8;
    attrib message         length=$200;
    attrib logMsgPrefix    length=$70;
    attrib logMsgPrimary   length=$70;
    attrib logMsg          length=$300;

	attrib newPath         length=$2048;

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/lsaf/macro/content/repository/RepositoryService");
    declare javaobj results("com/sas/lsaf/macro/results/Result");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticStringMethod("rename", "%bquote(&lsaf_source)", "%bquote(&lsaf_newname)", results, newPath);

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
    if (returnCode eq 0) then
    do;
      logMsgPrimary=logMsgPrefix;
      logMsg=" The rename operation was successful.";
    end;
    else
    do;
      logMsgPrimary="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Cannot successfully rename the item.  The item was not renamed." ;
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrimary " " logMsg;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    put logMsgPrefix " _lsafMsg_= " message ;
    put logMsgPrefix " _lsafRC_= " returnCode ;
    put logMsgPrefix " lsaf_source = &lsaf_source" ;
    put logMsgPrefix " lsaf_newname = &lsaf_newname" ;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);

  run;

%mend lsaf_rename;
