%* lsaf_roleexists                                                                                *;
%*                                                                                                *;
%* Indicates whether a role exists as defined or inherited in a specified context.                *;
%*                                                                                                *;
%* The SAS macro variable _lsafRoleExists_ is set to a value of 1, if the role exists, 0 if it    *;
%* does not. In the event of an incorrect call to the macro that results in an error, the return  *;
%* value of _lsafRoleExists_ is set to -1.                                                        *;
%*                                                                                                *;
%* Syntax:                                                                                        *;
%* %LSAF_ROLEEXISTS (LSAF_PATH=lsaf-path, LSAF_ROLE=lsaf-role-name,                               *;
%* LSAF_ROLE_CONTEXT=lsaf-role-context);                                                          *;
%*                                                                                                *;
%* @macvar _lsafRC_  The return code.                                                             *;
%* @macvar _lsafMsg_ The return message.                                                          *;
%* @macvar _lsafRoleExists_  Indicates the existence of the role.                                 *;
%*                                                                                                *;
%* @param lsaf_path - Required - The path of the context.                                         *;
%* @param lsaf_role - Required - The name of the role.                                            *;
%* @param lsaf_role_context - Optional - The context in which the role is defined, specified as a *;
%*            path (such as /SAS). If not provided or if the value is equal to the path parameter,*;
%*            the macro looks for the role to be defined at the path context. Otherwise, the      *;
%*            macro searches for the role inherited at the path context.                          *;
%*                                                                                                *;
%* @macrotype LSAF                                                                                *;
%* @since 1.2                                                                                     *;
%* @exposure external                                                                             *;
%*                                                                                                *;
%* @history 2013-08-30  updates for standardization and coding conventions                        *;
%* @history 2016-08-26  rebrand as LSAF                                                           *;
%* @history 2019-01-29  update to 2.1                                                             *;
%* @history 2020-12-07  updates for service refactor                                              *;
%*                                                                                                *;

%macro lsaf_roleexists(
  lsaf_path=,
  lsaf_role=,
  lsaf_role_context=
  ) /des='Does the role exist in the context.';

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafRoleExists_
          _lsafMsg_
          _lsafRC_
  ;

  %let _lsafRC_=%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data _null_;
    attrib returnCode      length=8;
    attrib message         length=$200;
    attrib logMsgPrefix    length=$70;
    attrib logMsgPrimary   length=$70;
    attrib logMsg          length=$300;
    attrib exists          length=3;

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/lsaf/macro/security/role/RoleService");
    declare javaobj results("com/sas/lsaf/macro/results/Result");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticBooleanMethod("roleExistsAtContext", "%bquote(&lsaf_path)", "%bquote(&lsaf_role)", "%bquote(&lsaf_role_context)", results, exists);

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
    if (returnCode ne 0) then
    do;
      logMsgPrimary="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Cannot determine the existence of the role &lsaf_role in &lsaf_path..";
      exists=-1;
    end;
    else
    do;
      logMsgPrimary=logMsgPrefix;
      if (exists) 
        then logMsg=" The role &lsaf_role exists in &lsaf_path.." ;
        else logMsg=" The role &lsaf_role does not exist in &lsaf_path.." ;
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrimary " " logMsg;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    put logMsgPrefix " _lsafMsg_= " message;
    put logMsgPrefix " _lsafRC_= " returnCode;
    put logMsgPrefix " _lsafRoleExists_= " exists;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);
    call symputx("_lsafRoleExists_", exists) ;

  run;

%mend lsaf_roleexists;
