%* lsaf_search                                                                                   *;
%*                                                                                               *;
%* Searches for an item in the repository. Although all parameters are optional, at least one    *;
%* parameter value must be specified to start a search.                                          *;
%*                                                                                               *;
%* The SAS Life Science Analytics Framework limits the number of items that are returned  from   *;
%* from a search. The macro variable _lsafSearchLimitReached_ returns a 0 if the limit is        *;
%* is not reached and a 1 if the limit is reached. If the limit is reached, there might be       *;
%* items in the repository that meet the search criteria. A more refined search is needed to     *;
%* retrieve those records.                                                                       *;
%*                                                                                               *;
%* With the exception of the location and datetime parameters, wildcards can be used in the      *;
%* seach values. For example:                                                                    *;
%*                                                                                               *;
%* lsaf_name=user* searches for any occurrence that starts with 'user'.                          *;
%* lsaf_name=*use* searches for any occurrence that contains 'use'.                              *;
%* lsaf_name=us?r searches for any occurence that starts with 'us', then any 1 character, and    *;
%* then ending in 'r'.                                                                           *;
%*                                                                                               *;
%* All datetime search parameters must be in one of the formats that are described below. Date   *;
%* values in formats other than the two described below result in an execution failure.          *;
%*          - SAS datetime20. format (such as 02NOV2013:09:33:22) - an offset of zero hours      *;
%*            from GMT is used.                                                                  *;
%*          - SAS E8601DZ25. format (such as 2013-11-02T09:33:22-04:00).                         *;
%*                                                                                               *;
%* Syntax:                                                                                       *;
%* %LSAF_SEARCH(<LSAF_LOCATION=lsaf-location, LSAF_NAME=lsaf-name, LSAF_TYPEID=lsaf-typeid,      *;
%*       LSAF_OWNER=lsaf-owner, LSAF_LASTMODIFIEDBY=lsaf-lastmodifiedby,                         *;
%*       LSAF_CREATEDBY=lsaf-createdby, LSAF_FROMCREATED=lsaf-fromcreated,                       *;
%*       LSAF_TOCREATED=lsaf-tocreated, LSAF_FROMLASTMODIFIED=lsaf-fromlastmodified,             *;
%*       LSAF_TOLASTMODIFIED=lsaf-tolastmodified, LSAF_CONTENT=lsaf-content=,                    *;
%*       LSAF_COPYTOWSSTATUS=lsaf-copy-to-workspace-status, SAS_DSNAME=work.lsafSearch,          *;
%*       LSAF_PRINTLIMITREACHEDWARNING = lsaf-print-limit-reached-warning>);                     *;
%*                                                                                               *;
%* @macvar _lsafRC_  The return code.                                                            *;
%* @macvar _lsafMsg_ The return message.                                                         *;
%* @macvar _lsafSearchLimitReached_ The search limit reached flag.                               *;
%*                                                                                               *;
%* @param lsaf_location - Optional - The location to search.                                     *;
%* @param lsaf_name - Optional - The name of the item.                                           *; 
%* @param lsaf_typeid - Optional - The type of the item. Type ids can be retrieved using the     *;
%*        macro lsaf_getsearchabletypes.                                                         *;
%* @param lsaf_owner - Optional - The owner of the item.                                         *;
%* @param lsaf_lastmodifiedby - Optional - The user account that last modified the item.         *;
%* @param lsaf_createdby - Optional - The user account that created the item.                    *;
%* @param lsaf_fromcreated - Optional - The earliest date on which the item was created. Must be *;
%*        in the Datetime format that is listed above.                                           *;
%* @param lsaf_tocreated - Optional - The latest date on which the item was created. Must be in  *;
%*        the Datetime format that is listed above.                                              *;
%* @param lsaf_fromlastmodified - Optional - The earliest date on which the item was last        *;
%*        modified. Must be in the Datetime format that is listed above.                         *;
%* @param lsaf_tolastmodified - Optional - The latest date on which the item was last modified.  *;
%*        Must be in the Datetime format that is listed above.                                   *;
%* @param lsaf_copytoworkspacestatus - Optional - The status of the Copy to Workspace property.  *;
%*            Valid values:                                                                      *;
%*            - ALLOW: Allow all.                                                                *;
%*            - WARN: Warn (excluding published jobs).                                           *;
%*            - DENY: Deny (excluding published jobs).                                           *;
%*            - DENY_ALL: Deny all.                                                              *;
%* @param lsaf_content - Optional - The content value of the file.                               *;
%* @param sas_dsname - Optional - The name of the output SAS data set to contain the metadata    *;
%*            for the items, specified as libref.dataset. The default value is                   *;
%*            WORK.LSAFSEARCH. The data set contains a row for each item in the list and         *;
%*            columns with the following names. The data set is be sorted by the path.           *;
%*            - name: The name of the item.                                                      *;
%*            - path: The path of the item.                                                      *;
%*            - itemType: The type of the item (such as folder or data set).                     *;
%*            - description: The description of the item.                                        *;
%*            - isContainer: Indicates whether the item is a container. Valid values:            *;
%*              0=no, 1=yes.                                                                     *;
%*            - isVersioned: Indicates whether the item is currently versioned. Valid values:    *;
%*              0=no, 1=yes.                                                                     *;
%*            - version: The version that is associated with the item.                           *;
%*            - isCheckedOut: Indicates whether the item is currently checked out. Valid values: *;
%*              0=no, 1=yes.                                                                     *;
%*            - isLocked: Indicates whether the item is currently locked. Valid values:          *;
%*              0=no, 1=yes.                                                                     *;
%*            - size: The size (in bytes) of the item. Thsi value is 0 for a container.          *;
%*            - createdBy: The ID of the user account that created the item.                     *;
%*            - created: The full date and time on which the item was created, represented       *;
%*              as a String.                                                                     *;
%*            - dateCreated: The date on which the item was created, represented as a SAS        *;
%*              Datetime format.                                                                 *;
%*            - lastModifiedBy: The ID of the user account that last modified the item.          *;
%*            - lastModified: The date on which the item was last modified, represented as a     *;
%*              String.                                                                          *;
%*            - dateLastModified: The date on which the item was last modified, represnted as    *;
%*              a SAS Datetime format.                                                           *;
%*            - propertiesLastModifiedBy: The ID of the user account that last modified the      *;
%*              properties for the item.                                                         *;
%*            - propertiesLastModified: The date on which the item properties were last          *;
%*              modified, represented as a String.                                               *;
%*            - datePropertiesLastModified: The date on which the item properties were last      *;
%*              modified, represented as a SAS Datetime format.                                  *;
%*            - state: This variable is for context items only. It indicates the state of the    *;
%*              context: ACTIVE or CLOSED.                                                       *;
%*            - signatureStatus: Indicates whether the item has electronic signatures attached   *;
%*              to it. Valild values:                                                            *;
%*              - NONE: There are no signatures associated with the file.                        *;
%*              - PREVIOUS: There is at least one previous version of the file that is           *;
%*                signed and the current version is not signed.                                  *;
%*              - CURRENT: The current version of the file is signed, and no previous version    *;
%*                is signed.                                                                     *;
%*              - CURRENT_AND_PREVIOUS: The current version of the file is signed, and there     *;
%*                is at least one previous version of the file that is signed.                   *;
%* @param lsaf_printLimitReachedWarning - Optional - Indicates whether to write to the SAS log   *;
%*            file the message about the server query limit being reached as a WARNING or a      *;
%*            NOTE. Valid values:                                                                *;
%*            - 0: Print the message as a NOTE.                                                  *;
%*            - 1 (Default): Print the message as a WARNING.                                     *;
%*            This option applies only when the _lsafSearchLimitReached_ macro is 1.             *;
%*                                                                                               *;
%* @macrotype LSAF                                                                               *;
%* @since 2.2                                                                                    *;
%* @exposure external                                                                            *;
%*                                                                                               *;
%* @history 2019-05-10  Initial development                                                      *;
%* @history 2019-11-01  update to handle a blank SAS output file.                                *;
%*                                                                                               *;

%macro lsaf_search(
  lsaf_location=,
  lsaf_name=,
  lsaf_typeid=,
  lsaf_owner=,
  lsaf_lastmodifiedby=,
  lsaf_createdby=,
  lsaf_fromcreated=,
  lsaf_tocreated=,
  lsaf_fromlastmodified=,
  lsaf_tolastmodified=,
  lsaf_content=,
  lsaf_copytoworkspacestatus=,
  sas_dsname=work.lsafSearch,
  lsaf_printLimitReachedWarning=1
  ) /des='Search for an item in the repository.';

  %if "&sas_dsname" EQ "" %then %let sas_dsname=work.lsafSearch;

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafRC_
          _lsaf_PrintMessage_
          _lsafSearchLimitReached_
  ;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data &sas_dsname(label="Created by &SYSMACRONAME"
                   keep=path name itemType isContainer description version isVersioned isCheckedOut isLocked 
                        signatureStatus size createdBy created dateCreated lastModifiedBy lastModified dateLastModified
                        propertiesLastModifiedBy propertiesLastModified datePropertiesLastModified state uppath);
    attrib returnCode                length=8
           message                   length=$200
           logMsgPrefix              length=$70
           logMsg                    length=$300
           logMsgPrimary             length=$200
           rowCount                  length=8

           path                      length=$2048    label="Item Path"
           name                      length=$256     label="Item Name"
           itemType                  length=$32      label="Item Type"
           isContainer               length=8        label="Is Item A Container?"

           description               length=$512     label="Description"
           version                   length=$32      label="Version Number"
           isVersioned               length=8        label="Is File Versioned"
           isCheckedOut              length=8        label="Is File Checked out"
           isLocked                  length=8        label="Is File Locked"
           signatureStatus           length=$32      label="Is File Signed"
           size                      length=8        label="Size In Bytes"

           createdBy                 length=$64      label="Userid that created the item"
           created                   length=$64      label="Item Creation Date"
           dateCreated                               label="Date the item was created (SAS DateTime Format)"

           lastModifiedBy            length=$64      label="Userid that last modified the item content"
           lastModified              length=$64      label="Last Content Modification Date"
           dateLastModified                          label="Date the item's content was last modified (SAS DateTime Format)"

           propertiesLastModifiedBy  length=$64      label="Userid that last modified the item properties"
           propertiesLastModified    length=$64      label="Last Properties Modification Date"
           datePropertiesLastModified                label="Date the item's properties were last modified (SAS DateTime Format)"

           state                     length=$64      label="State of a context item"

           createdTempSAS            length=$64      label="TEMPORARY - WILL BE DROPPED"
           lastModifiedTempSAS       length=$64      label="TEMPORARY - WILL BE DROPPED"
           uppath                    length=$2048    label="TEMPORARY - WILL BE DROPPED"
           propertiesLastModifiedTempSAS  length=$64 label="TEMPORARY - WILL BE DROPPED"
    ;

    format dateCreated
           dateLastModified
           datePropertiesLastModified datetime.
    ;

    rowCount=0;


    %***********************************;
    %* Declare the java items        *;
    %***********************************;
    declare javaobj srv("com/sas/lsaf/macro/content/repository/RepositoryService");
    declare javaobj results("com/sas/lsaf/macro/content/repository/SasRepositoryItemsSearchResult");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticVoidMethod("search", "%bquote(&lsaf_location)", "%bquote(&lsaf_name)", "%bquote(&lsaf_typeid)", "%bquote(&lsaf_owner)", "%bquote(&lsaf_lastmodifiedby)", 
                 "%bquote(&lsaf_createdby)", "%bquote(&lsaf_fromcreated)", "%bquote(&lsaf_tocreated)", "%bquote(&lsaf_fromlastmodified)", "%bquote(&lsaf_tolastmodified)", 
                 "%bquote(&lsaf_content)", "%bquote(&lsaf_copytoworkspacestatus)", results);

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);
    results.callBooleanMethod("isLimitReached", limitReached);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
    if (returnCode ne 0) then
    do;
      logMsgPrimary="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Cannot find search item(s)." ;
    end;
    else
    do;
      results.callIntMethod("getSize", rowCount);
      if limitReached then
      do;
        %lsaf_printwarningmessage(lsaf_printmessage=&lsaf_printlimitreachedwarning);
        logMsgPrimary="&_lsaf_PrintMessage_";
        logMsg=" Items were found matching the search criteria.";
      end;
      else 
      do;
        logMsgPrimary=logMsgPrefix;
        if (rowCount=0) 
          then logMsg = " No items were found matching the search criteria.";
          else logMsg = " Items were found matching the search criteria.";
      end;
      do row=0 to rowCount - 1;
        results.callStringMethod("getPath", row, path);
        results.callStringMethod("getName", row, name);
        results.callStringMethod("getDescription", row, description);
        results.callStringMethod("getVersion", row, version);
        results.callIntMethod("isContainer", row, isContainer);
        results.callIntMethod("isVersioned", row, isVersioned);
        results.callIntMethod("isCheckedOut", row, isCheckedOut);
        results.callIntMethod("isLocked", row, isLocked);
        results.callStringMethod("getSignatureStatus", row, signatureStatus);
        results.callLongMethod("getFileSize", row, size);
        results.callStringMethod("getItemType", row, itemType);
        results.callStringMethod("getCreated", row, created);
        results.callStringMethod("getCreatedBy", row, createdBy);
        results.callStringMethod("getLastModified", row, lastModified);
        results.callStringMethod("getLastModifiedBy", row, lastModifiedBy);
        results.callStringMethod("getPropertiesLastModified", row, propertiesLastModified);
        results.callStringMethod("getPropertiesLastModifiedBy", row, propertiesLastModifiedBy);
        results.callStringMethod("getState", row, state);

        results.callStringMethod("getCreatedSasFormat", row, createdTempSas);
        results.callStringMethod("getLastModifiedSasFormat", row, lastModifiedTempSas);
        results.callStringMethod("getPropertiesLastModifiedSasFormat", row, propertiesLastModifiedTempSAS);

        dateCreated=input(createdTempSAS, datetime.);
        dateLastModified=input(lastModifiedTempSas, datetime.);
        datePropertiesLastModified=input(propertiesLastModifiedTempSAS, datetime.);

        uppath=upcase(path) ;
        uppath=translate(uppath,'1','_');

        output;
      end;
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrimary " " logMsg;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    put logMsgPrefix " _lsafMsg_= " message ;
    put logMsgPrefix " _lsafRC_= " returnCode ;
    put logMsgPrefix " _lsafSearchLimitReached_= " limitReached;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);
    call symputx("_lsafRowCount_", rowCount);
    call symputx("_lsafSearchLimitReached_", limitReached);

  run;
  
  %if %symexist(_lsafRowCount_) %then
  %do;
    %let notesOption=%sysfunc(getoption(notes));
    options nonotes;

    %***********************************;
    %* Sort the data set               *;
    %***********************************;
    %if (&_lsafRowCount_ > 0) %then
    %do;
      proc sort data=&sas_dsname;
        by uppath;
      run;
    %end;
    %***********************************;
    %* Cleanup                         *;
    %***********************************;
    proc sql;
     alter table &sas_dsname
        drop uppath;
    quit;

    options &notesOption;
  %end;


%mend lsaf_search;
