%* lsaf_submitworkspacejob                                                                           *;
%*                                                                                                   *;
%* Submits a workspace job for immediate execution, and sets the SAS macro variable                  *;
%* _lsafWorkspaceJobSubmissionId_ to the identifier for the job submission. If the job cannot be     *;
%* submitted, _lsafWorkspaceJobSubmissionId_ is set to blank.                                        *;
%*                                                                                                   *;
%* A data set that represents the job parameter values to overwrite for the current job submission   *;
%* can be specified in the call to lsaf_submitworkspacejob. If no job parameters data set is         *;
%* specified, the default values for all job parameters are used.                                    *;
%*                                                                                                   *;
%* The output data sets from either lsaf_getjobparameters or lsaf_getworkspacejobparameters contain  *;
%* all of the data that is relevant to the parameters for a specific job and can be used as a source *;
%* for the input parameters data set for the lsaf_submitworkspacejob macro. Modify the data set with *;
%* any changes prior to calling lsaf_submitworkspacejob.                                             *;
%* - Rename the defaultValue variable to "value" or add a variable with the name "value" to          *;
%*   indicate the values to use for the current submission.                                          *;
%* - Default job parameter values are used for missing parameters (deleted rows).                    *;
%* - Specifying an empty string for a parameter value results in using a blank as the parameter      *;
%*   value for the current submission.                                                               *;
%* - Added parameters are ignored.                                                                   *;
%* - The parameters in the input data set are matched with the job parameters by name and type.      *;
%*   If no match is found for a parameter, its default value is used.                                *;
%* - Data set rows with duplicate values for parameter name cause an execution error.                *;
%* - DATE job parameter values must be in SAS datetime20. format (such as 02NOV2013:09:33:22), or    *;
%*   in SAS E8601DZ25. format (such as 2013-11-02T09:33:22-04:00). For SAS datetime20. format, an    *;
%*   offset of zero hours from GMT is used. To specify an offset from GMT, use the E8601DZ25.        *;
%*   format. Date values in formats other than the two described above result in an execution error. *;
%*                                                                                                   *;
%* Syntax:                                                                                           *;
%* %LSAF_SUBMITWORKSPACEJOB(LSAF_PATH=lsaf-path <, SAS_DSNAME=SAS-data-set>);                        *;
%*                                                                                                   *;
%* @macvar _lsafRC_  The return code.                                                                *;
%* @macvar _lsafMsg_ The return message.                                                             *;
%* @macvar _lsafWorkspaceJobSubmissionId_ The job submission identifier.                             *;
%*                                                                                                   *;
%* @param lsaf_path - Required - The path of the job in the workspace of the current user.           *;
%* @param sas_dsname - Optional - The name of the SAS data set that contains the job parameter       *;
%*            information, specified as libref.dataset. If not specified, the default job            *;
%*            parameters values are used. The data set must contain at least the characater          *;
%*            variables that are listed below. Additional variables are ignored.                     *;
%*            - name: The name of the parameter.                                                     *;
%*            - type: The parameter type. Valid values: CHARACTER, CHARACTER_MASKED, NUMERIC,        *;
%*              DATE, FILE, and FOLDER.                                                              *;
%*            - value: The value of the parameter to use in the current job submission.              *;
%*            - fileVersion: The file version to use, if the parameter is of type FILE.              *;
%*              For versioned files, a blank value results in the latest file version used.          *;
%*              For unversioned files, the value is ignored.                                         *;
%*            - includeSubFolders: Indicates whether the subfolders of a container are included      *;
%*              during job execution.                                                                *;
%*                                                                                                   *;
%* @macrotype LSAF                                                                                   *;
%* @since 1.5                                                                                        *;
%* @exposure external                                                                                *;
%*                                                                                                   *;
%* @history 2016-10-20  initial coding                                                               *;
%* @history 2019-02-11  update to 2.1                                                                *;
%* @history 2020-06-30  update service location                                                      *;
%*                                                                                                   *;

%macro lsaf_submitworkspacejob(
   lsaf_path=,
   sas_dsname=
   ) /des='Submits a workspace job for execution.';

   %****************************************;
   %* Initialize macro variables           *;
   %****************************************;
   %lsaf_getProductName();

   %global _lsafMsg_
           _lsafRC_
           _lsafWorkspaceJobSubmissionId_
   ; 
   %local logMsgPrefix
          logMsgPrimary
          macroName
          notesOption
          parametersProvided
          _tempRUNWSJOB_tempDS_
   ;

   %let _lsafWorkspaceJobSubmissionId_=;
   %let _lsafRC_ =%str(-999);
   %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

   %let macroName=%sysfunc(lowcase(&SYSMACRONAME));
   %let logMsgPrimary=%str(ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro: );
   %let logMsgPrefix=%str(NOTE: &_LSAF_PRODUCT_NAME_ Macro: );
   %let parametersProvided=0;

   %*************************************************************;
   %* Save original option setting so it can be reset properly. *;
   %*************************************************************;
   %let notesOption=%sysfunc(getoption(notes));

   %if ( %CMPRES(&sas_dsname) NE %str() ) %then
   %do;

      %***************************************************************************;
      %* Validate the parameters data set if supplied.                           *;
      %* sets global macro variable _lsafRC_=0 if validation successful           *;
      %***************************************************************************;
      %lsaf_validate_parameters_dataset(callingMacro=&macroName,
                                      sas_dsn=&sas_dsname);
      %if (&_lsafRC_ ne 0) %then
      %do;
         %GOTO PRINT_DSERROR_LOG_MSGS;
      %end;

      %let parametersProvided=1;
   %end;

   option notes;

   %**********************************************;
   %* Process the records                        *;
   %**********************************************;
   data _null_;
      attrib returnCode      length=8
             message         length=$200
             jobSubmissionId length=$525
             logMsgPrefix    length=$70
             logMsgPrimary   length=$70
             logMsg          length=$300
      ;

      %if (&parametersProvided) %then
      %do;
         set &sas_dsname end=eof;
         if (_n_ = 1) then
         do;
      %end;

      %***********************************;
      %* Declare the java objects, once  *;
      %***********************************;
      declare javaobj params("com/sas/lsaf/macro/execution/job/SasParameterValues");
      declare javaobj srv("com/sas/lsaf/macro/execution/jobsubmission/JobSubmissionService");
      declare javaobj results("com/sas/lsaf/macro/results/Result");

      %if (&parametersProvided) %then
      %do;
         end;
         params.callVoidMethod("addValue", kstrip(name), kstrip(value), kstrip(type), kstrip(fileVersion), includeSubfolders);
         if (eof);
      %end;

      %***********************************;
      %* Submit the job                  *;
      %***********************************;
      srv.callStaticStringMethod("submitWorkspaceJob", "%bquote(&lsaf_path)", params, results, jobSubmissionId);

      %***********************************;
      %* Retrieve the results            *;
      %***********************************;
      results.callIntMethod("getReturnCode", returnCode);
      results.callStringMethod("getReturnMessage", message);

      %***********************************;
      %* Process the results             *;
      %***********************************;
      logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
      if (returnCode ne 0) then
      do;
         logMsgPrimary="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
         logMsg=" Cannot submit job &lsaf_path.";
      end;
      else
      do;
         logMsgPrimary=logMsgPrefix;
         logMsg=" Submitted job &lsaf_path.";
      end;

      %***********************************;
      %* Print messages in log           *;
      %***********************************;
      put;
      put logMsgPrimary " " logMsg;
      put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
      put logMsgPrefix " _lsafMsg_= " message ;
      put logMsgPrefix " _lsafRC_= " returnCode ;
      if ("&sas_dsname" = "") 
        then put logMsgPrefix "No job parameters were supplied.";
        else put logMsgPrefix "sas_dsname = &sas_dsname";
      put logMsgPrefix "_lsafWorkspaceJobSubmissionId_= " jobSubmissionId;
      put;

      %***********************************;
      %* Set the macro variables         *;
      %***********************************;
      call symputx("_lsafRC_", returnCode);
      call symputx("_lsafMsg_", message);
      call symputx("_lsafWorkspaceJobSubmissionId_", strip(jobSubmissionId));

   run;

   options &notesOption;

   %return;

   %PRINT_DSERROR_LOG_MSGS:
      options &notesOption;
      %put &logMsgPrimary  &macroName ;
      %put &logMsgPrefix _lsafRC_= &_lsafRC_ ;
      %put &logMsgPrefix _lsafMsg_= &_lsafMsg_ ;
      %put &logMsgPrefix Parameter data set = &sas_dsname;
      %put;
      
%mend lsaf_submitworkspacejob;
