%* lsaf_typehascapability                                                                              *;
%*                                                                                                     *;
%* Indicates whether the type has the specified capability in the repository.                          *;
%*                                                                                                     *;
%* The SAS macro variable _lsafTypeHasCapability_ is set to a value of 1, if the specified item has    *;
%* the capability, 0 if it does not. In the event of an incorrect call to the macro that results in an *;
%* error, the return value of _lsafTypeHasCapability_ is set to -1. To get a list of the valid context *;
%* types, run %lsaf_getcontexttypes().                                                                 *;
%*                                                                                                     *;
%* Syntax:                                                                                             *;
%* %LSAF_TYPEHASCAPABILITY(LSAF_TYPEID=lsaf-typeid, LSAF_CAPABILITY=lsaf-capability);                  *;
%*                                                                                                     *;
%* @macvar _lsafRC_  The return code.                                                                  *;
%* @macvar _lsafMsg_ The return message.                                                               *;
%* @macvar _lsafTypeHasCapability_ Indicates whether the type has the capability.                      *;
%*                                                                                                     *;
%* @param lsaf_typeid - Required - The type identifier of the item.                                    *;
%* @param lsaf_capability - Required - The specified capability of the item.                           *;
%*                                                                                                     *;
%* @macrotype LSAF                                                                                     *;
%* @since 2.1                                                                                          *;
%* @exposure external                                                                                  *;
%*                                                                                                     *;
%* @history 2019-03-04  initial coding                                                                 *;
%* @history 2020-07-13  update for refactor of java service hierarchy.                                 *;
%*                                                                                                     *;

%macro lsaf_typehascapability(
  lsaf_typeid=,
  lsaf_capability=
  ) /des='Does the type have the specified capability?';

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafTypeHasCapability_
          _lsafMsg_
          _lsafRC_
  ;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data _null_;
    attrib returnCode          length=8;
    attrib message             length=$200;
    attrib logMsgPrefix        length=$70;
    attrib logMsg              length=$300;
    attrib typeHasCapability   length=3;
    attrib logMsgPrimary       length=$70;

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/lsaf/macro/core/type/TypeService");
    declare javaobj results("com/sas/lsaf/macro/results/Result");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticBooleanMethod("contextTypeHasCapability", "%bquote(&lsaf_typeid)", "%bquote(&lsaf_capability)", results, typeHasCapability);

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
    if (returnCode ne 0) then
    do;
      logMsgPrimary="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Cannot determine if the item &lsaf_typeid has capability &lsaf_capability.." ;
      typeHasCapability=-1;
    end;
    else
    do;
      logMsgPrimary=logMsgPrefix;
      if (typeHasCapability) 
        then logMsg=" The item &lsaf_typeid has capability: &lsaf_capability..";
        else logMsg=" The item &lsaf_typeid does not have capability: &lsaf_capability..";
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrimary " " logMsg;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    put logMsgPrefix " _lsafMsg_= " message;
    put logMsgPrefix " _lsafRC_= " returnCode;
    put logMsgPrefix " _lsafTypeHasCapability_= " typeHasCapability;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);
    call symputx("_lsafTypeHasCapability_", typeHasCapability);

  run;

%mend lsaf_typehascapability;
