%* lsaf_updateacls                                                                                 *;
%*                                                                                                 *;
%* Sets the permissions on SAS Life Science Analytics Framework items using a SAS data set as      *;
%* input.                                                                                          *;
%*                                                                                                 *;
%* The permission lists (known as Access Control Lists or ACLs) for multiple items can be modified *;
%* using a single data set. Access permissions are applied only to items in the repository         *;
%* within an organization.                                                                         *;
%*                                                                                                 *;
%* There are two types of access permission lists: current and default.                            *;
%*                                                                                                 *;
%* The current access permissions are applied to an individual item. The default access            *;
%* permissions apply only to container items (such as folders and projects) and define the         *;
%* permissions that are inherited by any new items that are added to the container.                *;
%*                                                                                                 *;
%* In the context of this macro, the access permission type is referred to as aclType. All records *;
%* in the input data set that pertain to the same repository item and aclType are processed        *;
%* together as a single ACL.                                                                       *;
%*                                                                                                 *;
%* A set of permissions for a specific repository item for an individual principal within an       *;
%* access permission type is referred to as an Access Control Entry (ACE). An ACE is               *;
%* represented as a single row in the input data set.                                              *;
%*                                                                                                 *;
%* An ACE is defined by the following characteristics:                                             *;
%*      path: The full path of the repository item (file or container).                            *;
%*      ACL type: The access permission type (DEFAULT or CURRENT).                                 *;
%*      ACL principal type: The type of the principal. The principal type can be Owner, Members,   *;
%*            User, or Group.                                                                      *;
%*      ACL principal name: The name of the user or group.                                         *;
%*            set of permissions: The specific permissions to assign (such as Administration, Read,*;
%*            Properties Write, Content Write, or Delete).                                         *;
%*                                                                                                 *;
%* The output data set from the lsaf_getacls macro contains all of the necessary variables         *;
%* and data needed as the input data set for the lsaf_updateACLs macro. Modify the data set with   *;
%* any changes before calling lsaf_updateacls.                                                     *;
%*                                                                                                 *;
%* To add a principal with specific access permissions to a repository item, add a new row to the  *;
%* data set for that principal.                                                                    *;
%*                                                                                                 *;
%* To modify the access permissions for a principal who is associated with a repository item,      *;
%* edit the appropriate row in the data set.                                                       *;
%*                                                                                                 *;
%* To remove a principal with specific access permissions from a repository item, delete the       *;
%* appropriate row from the data set. Note that the Owner and Members access control entries       *;
%* cannot be removed from the item. Removing them from the data set causes a failure.              *;
%*                                                                                                 *;
%* To maintain the existing permissions for a principal who is associated with a repository item,  *;
%* leave the record as it is in the data set. The record must remain, otherwise the access         *;
%* permissions for this principal are removed.                                                     *;
%*                                                                                                 *;
%* Failure and Message Handling                                                                    *;
%*                                                                                                 *;
%* The macro performs a series of validation procedures to determine that the input data set has   *;
%* the necessary structure for processing. If any of these validation procedures fail, an          *;
%* appropriate failure message is printed to the SAS log file and the macro stops additional       *;
%* processing.                                                                                     *;
%*                                                                                                 *;
%* In addition to other modifications, the macro sorts the input data set and stores the modified  *;
%* data in a temporary data set called WORK.__SASMACRO_ACLTEMP__. In the event that a failure      *;
%* occurs during processing, this temporary data set is retained as a reference. If all updates    *;
%* are performed successfully, this data set is deleted from the work library. A message is        *;
%* printed to the SAS log file that indicates whether the data set was retained or not. Regardless *;
%* of the success or failure of previous runs, this data set is deleted at the beginning of each   *;
%* new run of the macro. The data values and sort order of the original input data set are         *;
%* maintained.                                                                                     *;
%*                                                                                                 *;
%* All records that pertain to the same repository item and aclType are processed as a single      *;
%* access control list by the macro. Messages are printed to the SAS log file as each ACL is       *;
%* processed. If the update of an ACL fails, the macro stops processing.                           *;
%*                                                                                                 *;
%* If a failure occurs while processing a specific access control entry (such as a principal), the *;
%* macro stops processing and a failure message is written to the SAS log file. Successful updates *;
%* to ACLs made prior to the observation that contains the failure are maintained in SAS Life      *;
%* Science Analytics Framework. Changes made to access permissions for entries that are part of    *;
%* the same ACL that contained the observation with the failure are lost.                          *;                                                                                           *;
%*                                                                                                 *;
%* Valid values for permissions are as follows.  All values must be of type numeric.               *;
%*  1: ALLOW                                                                                       *;
%* -1: DENY                                                                                        *;
%*  0: Inherited                                                                                   *;
%*                                                                                                 *;
%* Syntax:                                                                                         *;
%* %LSAF_UPDATEACLS(SAS_DSNAME=SAS-data-set);                                                      *;
%*                                                                                                 *;
%* Example Code:                                                                                   *;
%* Get permissions for a folder:                                                                   *;
%*                                                                                                 *;
%* * start code example;                                                                           *;
%* %lsaf_getacls(lsaf_path=%str(/SAS/Study1/Files/folder1), lsaf_recursive=0);                     *;
%*                                                                                                 *;
%* data UpdateACLS;                                                                                *;
%* set lsafGetACLS end=eof;                                                                        *;
%*                                                                                                 *;
%* output;                                                                                         *;
%*                                                                                                 *;
%* if eof then do;                                                                                 *;
%*   * Adding user permission under DEFAULT;                                                       *;
%*   aclType = "DEFAULT";                                                                          *;
%*   aclPrincipalType = "USER";                                                                    *;
%*   aclPrincipalName = "user1";                                                                   *;
%*   grpSrcCtxt = "";                                                                              *;
%*   adminPermission = 0;                                                                          *;
%*   readPermission = 1;                                                                           *;
%*   writePropPerm = 1;                                                                            *;
%*   writeContentPerm = 1;                                                                         *;
%*   deletePermission = 1;                                                                         *;
%*   output;                                                                                       *;
%*                                                                                                 *;
%*   * Adding group permission under CURRENT;                                                      *;
%*   aclType = "CURRENT";                                                                          *;
%*   aclPrincipalType = "GROUP";                                                                   *;
%*   aclPrincipalName = "group1";                                                                  *;
%*   grpSrcCtxt = "/SAS";                                                                          *;
%*   adminPermission = 0;                                                                          *;
%*   readPermission = 1;                                                                           *;
%*   writePropPerm = 1;                                                                            *;
%*   writeContentPerm = 1;                                                                         *;
%*   deletePermission = 1;                                                                         *;
%*   output;                                                                                       *;
%* end;                                                                                            *;
%*                                                                                                 *;
%* run;                                                                                            *;
%*                                                                                                 *;
%* %lsaf_updateacls(sas_dsname=UpdateACLS);                                                        *;
%* * end code example;                                                                             *;
%*                                                                                                 *;
%* Note: User or group Access Control Entries (ACEs) with values of 0 for all permission variables *;
%*       will result in a validation error that will cause the processing to stop.                 *;
%* Note: Users and groups being added as ACEs must be members in the context containing the item   *;
%*       being updated.                                                                            *;
%*                                                                                                 *;
%* @macvar _lsafRC_  The return code.                                                              *;
%* @macvar _lsafMsg_ The return message.                                                           *;
%*                                                                                                 *;
%* @param sas_dsname - Required - the name of a SAS data set that contains the permission          *;
%*            information for all of the items to modify, specified as libref.dataset. The         *;
%*            data set must contain at least the variables listed below. Any additional            *;
%*            variables are ignored.                                                               *;
%*                                                                                                 *;
%*            path: The case-sensitive path of the item, which includes the item name.             *;
%*            aclType: Indicates the ACL to modify for the item and is case-insensitive. For       *;
%*                  file items, the variable must be present but the value must be CURRENT or      *;
%*                  blank. For container items, the value must be either DEFAULT or CURRENT.       *;
%*            aclPrincipalType: Indicates the type of ACE to update.                               *;
%*                  Values:  ACLOWNER | ACLMEMBERS | USER | GROUP (case-insensitive)               *;
%*                  Default: None                                                                  *;
%*            aclPrincipalName: The name of the principal ACE to update or add.                    *;
%*                  For aclPrincipalType ACLOWNER, this must have the value OWNER.                 *;
%*                  For aclPrincipalType ACLMEMBERS, this must have the value MEMBERS.             *;
%*                  For aclPrincipalType USER, this must be a valid userid (case-sensitive).       *;
%*                  For aclPrincipalType GROUP, this must be a valid group name (case-sensitive).  *;
%*            grpSrcCtxt: The case-sensitive path of the context in which the group is defined, if *;
%*                  aclPrincipalType is GROUP. For aclPrincipalType GROUP, this value is required, *;
%*                  otherwise the variable must be present but the value is ignored.               *;
%*            adminPermission: The permission to administer the item.                              *;
%*            readPermission: The permission to view the item.                                     *;
%*            writePropPerm: The permission to update the properties of the item.                  *;
%*            writeContentPerm: The permission to write to the content of the item.                *;
%*            deletePermission: The permission to the delete the item.                             *;
%*                                                                                                 *;
%* @macrotype LSAF                                                                                 *;
%* @since 1.2                                                                                      *;
%* @exposure external                                                                              *;
%*                                                                                                 *;
%* @history 2013-08-30  updates for standardization and coding conventions                         *;
%* @history 2016-08-26  rebrand as LSAF                                                            *;
%* @history 2016-02-09  updates for path sorting, notes to log, stop after first failure.          *;
%* @history 2019-02-05  update to 2.1                                                              *;
%* @history 2020-07-13  update for refactor of java service hierarchy.                             *;
%*                                                                                                 *;

%macro lsaf_updateacls(
  sas_dsname=
  ) /des='Set permissions on SAS Life Science Analytics Framework items.';

  %*************************************************************;
  %* Turn off the notes generated for all the pre-processing.  *;
  %* Save original option setting so it can be reset properly. *;
  %*************************************************************;
  %let notesOption=%sysfunc(getoption(notes));
  options nonotes;

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafRC_
  ;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %***************************************************************************;
  %* Cleanup from any previous runs that may have left temp data set behind. *;
  %***************************************************************************;
  %let _tempACLUPDATE_tempDS_=%str(__SASMacro_acltemp__);

  %if (%sysfunc(exist(&_tempACLUPDATE_tempDS_)) > 0) %then
  %do;
    %put NOTE: lsaf_updateacls: Deleting temporary data set &_tempACLUPDATE_tempDS_ before processing.;
    proc datasets library=work nolist;
      delete &_tempACLUPDATE_tempDS_;
    quit; run;
  %end;

  %***************************************************************************;
  %* Set local macro variables for processing.                               *;
  %*                                                                         *;
  %* setting a new macro variable for operation success because there are    *;
  %* operations in updateACL that could set _lsafRC_ to something other than *;
  %* 0 but still not be a failure.                                           *;
  %***************************************************************************;
  %local logMsgPrefix logMsgPrimary tempDsMsg;
  %let logMsgPrimary=%str(ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro: );
  %let logMsgPrefix=%str(NOTE: &_LSAF_PRODUCT_NAME_ Macro: );
  %let tempDsMsg=%str(Temporary sorted data set work.&_tempACLUPDATE_tempDS_ will not be created.);

  %***************************************************************************;
  %* Validate the input data set                                             *;
  %* sets global macro variable _lsafRC_=0 if validation successful          *;
  %***************************************************************************;
  %let macroName=%sysfunc(lowcase(&SYSMACRONAME));
  %lsaf_validate_sasdataset(callingMacro=&macroName,
        sas_dsn=&sas_dsname,
        required_variables=%str(path aclType aclPrincipalType aclPrincipalName grpSrcCtxt
                    adminPermission readPermission writePropPerm writeContentPerm deletePermission),
        required_numeric_variables=%str(adminPermission readPermission writePropPerm writeContentPerm deletePermission)
        );
  %if (&_lsafRC_ ne 0) %then
  %do;
    options notes;
    %put &logMsgPrimary &macroName - &tempDsMsg;
    %put &logMsgPrefix Input data set = &sas_dsname;
    %put &logMsgPrefix _lsafRC_= &_lsafRC_ ;
    %put &logMsgPrefix _lsafMsg_= &_lsafMsg_ ;
    %put;
	options &notesOption;
    %return;
  %end;

  %***************************************************************************;
  %* If data set is valid, process it.                                       *;
  %***************************************************************************;
  %put NOTE: &macroName: Processing data set &sas_dsname;

  %***************************************************************************;
  %* Clean the data set values                                               *;
  %* The data set that the user sends in can have very corrupt data.  Need   *;
  %* to clean it up (without changing the values) as much as possible before *;
  %* processing.                                                             *;
  %* Note: kstrip() of the required variables here did not help with sending *;
  %*       them into the macro.  Still had to do it there so dropped it from *;
  %*       here.                                                             *;
  %***************************************************************************;
  data &_tempACLUPDATE_tempDS_;
    set &sas_dsname;

    %*************************************************************************;
    %* Upcase values that should be case insensitive.  This is also for      *;
    %* sorting purposes.                                                     *;
    %* NOTE: PrincipalName and all paths must be case-sensitive              *;
    %*************************************************************************;
    aclType = upcase(aclType);
    aclPrincipalType = upcase(aclPrincipalType);
  run;

  %************************;
  %* Sort the data        *;
  %************************;
  proc sort data=&_tempACLUPDATE_tempDS_
             out=&_tempACLUPDATE_tempDS_;
    by path aclType;
  run;

  option notes;

  %**********************************************;
  %* Process the records                        *;
  %**********************************************;
  data _null_;
    attrib returnCode               length=8;
    attrib message                  length=$200;
    attrib emptyAceCount            length=8;
    attrib aclUpdateCount           length=8;

    set &_tempACLUPDATE_tempDS_ end=eof;
    by path aclType;

    %***********************************;
    %* Declare the java items, once    *;
    %***********************************;
    if (_n_ = 1) then
    do;
      declare javaobj acl("com/sas/lsaf/macro/security/acl/SasAcl");
      declare javaobj srv("com/sas/lsaf/macro/security/acl/AclService");
      declare javaobj results("com/sas/lsaf/macro/security/acl/SasUpdateAclResult");

      put "&logMsgPrefix * " %sysfunc(lowcase("&SYSMACRONAME"));
      put "&logMsgPrefix * Input data set = &sas_dsname";
    end;

    if (first.aclType) then acl.callVoidMethod("resetAcl", kstrip(path), kstrip(aclType));

    %***********************************;
    %* Create a new ace                *;
    %***********************************;
    acl.callVoidMethod("addAce", kstrip(aclPrincipalType), kstrip(aclPrincipalName), kstrip(grpSrcCtxt),
                        adminPermission, readPermission, writePropPerm, writeContentPerm, deletePermission);

    if (last.aclType) then
    do;

      %***********************************;
      %* Call the update                 *;
      %***********************************;
      put "&logMsgPrefix * Processing " aclType "ACL for path " path;
      srv.callStaticVoidMethod("updateAcl", acl, results);

      %***********************************;
      %* Retrieve the results            *;
      %***********************************;
      results.callIntMethod("getReturnCode", returnCode);
      results.callStringMethod("getReturnMessage", message);

      %***********************************;
      %* Process the results             *;
      %***********************************;
      if (returnCode ne 0) then
      do;
        put "&logMsgPrimary * Unable to process " aclType "ACL for path " path;
        put "&logMsgPrefix * _lsafMsg_ = " message;
        put "&logMsgPrefix * _lsafRC_ = " returnCode;
        put;
        GOTO ENDPROCESS;
      end;
    end;

    if (eof) then GOTO ENDPROCESS;

    return;

    ENDPROCESS:
    results.callIntMethod("getAclUpdateCount", aclUpdateCount);
    results.callIntMethod("getBlankAceCount", emptyAceCount);
    if (emptyAceCount gt 0) 
      then put "WARNING: &_LSAF_PRODUCT_NAME_ Macro: * " emptyAceCount "record(s) had no permissions "
               "explicitly set for the ACL being processed.  The ACE for those records was not "
               "added or was removed from the ACL.";

    put "&logMsgPrefix * Successfully processed " aclUpdateCount "ACLs";
    put " ";

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);
    STOP;
  run;

  %************************************************************;
  %* Cleanup temporary data set if the operation did not fail *;
  %************************************************************;
  %if (&_lsafRC_ eq 0) %then
  %do;
      %if (%sysfunc(exist(&_tempACLUPDATE_tempDS_))>0) %then
      %do;
        %put &logMsgPrefix * %sysfunc(lowcase(&SYSMACRONAME)) is deleting temporary data set &_tempACLUPDATE_tempDS_;
         proc datasets library=work nolist;
            delete &_tempACLUPDATE_tempDS_;
         quit; run;
      %end;
  %end;
  %else
  %do;
      %put WARNING: &_LSAF_PRODUCT_NAME_ Macro: * Temporary sorted data set work.&_tempACLUPDATE_tempDS_ is not being deleted.;
  %end;

  options &notesOption;

%mend lsaf_updateacls;
