%* lsaf_updatefile                                                                                      *;
%*                                                                                                      *;
%* Updates the contents of a file in the repository.                                                    *;
%*                                                                                                      *;
%* If the file is versioned, a new version is created. If the file is not versioned, the current        *;
%* contents are overwritten.                                                                            *;
%*                                                                                                      *;
%* NOTE: This macro is supported only from a PC SAS session. Using this macro within an active          *;
%* SAS Life Science Analytics Framework application session results in a failure.                       *;
%*                                                                                                      *;
%* Syntax:                                                                                              *;
%* %LSAF_UPDATEFILE(LOCAL_PATH=local-path, LSAF_PATH=lsaf-path <, LSAF_COMMENT=lsaf-comment>,           *;
%* LSAF_VERSIONTYPE=lsaf-version-type, LSAF_CUSTOMVERSION=lsaf-customversion>);                         *;
%*                                                                                                      *;
%* @macvar _lsafRC_  The return code.                                                                   *;
%* @macvar _lsafMsg_ The return message.                                                                *;
%*                                                                                                      *;
%* @param local_path - Required - The absolute path to the file on the local computer.                  *;
%* @param lsaf_path - Required - The path of the file in the repository to update. The file must        *;
%*            exist and must not be a container.                                                        *;
%* @param lsaf_versiontype - Conditional - The type of version to create, if the file to update is      *;
%*            versioned. Valid values: MAJOR, MINOR, and CUSTOM. If not provided, the system default    *;
%*            will be applied. This value is ignored for an unversioned file.                           *;
%* @param lsaf_customversion - Conditional - The specific version to create. If lsaf_versiontype is     *;
%*            CUSTOM, the  parameter is required. Otherwise, it is ignored.                             *;
%*            Custom version numbers must be in the valid format, which includes a decimal point,       *;
%*            such as 1.0 or 4.2.                                                                       *;
%* @param lsaf_comment - Optional - The comment associated with the new file or file version.           *;
%*                                                                                                      *;
%* @macrotype LSAF                                                                                      *;
%* @since 1.1                                                                                           *;
%* @exposure external                                                                                   *;
%*                                                                                                      *;
%* @history 2013-08-30  updates for standardization and coding conventions                              *;
%* @history 2016-08-26  rebrand as LSAF                                                                 *;
%* @history 2018-01-09  updates to documentation for LSAF 5.1                                           *;
%* @history 2019-02-19  update to 2.1                                                                   *;
%* @history 2019-10-30  update to allow custom version                                                  *;
%* @history 2020-11-18  updated for version handling.                                                   *;
%*                                                                                                      *;

%macro lsaf_updatefile(
  local_path=,
  lsaf_path=,
  lsaf_comment=,
  lsaf_versiontype=,
  lsaf_customversion=
  ) /des='Update the contents of an existing SAS Life Science Analytics Framework item.';

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafRC_
  ;

  %let _lsafRC_=%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data _null_;
    attrib returnCode      length=8;
    attrib message         length=$200;
    attrib logMsgPrefix    length=$70;
    attrib logMsgPrimary   length=$70;
    attrib logMsg          length=$300;

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/lsaf/macro/content/repository/RepositoryService");
	declare javaobj checkin("com/sas/lsaf/macro/content/repository/SasFileCheckinParameters");
    declare javaobj results("com/sas/lsaf/macro/results/Result");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
	checkin.callVoidMethod("setData","","%bquote(&lsaf_versiontype)","%bquote(&lsaf_customversion)","%bquote(&lsaf_comment)");

    srv.callStaticVoidMethod("updateFile", "%bquote(&local_path)", "%bquote(&lsaf_path)", checkin, results);

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
    if (returnCode eq 0) then
    do;
      logMsgPrimary=logMsgPrefix;
      logMsg=" The file was updated: &lsaf_path.." ;
    end;
    else
    do;
      logMsgPrimary="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Cannot update file: &lsaf_path.." ;
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrimary " " logMsg;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    put logMsgPrefix " _lsafMsg_= " message ;
    put logMsgPrefix " _lsafRC_= " returnCode ;
    put logMsgPrefix " local_path= &local_path" ;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);

  run;

%mend lsaf_updatefile ;
