%* lsaf_updateprocessflowdata                                                                    *;
%*                                                                                               *;
%* Updates the metadata for the specified process flow.                                          *;
%*                                                                                               *;
%* Syntax:                                                                                       *;
%* %LSAF_UPDATEPROCESSFLOWDATA(LSAF_PATH=lsaf-path, LSAF_processflow=process-flow-name,          *;
%*            SAS_DSNAME=SAS-data-set);                                                          *;
%*                                                                                               *;
%* @macvar _lsafRC_  The return code.                                                            *;
%* @macvar _lsafMsg_ The return message.                                                         *;
%*                                                                                               *;
%* @param lsaf_path - Required - The path of the context in which the process flow is located.   *;
%* @param lsaf_processflow - Required - The name of the process flow.                            *;
%* @param sas_dsname - Required - The name of a SAS data set that contains the process flow data *;
%*            values to set on the process flow, specified as libref.dataset.                    *;
%*            The data set must contain at least the variables listed below. Additional          *;
%*            variables are ignored.                                                             *;
%*            - dataID: The ID for the data element property.                                    *;
%*            - dataValue: The value to set.                                                     *;
%*            The output data set from the lsaf_getprocessflowdata macro can be used to          *;
%*            determine the ID and current values of the process flow data.                      *;
%*            Rows where the ID matches an existing process flow data element are used to        *;
%*            attempt to update the element. If the specified value is invalid for the element   *;
%*            type, the update is aborted and the appropriate error message is printed to the    *;
%*            SAS log file.                                                                      *;
%*            Values of type DATE must be one of the following formats. Date values in formats   *;
%*            other than the two described below result in an execution error.                   *;
%*            - SAS datetime20. format (such as 02NOV2013:09:33:22) -  an offset of zero hours   *;
%*              from GMT is used.                                                                *;
%*            - SAS E8601DZ25. format (such as 2013-11-02T09:33:22-04:00).                       *;
%*                                                                                               *;
%* @macrotype LSAF                                                                               *;
%* @since 1.5                                                                                    *;
%* @exposure external                                                                            *;
%*                                                                                               *;
%* @history 2016-12-06  initial programming                                                      *;
%* @history 2019-02-12  update to 2.1                                                            *;
%*                                                                                               *;

%macro lsaf_updateprocessflowdata(
   lsaf_path=,
   lsaf_processflow=,
   sas_dsname=_lsaf_updatepf_donotupdate_
   ) /des='Updates the process data for a process flow in the specified SAS Life Science Analytics Framework context.';

   %****************************************;
   %* Initialize macro variables           *;
   %****************************************;
   %lsaf_getProductName();

   %global _lsafMsg_
           _lsafRC_
           notesOption
   ;

   %local logMsgPrefix
          logMsgPrimary
          macroName
   ;

   %let _lsafRC_ =%str(-999);
   %let _lsafMsg_=%str(The SAS Macro did not execute properly. Unknown err%str(or).);
   %let logMsgPrimary=%str(ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro: );
   %let logMsgPrefix=%str(NOTE: &_LSAF_PRODUCT_NAME_ Macro: );
   %let macroName=%sysfunc(lowcase(&SYSMACRONAME));

   %*************************************************************;
   %* Save original option setting so it can be reset properly. *;
   %*************************************************************;
   %let notesOption=%sysfunc(getoption(notes));

   %if (&sas_dsname=_lsaf_updatepf_donotupdate_) %then
   %do;
     %let _lsafMsg_= %str(No data has been provided for the update.);
     %GOTO PRINT_DSERROR_LOG_MSGS;
   %end;

   %if ( %CMPRES(&sas_dsname) NE %str(_lsaf_updatepf_donotupdate_) ) %then
   %do;
     %***************************************************************************;
     %* Validate the Process Flow Data data set if supplied.                    *;
     %* sets global macro variable _lsafRC_=0 if validation successful          *;
     %***************************************************************************;
     %lsaf_validate_sasdataset(callingMacro=&macroName,
                               sas_dsn=&sas_dsname,
                               required_variables=%str(dataID dataValue),
                               allow_empty_datasets=0);
     %if (&_lsafRC_ ne 0) %then
     %do;
       %let _lsafMsg_= sas_dsname: &_lsafMsg_;
       %GOTO PRINT_DSERROR_LOG_MSGS;
     %end;
   %end;

   option notes;

   %**********************************************;
   %* Process the records                        *;
   %**********************************************;
   data _null_;
      attrib returnCode      length=8
              message        length=$200
              logMsgPrefix   length=$70
              logMsgPrimary  length=$70
              logMsg         length=$300
      ;

      set &sas_dsname end=eof;
      if (_n_ EQ 1) then
      do;
        %***********************************;
        %* Declare the java objects        *;
        %***********************************;
        declare javaobj srv("com/sas/lsaf/macro/workflow/processflow/ProcessFlowService");
        declare javaobj results("com/sas/lsaf/macro/results/Result");
        declare javaobj data("com/sas/lsaf/macro/common/SasProperties");
      end;

      data.callVoidMethod("addProperty", kstrip(dataID), kstrip(dataValue));

      if (eof);

      %***********************************;
      %* Attempt the operation           *;
      %***********************************;
      srv.callStaticVoidMethod("updateProcessFlowData", "%bquote(&lsaf_path)", "%bquote(&lsaf_processflow)",
                                           data, results);

      %***********************************;
      %* Retrieve the results            *;
      %***********************************;
      results.callIntMethod("getReturnCode", returnCode);
      results.callStringMethod("getReturnMessage", message);

      %***********************************;
      %* Process the results             *;
      %***********************************;
      logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
      if (returnCode ne 0) then
      do;
        logMsgPrimary="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
        logMsg=" Cannot update process data for &lsaf_processflow in &lsaf_path.." ;
      end;
      else
      do;
        logMsgPrimary=logMsgPrefix;
        logMsg=" The process data were updated for &lsaf_processflow in &lsaf_path.." ;
      end;

      %***********************************;
      %* Print messages in log           *;
      %***********************************;
      put;
      put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
      put logMsgPrimary " " logMsg;
      put logMsgPrefix " _lsafMsg_= " message ;
      put logMsgPrefix " _lsafRC_= " returnCode ;
      put;

      %***********************************;
      %* Set the macro variables         *;
      %***********************************;
      call symputx("_lsafRC_", returnCode);
      call symputx("_lsafMsg_", message);

   run;

   options &notesOption;

   %return;

   %PRINT_DSERROR_LOG_MSGS:
      options &notesOption;
      %put &logMsgPrimary _lsafMsg_= &_lsafMsg_ ;
      %put &logMsgPrefix  &macroName ;
      %put &logMsgPrefix _lsafRC_= &_lsafRC_ ;
      %put;

%mend lsaf_updateprocessflowdata;
