%* lsaf_updateworkspacejob                                                                           *;
%*                                                                                                   *;
%* Updates a job in workspace of the current user.                                                   *;
%*                                                                                                   *;
%* Data sets that represent the job data to update must be specified in the call to                  *;
%* lsaf_updateworkspacejob. The information in the data sets replaces the data of the existing       *;
%* job. The output data sets from the following macros contain all of the data that is relevant to   *;
%* the job properties that can be updated. See the macro documentation for more details.             *;
%* - lsaf_getjobinfo or lsaf_getworkspacejobinfo                                                     *;
%* - lsaf_getjobprograms or lsaf_getworkspacejobprograms                                             *;
%* - lsaf_getjobparameters or lsaf_getworkspacejobparameters                                         *;
%* - lsaf_getjobsetup or lsaf_getworkspacejobsetup                                                   *;
%* - lsaf_getjobcheckin or lsaf_getworkspacejobcheckin                                               *;
%*                                                                                                   *;
%* Syntax:                                                                                           *;
%* %LSAF_UPDATEWORKSPACEJOB(LSAF_PATH=lsaf-path <SAS_DSNAME_INFO=SAS-data-set-info,                  *;
%*      SAS_DSNAME_PROGRAMS=SAS-data-set-jobprograms, SAS_DSNAME_PARAMETERS=SAS-data-set-parameters, *;
%*      SAS_DSNAME_SETUP=SAS-data-set-setup, SAS_DSNAME_CHECKIN=SAS-data-set-checkin>);              *;
%*                                                                                                   *;
%* @macvar _lsafRC_  The return code.                                                                *;
%* @macvar _lsafMsg_ The return message.                                                             *;
%*                                                                                                   *;
%* @param lsaf_path - Required - The path for the job to update in the workspace of the current user.*;
%* @param sas_dsname_info - Optional - The name of the SAS data set that contains the job            *;
%*            information, specified as libref.dataset. The data set must contain at least the       *;
%*            character variables listed below. Additional variables are ignored.                    *;
%*            - name: The name of the job property.                                                  *;
%*            - value: The value of the property.                                                    *;
%*            The information that can be specified is DOCUMENTATION, LOGSLOCATION, RESULTSLOCATION, *;
%*            MANIFESTLOCATION, NEWOUTPUTS_CUSTOMVERSION, NEWOUTPUTS_ENABLEVERSION, and              *;
%*            NEWOUTPUTS_VERSIONTYPE. All other values listed in the data set are ignored. All data  *;
%*             are case-insensitive. If a name is listed more than once, the last value processed is *;
%*             used. If a name is omitted, the value in the job is not updated.                      *;
%* @param sas_dsname_programs - Optional - The name of the SAS data set that contains the list of    *;
%*            the paths to the SAS programs to run, specified as libref.dataset. If the data set is  *;
%*            empty, all programs are removed from the job. The data set must contain at least the   *;
%*            character variables listed below. Additional variables are ignored.                    *;
%*            - programPath: The path of the SAS program file.                                       *;
%*            - programVersion: The version of the SAS program file. Blank values specify the        *;
%*              latest file version.                                                                 *;
%* @param sas_dsname_parameters - Optional - The name of the SAS data set that contains the job      *;
%*            parameter information, specified as libref.dataset. If the data set is empty, all job  *;
%*            parameters are removed from the job. The data set must contain at least the            *;
%*            character variables listed below. Additional variables are ignored.                    *;
%*            - name: The name of the parameter.                                                     *;
%*            - label: The label for the parameter.                                                  *;
%*            - type: The parameter type.  Valid values are CHARACTER, CHARACTER_MASKED, NUMERIC,    *;
%*              DATE, FILE, and FOLDER.                                                              *;
%*            - defaultValue: The value to set as the default value for the parameter.               *;
%*              DATE job parameter values must be one of the following formats. Date values in       *;
%*              formats other than the two described below result in an execution error.             *;
%*                  - SAS datetime20. format (such as 02NOV2013:09:33:22) -  an offset of zero hours *;
%*                    from GMT is used.                                                              *;
%*                  - SAS E8601DZ25. format (such as 2013-11-02T09:33:22-04:00).                     *;
%*            - fileVersion: The file version to use, if the parameter is of type FILE. Blank        *;
%*              values specify the latest file version.                                              *;
%*            - includeSubFolders (numeric): Indicates whether the container includes subfolders.    *;
%*              Valid values: 0 and 1.                                                               *;
%*            Data set rows with duplicate values for parameter name result in a data set            *;
%*            validation error.                                                                      *;
%* @param sas_dsname_setup - Optional - The name of the SAS data set that contains the setup list of *;
%*            the job , specified as libref.dataset. If the data set is empty, all job setup items   *;
%*            are removed from the job. The data set must contain at least the variables listed      *;
%*            below as the type specified. Additional variables are ignored.                         *;
%*            - setupPath (character): The path of the file or container to use as setup.            *;
%*            - setupType (character): The type of the setup item. Valid values: FILE and CONTAINER. *;
%*            - setupVersion (character): The version of the FILE item. Blank values specify         *;
%*              the latest file version. The value is ignored for setup items of type CONTAINER.     *;
%*            - setupIncludeSubFolders (numeric): Indicates whether the setup item folder includes   *;
%*              subfolders. Valid values: 0 and 1. The value is ignored for setup items of type FILE.*;
%*            - setupIncludeFiles (numeric): Indicates whether the setup includes files.             *;
%*              Valid values: 0 and 1.                                                               *;
%*            Data set rows with duplicate values for setupPath result in a data set validation      *;
%*            error.                                                                                 *;
%* @param sas_dsname_checkin - Optional - The name of the SAS data set that contains the list of the *;
%*            job checkin locations, specified as libref.dataset. If the data set is empty, all job  *;
%*            checkin locations are removed from the job. The data set must contain at least the     *;
%*            variables listed below as the type specified. Additional variables are ignored.        *;
%*            - checkinPath (character): The checkin location path of the container.                 *;
%*            - checkinIncludeSubFolders (numeric): Indicates whether the checkin location container *;
%*              includes subfolders. Valid values: 0 and 1.                                          *;
%*            Data set rows with duplicate values for checkinPath result in a data set validation    *;
%*            error.                                                                                 *;
%*                                                                                                   *;
%* @since 1.5                                                                                        *;
%* @exposure external                                                                                *;
%*                                                                                                   *;
%* @history 2016-09-23  initial coding                                                               *;
%* @history 2019-02-11  update to 2.1                                                                *;
%* @history 2020-08-11  update parameters and column names to reflect LSAF 5.3 compatibility.        *;
%* @history 2021-04-07  update for java code refactor                                                *;
%*                                                                                                   *;

%macro lsaf_updateworkspacejob(
   lsaf_path=,
   sas_dsname_info=%str(_lsaf_updateworkspacejob_ds_notprovided_),
   sas_dsname_programs=%str(_lsaf_updateworkspacejob_ds_notprovided_),
   sas_dsname_parameters=%str(_lsaf_updateworkspacejob_ds_notprovided_),
   sas_dsname_setup=%str(_lsaf_updateworkspacejob_ds_notprovided_),
   sas_dsname_checkin=%str(_lsaf_updateworkspacejob_ds_notprovided_)
   ) /des='Update a job in the workspace.';

   %****************************************;
   %* Initialize macro variables           *;
   %****************************************;
   %lsaf_getProductName();

   %global _lsafMsg_
           _lsafRC_
           notesOption
   ;

   %local haveData
          haveObs
          logMsgPrefix
          logMsgPrimary
          macroName
   ;

   %let haveData =%str(1);
   %let _lsafRC_ =%str(-999);
   %let _lsafMsg_=%str(The SAS Macro did not execute properly. Unknown err%str(or).);
   %let _lsafWorkspaceJobPath_=;

   %let notesOption=%sysfunc(getoption(notes));

   %*********************************************************************************;
   %* Validate all input data sets and update master data set to set server values. *;
   %* Initializes _lsafMASTERJobDATA_tempDS_ input macro variable.                  *;
   %*********************************************************************************;
   %let macroName=%sysfunc(lowcase(&SYSMACRONAME));
   %lsaf_process_jobdatasets(callingMacro=&macroName,
                              sas_dsname_info=&sas_dsname_info,
                              sas_dsname_programs=&sas_dsname_programs,
                              sas_dsname_parameters=&sas_dsname_parameters,
                              sas_dsname_setup=&sas_dsname_setup,
                              sas_dsname_checkin=&sas_dsname_checkin
   );
   %let logMsgPrimary=%str(ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro: );
   %let logMsgPrefix=%str(NOTE: &_LSAF_PRODUCT_NAME_ Macro: );

   options notes;
   %if (&_lsafRC_ ne 0) %then
   %do;
     %GOTO PRINT_DSERROR_LOG_MSGS;
   %end;

   %if (&_lsafMASTERJobDATA_tempDS_ EQ %str(_null_)) %then
   %do;
     %LET haveData=0;
   %end;

   %LET haveObs=0;

   %let _lsafMASTERJobDATA_ID_ = %sysfunc( open(&_lsafMASTERJobDATA_tempDS_) );
   %let _lsafMASTERJobDATA_NOBS_ = %sysfunc( attrn(&_lsafMASTERJobDATA_ID_,nobs) );
   %let _lsafMASTERJobDATA_RC_ = %sysfunc( close(&_lsafMASTERJobDATA_ID_) );

   %if (&_lsafMASTERJobDATA_NOBS_ GE 1) %then
   %do;
     %LET haveObs=1;
   %end;

   %let _lsafRC_ =%str(-999);

   %**********************************************;
   %* Process the records                        *;
   %**********************************************;
   data _null_;
      attrib returnCode      length=8
             message         length=$200
             logMsgPrefix    length=$70
             logMsgPrimary   length=$70
             logMsg          length=$300
      ;

      %if (&haveObs EQ 1) %then
      %do;
        set &_lsafMASTERJobDATA_tempDS_ end=eof;
        if (_n_ EQ 1) then
        do;
      %end;
         %***********************************;
         %* Declare the java objects, once  *;
         %***********************************;
         declare javaobj srv("com/sas/lsaf/macro/execution/job/JobService");
         declare javaobj results("com/sas/lsaf/macro/results/Result");
         declare javaobj info("com/sas/lsaf/macro/execution/job/SasJobInfo");
         declare javaobj programs("com/sas/lsaf/macro/execution/job/SasJobPrograms");
         declare javaobj params("com/sas/lsaf/macro/execution/job/SasJobParameters");
         declare javaobj setup("com/sas/lsaf/macro/execution/job/SasJobPublishSetups");
         declare javaobj checkin("com/sas/lsaf/macro/execution/job/SasJobPublishCheckinLocations");

         %if (&haveData EQ 1) %then
         %do;
           setData="1";
           %if (&sas_dsname_programs NE %str(_lsaf_updateworkspacejob_ds_notprovided_)) %then
           %do;
             programs.callVoidMethod("setUpdateData", kstrip(setData));
           %end;
           %if (&sas_dsname_parameters NE %str(_lsaf_updateworkspacejob_ds_notprovided_)) %then
           %do;
             params.callVoidMethod("setUpdateData", kstrip(setData));
           %end;
           %if (&sas_dsname_setup NE %str(_lsaf_updateworkspacejob_ds_notprovided_)) %then
           %do;
             setup.callVoidMethod("setUpdateData", kstrip(setData));
           %end;
           %if (&sas_dsname_checkin NE %str(_lsaf_updateworkspacejob_ds_notprovided_)) %then
           %do;
             checkin.callVoidMethod("setUpdateData", kstrip(setData));
           %end;
         %end;

      %if (&haveObs EQ 1) %then
      %do;
        end;  * ends if (_n_=1);
        if (source EQ "&source_info") then
        do;
          name=kstrip(lowcase(name));
          select (name);
            when ("documentation") info.callVoidMethod("setDescription", kstrip(value));
            when ("logslocation") info.callVoidMethod("setLogsLocation", kstrip(value));
            when ("resultslocation") info.callVoidMethod("setResultsLocation", kstrip(value));
            when ("manifestlocation") info.callVoidMethod("setManifestLocation", kstrip(value));
            when ("newoutputs_customversion") info.callVoidMethod("setCustomVersion", kstrip(value));
            when ("newoutputs_enableversion") info.callVoidMethod("setEnableVersioning", kstrip(value));
            when ("newoutputs_versiontype") info.callVoidMethod("setVersionType", kstrip(upcase(value)));
            otherwise;
          end;
        end;
        else if (source EQ "&source_parms") then
        do;
          params.callVoidMethod("addValue", kstrip(name), kstrip(pLabel), kstrip(type), kstrip(value), kstrip(version), includeSubFolders);
        end;
        else if (source EQ "&source_programs") then
        do;
          programs.callVoidMethod("addItem", kstrip(value), kstrip(version));
        end;
        else if (source EQ "&source_setup") then
        do;
          setup.callVoidMethod("addItem", kstrip(value), kstrip(type), kstrip(version), includeSubFolders, includeFiles);
        end;
        else if (source EQ "&source_checkin") then
        do;
          checkin.callVoidMethod("addItem", kstrip(value), includeSubFolders);
        end;

        %***********************************;
        %* Attempt the operation           *;
        %***********************************;
        if (eof);
      %end;

      srv.callStaticVoidMethod("updateWorkspaceJob", "%bquote(&lsaf_path)", info, programs, params, setup, checkin, results);

      %***********************************;
      %* Retrieve the results            *;
      %***********************************;
      results.callIntMethod("getReturnCode", returnCode);
      results.callStringMethod("getReturnMessage", message);

      %***********************************;
      %* Process the results             *;
      %***********************************;
      logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
      if (returnCode ne 0) 
      then
      do;
        logMsgPrimary="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
        logMsg=" Could not update job in the workspace: &lsaf_path.";
      end;
      else
      do;
        logMsgPrimary=logMsgPrefix;
        logMsg=" Updated job in workspace using path &lsaf_path.";
      end;

      %***********************************;
      %* Print messages in log           *;
      %***********************************;
      put;
      put logMsgPrimary " " logMsg;
      put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
      put logMsgPrefix " _lsafMsg_= " message;
      put logMsgPrefix " _lsafRC_= " returnCode;

      %if (&haveData NE 1) %then
      %do;
        put logMsgPrefix " No job data was provided.";
      %end;
      %else
      %do;
        %if (&sas_dsname_info NE _lsaf_updateworkspacejob_ds_notprovided_) %then
        %do;
          put logMsgPrefix " sas_dsname_info = &sas_dsname_info";
        %end;
        %if (&sas_dsname_programs NE _lsaf_updateworkspacejob_ds_notprovided_) %then
        %do;
          put logMsgPrefix " sas_dsname_programs = &sas_dsname_programs";
        %end;
        %if (&sas_dsname_parameters NE _lsaf_updateworkspacejob_ds_notprovided_) %then
        %do;
          put logMsgPrefix " sas_dsname_parameters = &sas_dsname_parameters";
        %end;
        %if (&sas_dsname_setup NE _lsaf_updateworkspacejob_ds_notprovided_) %then
        %do;
          put logMsgPrefix " sas_dsname_setup = &sas_dsname_setup";
        %end;
        %if (&sas_dsname_checkin NE _lsaf_updateworkspacejob_ds_notprovided_) %then
        %do;
          put logMsgPrefix " sas_dsname_checkin = &sas_dsname_checkin";
        %end;
      %end;

      %***********************************;
      %* Set the macro variables         *;
      %***********************************;
      call symputx("_lsafRC_", returnCode);
      call symputx("_lsafMsg_", message);

   run;

   %if (%sysfunc(exist(&_lsafMASTERJobDATA_tempDS_)) > 0) %then
   %do;
     proc datasets library=work nolist;
        delete &_lsafMASTERJobDATA_tempDS_;
     quit;
     run;
   %end;

   options &notesOption;

   %return;

   %PRINT_DSERROR_LOG_MSGS:
      %put &logMsgPrimary  &macroName ;
      %put &logMsgPrefix _lsafRC_= &_lsafRC_ ;
      %put &logMsgPrefix _lsafMsg_= &_lsafMsg_ ;
      %put;

   options &notesOption;

%mend lsaf_updateworkspacejob;
